@@ -1,8 +1,8 @@
 """Beamformers for source localization."""
 
 from ._lcmv import (make_lcmv, apply_lcmv, apply_lcmv_epochs, apply_lcmv_raw,
-                    tf_lcmv, apply_lcmv_cov)
-from ._dics import (make_dics, apply_dics, apply_dics_epochs, apply_dics_csd,
-                    tf_dics)
+                    apply_lcmv_cov)
+from ._dics import make_dics, apply_dics, apply_dics_epochs, apply_dics_csd
 from ._rap_music import rap_music
 from ._compute_beamformer import Beamformer, read_beamformer
+from .resolution_matrix import make_lcmv_resolution_matrix
