@@ -1,12 +1,13 @@
 # Author: Daniel G Wakeman <dwakeman@nmr.mgh.harvard.edu>
 #         Denis A. Engemann <denis.engemann@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import os.path as op
-
+from pathlib import Path
 from copy import deepcopy
 from functools import partial
+import hashlib
 
 import pytest
 import numpy as np
@@ -15,16 +16,20 @@ from numpy.testing import assert_array_equal, assert_equal, assert_allclose
 
 from mne.channels import (rename_channels, read_ch_adjacency, combine_channels,
                           find_ch_adjacency, make_1020_channel_selections,
-                          read_custom_montage, equalize_channels)
-from mne.channels.channels import (_ch_neighbor_adjacency,
-                                   _compute_ch_adjacency)
+                          read_custom_montage, equalize_channels,
+                          get_builtin_ch_adjacencies)
+from mne.channels.channels import (
+    _ch_neighbor_adjacency, _compute_ch_adjacency,
+    _BUILTIN_CHANNEL_ADJACENCIES, _BuiltinChannelAdjacency
+)
 from mne.io import (read_info, read_raw_fif, read_raw_ctf, read_raw_bti,
                     read_raw_eeglab, read_raw_kit, RawArray)
 from mne.io.constants import FIFF
-from mne.utils import _TempDir, run_tests_if_main
 from mne import (pick_types, pick_channels, EpochsArray, EvokedArray,
                  make_ad_hoc_cov, create_info, read_events, Epochs)
 from mne.datasets import testing
+from mne.utils import requires_pandas, requires_version
+from mne.parallel import parallel_func
 
 io_dir = op.join(op.dirname(__file__), '..', '..', 'io')
 base_dir = op.join(io_dir, 'tests', 'data')
@@ -32,6 +37,8 @@ raw_fname = op.join(base_dir, 'test_raw.fif')
 eve_fname = op.join(base_dir, 'test-eve.fif')
 fname_kit_157 = op.join(io_dir, 'kit', 'tests', 'data', 'test.sqd')
 
+testing_path = testing.data_path(download=False)
+
 
 @pytest.mark.parametrize('preload', (True, False))
 @pytest.mark.parametrize('proj', (True, False))
@@ -178,9 +185,21 @@ def test_set_channel_types():
     pytest.raises(ValueError, raw.set_channel_types, ch_types)
 
 
-def test_read_ch_adjacency():
+def test_get_builtin_ch_adjacencies():
+    """Test retrieving the names of all built-in FieldTrip neighbors."""
+    names = get_builtin_ch_adjacencies()
+    assert names
+    assert len(names) == len(set(names))  # no duplicates
+    assert len(names) == len(_BUILTIN_CHANNEL_ADJACENCIES)
+
+    names_and_descriptions = get_builtin_ch_adjacencies(descriptions=True)
+    for name_and_description in names_and_descriptions:
+        assert len(name_and_description) == 2
+
+
+def test_read_ch_adjacency(tmp_path):
     """Test reading channel adjacency templates."""
-    tempdir = _TempDir()
+    tempdir = str(tmp_path)
     a = partial(np.array, dtype='<U7')
     # no pep8
     nbh = np.array([[(['MEG0111'], [[a(['MEG0131'])]]),
@@ -255,6 +274,73 @@ def test_read_ch_adjacency():
     savemat(mat_fname, mat, oned_as='row')
     pytest.raises(ValueError, read_ch_adjacency, mat_fname)
 
+    # Try reading all built-in FieldTrip neighbors
+    for name in get_builtin_ch_adjacencies():
+        ch_adjacency, ch_names = read_ch_adjacency(name)
+        assert_equal(ch_adjacency.shape[0], len(ch_names))
+
+
+def _download_ft_neighbors(target_dir):
+    """Download the known neighbors from FieldTrip."""
+    # The entire FT repository is larger than a GB, so we'll just download
+    # the few files we need.
+    def _download_one_ft_neighbor(
+        neighbor: _BuiltinChannelAdjacency
+    ):
+        # Log level setting must happen inside the job to work properly
+        import pooch
+        pooch.get_logger().setLevel('ERROR')  # reduce verbosity
+        fname = neighbor.fname
+        url = neighbor.source_url
+
+        pooch.retrieve(
+            url=url,
+            known_hash=None,
+            fname=fname,
+            path=target_dir,
+        )
+
+    parallel, p_fun, _ = parallel_func(
+        func=_download_one_ft_neighbor, n_jobs=-1
+    )
+    parallel(
+        p_fun(neighbor)
+        for neighbor in _BUILTIN_CHANNEL_ADJACENCIES
+        if neighbor.source_url is not None
+    )
+
+
+@pytest.mark.slowtest
+def test_adjacency_matches_ft(tmp_path):
+    """Test correspondence of built-in adjacency matrices with FT repo."""
+    builtin_neighbors_dir = Path(__file__).parents[1] / 'data' / 'neighbors'
+    ft_neighbors_dir = tmp_path
+    del tmp_path
+
+    _download_ft_neighbors(target_dir=ft_neighbors_dir)
+
+    for adj in _BUILTIN_CHANNEL_ADJACENCIES:
+        fname = adj.fname
+        if not (ft_neighbors_dir / fname).exists():
+            continue  # only exists in MNE, not FT
+
+        hash_mne = hashlib.sha256()
+        hash_ft = hashlib.sha256()
+
+        with open(builtin_neighbors_dir / fname, 'rb') as f:
+            data = f.read()
+            hash_mne.update(data)
+
+        with open(ft_neighbors_dir / fname, 'rb') as f:
+            data = f.read()
+            hash_ft.update(data)
+
+        if hash_mne.hexdigest() != hash_ft.hexdigest():
+            raise ValueError(
+                f'Hash mismatch between built-in and FieldTrip neighbors '
+                f'for {fname}'
+            )
+
 
 def test_get_set_sensor_positions():
     """Test get/set functions for sensor positions."""
@@ -273,12 +359,12 @@ def test_get_set_sensor_positions():
                        raw2.info['chs'][13]['loc'])
 
 
+@requires_version('pymatreader')
 @testing.requires_testing_data
 def test_1020_selection():
     """Test making a 10/20 selection dict."""
-    base_dir = op.join(testing.data_path(download=False), 'EEGLAB')
-    raw_fname = op.join(base_dir, 'test_raw.set')
-    loc_fname = op.join(base_dir, 'test_chans.locs')
+    raw_fname = op.join(testing_path, 'EEGLAB', 'test_raw.set')
+    loc_fname = op.join(testing_path, 'EEGLAB', 'test_chans.locs')
     raw = read_raw_eeglab(raw_fname, preload=True)
     montage = read_custom_montage(loc_fname)
     raw = raw.rename_channels(dict(zip(raw.ch_names, montage.ch_names)))
@@ -305,8 +391,6 @@ def test_1020_selection():
 @testing.requires_testing_data
 def test_find_ch_adjacency():
     """Test computing the adjacency matrix."""
-    data_path = testing.data_path()
-
     raw = read_raw_fif(raw_fname, preload=True)
     sizes = {'mag': 828, 'grad': 1700, 'eeg': 384}
     nchans = {'mag': 102, 'grad': 204, 'eeg': 60}
@@ -325,13 +409,13 @@ def test_find_ch_adjacency():
     raw.pick_types(meg='mag')
     find_ch_adjacency(raw.info, None)
 
-    bti_fname = op.join(data_path, 'BTi', 'erm_HFH', 'c,rfDC')
-    bti_config_name = op.join(data_path, 'BTi', 'erm_HFH', 'config')
+    bti_fname = op.join(testing_path, 'BTi', 'erm_HFH', 'c,rfDC')
+    bti_config_name = op.join(testing_path, 'BTi', 'erm_HFH', 'config')
     raw = read_raw_bti(bti_fname, bti_config_name, None)
     _, ch_names = find_ch_adjacency(raw.info, 'mag')
     assert 'A1' in ch_names
 
-    ctf_fname = op.join(data_path, 'CTF', 'testdata_ctf_short.ds')
+    ctf_fname = op.join(testing_path, 'CTF', 'testdata_ctf_short.ds')
     raw = read_raw_ctf(ctf_fname)
     _, ch_names = find_ch_adjacency(raw.info, 'mag')
     assert 'MLC11' in ch_names
@@ -347,7 +431,7 @@ def test_find_ch_adjacency():
 @testing.requires_testing_data
 def test_neuromag122_adjacency():
     """Test computing the adjacency matrix of Neuromag122-Data."""
-    nm122_fname = op.join(testing.data_path(), 'misc',
+    nm122_fname = op.join(testing_path, 'misc',
                           'neuromag122_test_file-raw.fif')
     raw = read_raw_fif(nm122_fname, preload=True)
     conn, ch_names = find_ch_adjacency(raw.info, 'grad')
@@ -366,6 +450,26 @@ def test_drop_channels():
     pytest.raises(ValueError, raw.drop_channels, 5)  # must be list or str
 
 
+def test_pick_channels():
+    """Test if picking channels works with various arguments."""
+    raw = read_raw_fif(raw_fname, preload=True).crop(0, 0.1)
+
+    # selected correctly 3 channels
+    raw.pick(['MEG 0113', 'MEG 0112', 'MEG 0111'])
+    assert len(raw.ch_names) == 3
+
+    # selected correctly 3 channels and ignored 'meg', and emit warning
+    with pytest.warns(RuntimeWarning, match='not present in the info'):
+        raw.pick(['MEG 0113', "meg", 'MEG 0112', 'MEG 0111'])
+        assert len(raw.ch_names) == 3
+
+    names_len = len(raw.ch_names)
+    raw.pick(['all'])  # selected correctly all channels
+    assert len(raw.ch_names) == names_len
+    raw.pick('all')  # selected correctly all channels
+    assert len(raw.ch_names) == names_len
+
+
 def test_add_reference_channels():
     """Test if there is a new reference channel that consist of all zeros."""
     raw = read_raw_fif(raw_fname, preload=True)
@@ -449,7 +553,9 @@ def test_combine_channels():
     combine_channels(raw, good)
     combined_epochs = combine_channels(epochs, good)
     assert_array_equal(combined_epochs.events, epochs.events)
-    combine_channels(evoked, good)
+    assert epochs.baseline == combined_epochs.baseline
+    combined_evoked = combine_channels(evoked, good)
+    assert evoked.baseline == combined_evoked.baseline
     combine_channels(raw, good, drop_bad=True)
     combine_channels(raw_ch_bad, good, drop_bad=True)
 
@@ -502,4 +608,18 @@ def test_combine_channels():
     assert len(record) == 3
 
 
-run_tests_if_main()
+@requires_pandas
+def test_combine_channels_metadata():
+    """Test if metadata is correctly retained in combined object."""
+    import pandas as pd
+
+    raw = read_raw_fif(raw_fname, preload=True)
+    epochs = Epochs(raw, read_events(eve_fname), preload=True)
+
+    metadata = pd.DataFrame({"A": np.arange(len(epochs)),
+                             "B": np.ones(len(epochs))})
+    epochs.metadata = metadata
+
+    good = dict(foo=[0, 1, 3, 4], bar=[5, 2])  # good grad and mag
+    combined_epochs = combine_channels(epochs, good)
+    pd.testing.assert_frame_equal(epochs.metadata, combined_epochs.metadata)
