@@ -17,7 +17,7 @@
 
 # Authors: Eric Larson <larson.eric.d@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 from functools import partial
 
@@ -30,7 +30,7 @@ from .io.kit.constants import KIT
 from .io.kit.kit import RawKIT as _RawKIT
 from .io.meas_info import _simplify_info, Info
 from .io.pick import (pick_types, pick_channels, pick_channels_regexp,
-                      pick_info)
+                      pick_info, _picks_to_idx)
 from .io.proj import Projection, setup_proj
 from .io.constants import FIFF
 from .io.ctf.trans import _make_ctf_coord_trans_set
@@ -45,7 +45,8 @@ from .transforms import (apply_trans, invert_transform, _angle_between_quats,
                          quat_to_rot, rot_to_quat, _fit_matched_points,
                          _quat_to_affine, als_ras_trans)
 from .utils import (verbose, logger, use_log_level, _check_fname, warn,
-                    _validate_type, ProgressBar, _check_option, _pl)
+                    _validate_type, ProgressBar, _check_option, _pl,
+                    _on_missing)
 
 # Eventually we should add:
 #   hpicons
@@ -295,16 +296,42 @@ def extract_chpi_locs_kit(raw, stim_channel='MISC 064', *, verbose=None):
 
 # ############################################################################
 # Estimate positions from data
+
 @verbose
-def _get_hpi_info(info, allow_empty=False, verbose=None):
-    """Get HPI information from raw."""
+def get_chpi_info(info, on_missing='raise', verbose=None):
+    """Retrieve cHPI information from the data.
+
+    Parameters
+    ----------
+    %(info_not_none)s
+    %(on_missing_chpi)s
+    %(verbose)s
+
+    Returns
+    -------
+    hpi_freqs : array, shape (n_coils,)
+        The frequency used for each individual cHPI coil.
+    hpi_pick : int | None
+        The index of the ``STIM`` channel containing information about when
+        which cHPI coils were switched on.
+    hpi_on : array, shape (n_coils,)
+        The values coding for the "on" state of each individual cHPI coil.
+
+    Notes
+    -----
+    .. versionadded:: 0.24
+    """
+    _validate_type(item=info, item_name='info', types=Info)
+    _check_option(parameter='on_missing', value=on_missing,
+                  allowed_values=['ignore', 'raise', 'warn'])
+
     if len(info['hpi_meas']) == 0 or \
             ('coil_freq' not in info['hpi_meas'][0]['hpi_coils'][0]):
-        if allow_empty:
-            return np.empty(0), None, np.empty(0)
-        raise RuntimeError('Appropriate cHPI information not found in'
-                           'info["hpi_meas"] and info["hpi_subsystem"], '
-                           'cannot process cHPI')
+        _on_missing(on_missing,
+                    msg='No appropriate cHPI information found in '
+                        'info["hpi_meas"] and info["hpi_subsystem"]')
+        return np.empty(0), None, np.empty(0)
+
     hpi_coils = sorted(info['hpi_meas'][-1]['hpi_coils'],
                        key=lambda x: x['number'])  # ascending (info) order
 
@@ -525,7 +552,9 @@ def _setup_hpi_amplitude_fitting(info, t_window, remove_aliased=False,
                                  ext_order=1, allow_empty=False, verbose=None):
     """Generate HPI structure for HPI localization."""
     # grab basic info.
-    hpi_freqs, hpi_pick, hpi_ons = _get_hpi_info(info, allow_empty=allow_empty)
+    on_missing = 'raise' if not allow_empty else 'ignore'
+    hpi_freqs, hpi_pick, hpi_ons = get_chpi_info(info, on_missing=on_missing)
+
     _validate_type(t_window, (str, 'numeric'), 't_window')
     if info['line_freq'] is not None:
         line_freqs = np.arange(info['line_freq'], info['sfreq'] / 3.,
@@ -560,30 +589,35 @@ def _setup_hpi_amplitude_fitting(info, t_window, remove_aliased=False,
     if t_window <= 0:
         raise ValueError('t_window (%s) must be > 0' % (t_window,))
     logger.info('Using time window: %0.1f ms' % (1000 * t_window,))
-    model_n_window = int(round(float(t_window) * info['sfreq']))
-    # build model to extract sinusoidal amplitudes.
-    slope = np.linspace(-0.5, 0.5, model_n_window)[:, np.newaxis]
-    rps = np.arange(model_n_window)[:, np.newaxis].astype(float)
-    rps *= 2 * np.pi / info['sfreq']  # radians/sec
-    f_t = hpi_freqs[np.newaxis, :] * rps
-    l_t = line_freqs[np.newaxis, :] * rps
-    model = [np.sin(f_t), np.cos(f_t)]  # hpi freqs
-    model += [np.sin(l_t), np.cos(l_t)]  # line freqs
-    model += [slope, np.ones(slope.shape)]
-    model = np.concatenate(model, axis=1)
+    window_nsamp = np.rint(t_window * info['sfreq']).astype(int)
+    model = _setup_hpi_glm(hpi_freqs, line_freqs, info['sfreq'], window_nsamp)
     inv_model = np.linalg.pinv(model)
     inv_model_reord = _reorder_inv_model(inv_model, len(hpi_freqs))
     proj, proj_op, meg_picks = _setup_ext_proj(info, ext_order)
-
+    # include mag and grad picks separately, for SNR computations
+    mag_picks = _picks_to_idx(info, 'mag', allow_empty=True)
+    grad_picks = _picks_to_idx(info, 'grad', allow_empty=True)
     # Set up magnetic dipole fits
-    hpi = dict(meg_picks=meg_picks, hpi_pick=hpi_pick,
-               model=model, inv_model=inv_model, t_window=t_window,
-               inv_model_reord=inv_model_reord,
-               on=hpi_ons, n_window=model_n_window, proj=proj, proj_op=proj_op,
-               freqs=hpi_freqs, line_freqs=line_freqs)
+    hpi = dict(
+        meg_picks=meg_picks, mag_picks=mag_picks, grad_picks=grad_picks,
+        hpi_pick=hpi_pick, model=model, inv_model=inv_model, t_window=t_window,
+        inv_model_reord=inv_model_reord, on=hpi_ons, n_window=window_nsamp,
+        proj=proj, proj_op=proj_op, freqs=hpi_freqs, line_freqs=line_freqs)
     return hpi
 
 
+def _setup_hpi_glm(hpi_freqs, line_freqs, sfreq, window_nsamp):
+    """Initialize a general linear model for HPI amplitude estimation."""
+    slope = np.linspace(-0.5, 0.5, window_nsamp)[:, np.newaxis]
+    radians_per_sec = 2 * np.pi * np.arange(window_nsamp, dtype=float) / sfreq
+    f_t = hpi_freqs[np.newaxis, :] * radians_per_sec[:, np.newaxis]
+    l_t = line_freqs[np.newaxis, :] * radians_per_sec[:, np.newaxis]
+    model = [np.sin(f_t), np.cos(f_t),    # hpi freqs
+             np.sin(l_t), np.cos(l_t),    # line freqs
+             slope, np.ones_like(slope)]  # drift, DC
+    return np.hstack(model)
+
+
 @jit()
 def _reorder_inv_model(inv_model, n_freqs):
     # Reorder for faster computation
@@ -610,7 +644,8 @@ def _setup_ext_proj(info, ext_order):
         kind=FIFF.FIFFV_PROJ_ITEM_HOMOG_FIELD, desc='SSS', active=False,
         data=dict(data=ext, ncol=info['nchan'], col_names=info['ch_names'],
                   nrow=len(ext)))
-    info['projs'] = [proj]
+    with info._unlock():
+        info['projs'] = [proj]
     proj_op, _ = setup_proj(
         info, add_eeg_ref=False, activate=False, verbose=False)
     assert proj_op.shape == (len(meg_picks),) * 2
@@ -622,14 +657,16 @@ def _time_prefix(fit_time):
     return ('    t=%0.3f:' % fit_time).ljust(17)
 
 
-def _fit_chpi_amplitudes(raw, time_sl, hpi):
+def _fit_chpi_amplitudes(raw, time_sl, hpi, snr=False):
     """Fit amplitudes for each channel from each of the N cHPI sinusoids.
 
     Returns
     -------
-    sin_fit : ndarray, shape (n_freqs, n_channels))
+    sin_fit : ndarray, shape (n_freqs, n_channels)
         The sin amplitudes matching each cHPI frequency.
         Will be all nan if this time window should be skipped.
+    snr : ndarray, shape (n_freqs, 2)
+        Estimated SNR for this window, separately for mag and grad channels.
     """
     # No need to detrend the data because our model has a DC term
     with use_log_level(False):
@@ -647,6 +684,10 @@ def _fit_chpi_amplitudes(raw, time_sl, hpi):
         n_on = ons.all(axis=-1).sum(axis=0)
         if not (n_on >= 3).all():
             return None
+    if snr:
+        return _fast_fit_snr(
+            this_data, len(hpi['freqs']), hpi['model'], hpi['inv_model'],
+            hpi['mag_picks'], hpi['grad_picks'])
     return _fast_fit(this_data, hpi['proj_op'], len(hpi['freqs']),
                      hpi['model'], hpi['inv_model_reord'])
 
@@ -657,8 +698,8 @@ def _fast_fit(this_data, proj, n_freqs, model, inv_model_reord):
     if this_data.shape[1] != model.shape[0]:
         model = model[:this_data.shape[1]]
         inv_model_reord = _reorder_inv_model(np.linalg.pinv(model), n_freqs)
-    proj_data = np.dot(proj, this_data)
-    X = np.dot(inv_model_reord, proj_data.T)
+    proj_data = proj @ this_data
+    X = inv_model_reord @ proj_data.T
 
     sin_fit = np.zeros((n_freqs, X.shape[1]))
     for fi in range(n_freqs):
@@ -670,6 +711,34 @@ def _fast_fit(this_data, proj, n_freqs, model, inv_model_reord):
     return sin_fit
 
 
+@jit()
+def _fast_fit_snr(this_data, n_freqs, model, inv_model, mag_picks, grad_picks):
+    # first or last window
+    if this_data.shape[1] != model.shape[0]:
+        model = model[:this_data.shape[1]]
+        inv_model = np.linalg.pinv(model)
+    coefs = np.ascontiguousarray(inv_model) @ np.ascontiguousarray(this_data.T)
+    # average sin & cos terms (special property of sinusoids: power=A²/2)
+    hpi_power = (coefs[:n_freqs] ** 2 + coefs[n_freqs:(2 * n_freqs)] ** 2) / 2
+    resid = this_data - np.ascontiguousarray((model @ coefs).T)
+    # can't use np.var(..., axis=1) with Numba, so do it manually:
+    resid_mean = np.atleast_2d(resid.sum(axis=1) / resid.shape[1]).T
+    squared_devs = np.abs(resid - resid_mean) ** 2
+    resid_var = squared_devs.sum(axis=1) / squared_devs.shape[1]
+    # output array will be (n_freqs, 3 * n_ch_types). The 3 columns for each
+    # channel type are the SNR, the mean cHPI power and the residual variance
+    # (which gets tiled to shape (n_freqs,) because it's a scalar).
+    snrs = np.empty((n_freqs, 0))
+    # average power & compute residual variance separately for each ch type
+    for _picks in (mag_picks, grad_picks):
+        if len(_picks):
+            avg_power = hpi_power[:, _picks].sum(axis=1) / len(_picks)
+            avg_resid = resid_var[_picks].mean() * np.ones(n_freqs)
+            snr = 10 * np.log10(avg_power / avg_resid)
+            snrs = np.hstack((snrs, np.stack((snr, avg_power, avg_resid), 1)))
+    return snrs
+
+
 def _check_chpi_param(chpi_, name):
     if name == 'chpi_locs':
         want_ndims = dict(times=1, rrs=3, moments=3, gofs=2)
@@ -728,8 +797,7 @@ def compute_head_pos(info, chpi_locs, dist_limit=0.005, gof_limit=0.98,
 
     Parameters
     ----------
-    info : instance of Info
-        Measurement information.
+    %(info_not_none)s
     %(chpi_locs)s
         Typically obtained by :func:`~mne.chpi.compute_chpi_locs` or
         :func:`~mne.chpi.extract_chpi_locs_ctf`.
@@ -737,7 +805,7 @@ def compute_head_pos(info, chpi_locs, dist_limit=0.005, gof_limit=0.98,
         Minimum distance (m) to accept for coil position fitting.
     gof_limit : float
         Minimum goodness of fit to accept for each coil.
-    %(chpi_adjust_dig)s
+    %(adjust_dig_chpi)s
     %(verbose)s
 
     Returns
@@ -885,6 +953,43 @@ def _unit_quat_constraint(x):
     return 1 - (x * x).sum()
 
 
+@verbose
+def compute_chpi_snr(raw, t_step_min=0.01, t_window='auto', ext_order=1,
+                     tmin=0, tmax=None, verbose=None):
+    """Compute time-varying estimates of cHPI SNR.
+
+    Parameters
+    ----------
+    raw : instance of Raw
+        Raw data with cHPI information.
+    t_step_min : float
+        Minimum time step to use.
+    %(t_window_chpi_t)s
+    %(ext_order_chpi)s
+    %(tmin_raw)s
+    %(tmax_raw)s
+    %(verbose)s
+
+    Returns
+    -------
+    chpi_snrs : dict
+        The time-varying cHPI SNR estimates, with entries "times", "freqs",
+        "snr_mag", "power_mag", and "resid_mag" (and/or "snr_grad",
+        "power_grad", and "resid_grad", depending on which channel types are
+        present in ``raw``).
+
+    See Also
+    --------
+    mne.chpi.compute_chpi_locs, mne.chpi.compute_chpi_amplitudes
+
+    Notes
+    -----
+    .. versionadded:: 0.24
+    """
+    return _compute_chpi_amp_or_snr(raw, t_step_min, t_window, ext_order,
+                                    tmin, tmax, verbose, snr=True)
+
+
 @verbose
 def compute_chpi_amplitudes(raw, t_step_min=0.01, t_window='auto',
                             ext_order=1, tmin=0, tmax=None, verbose=None):
@@ -895,12 +1000,11 @@ def compute_chpi_amplitudes(raw, t_step_min=0.01, t_window='auto',
     raw : instance of Raw
         Raw data with cHPI information.
     t_step_min : float
-        Minimum time step to use. If correlations are sufficiently high,
-        t_step_max will be used.
-    %(chpi_t_window)s
-    %(chpi_ext_order)s
-    %(raw_tmin)s
-    %(raw_tmax)s
+        Minimum time step to use.
+    %(t_window_chpi_t)s
+    %(ext_order_chpi)s
+    %(tmin_raw)s
+    %(tmax_raw)s
     %(verbose)s
 
     Returns
@@ -909,7 +1013,7 @@ def compute_chpi_amplitudes(raw, t_step_min=0.01, t_window='auto',
 
     See Also
     --------
-    mne.chpi.compute_chpi_locs
+    mne.chpi.compute_chpi_locs, mne.chpi.compute_chpi_snr
 
     Notes
     -----
@@ -934,6 +1038,19 @@ def compute_chpi_amplitudes(raw, t_step_min=0.01, t_window='auto',
 
     .. versionadded:: 0.20
     """
+    return _compute_chpi_amp_or_snr(raw, t_step_min, t_window, ext_order,
+                                    tmin, tmax, verbose)
+
+
+def _compute_chpi_amp_or_snr(raw, t_step_min=0.01, t_window='auto',
+                             ext_order=1, tmin=0, tmax=None, verbose=None,
+                             snr=False):
+    """Compute cHPI amplitude or SNR.
+
+    See compute_chpi_amplitudes for parameter descriptions. One additional
+    boolean parameter ``snr`` signals whether to return SNR instead of
+    amplitude.
+    """
     hpi = _setup_hpi_amplitude_fitting(raw.info, t_window, ext_order=ext_order)
     tmin, tmax = raw._tmin_tmax_to_start_stop(tmin, tmax)
     tmin = tmin / raw.info['sfreq']
@@ -946,14 +1063,26 @@ def compute_chpi_amplitudes(raw, t_step_min=0.01, t_window='auto',
                 % (len(hpi['freqs']), len(fit_idxs), tmax - tmin))
     del tmin, tmax
     sin_fits = dict()
+    sin_fits['proj'] = hpi['proj']
     sin_fits['times'] = np.round(fit_idxs + raw.first_samp -
                                  hpi['n_window'] / 2.) / raw.info['sfreq']
-    sin_fits['proj'] = hpi['proj']
-    sin_fits['slopes'] = np.empty(
-        (len(sin_fits['times']),
-         len(hpi['freqs']),
-         len(sin_fits['proj']['data']['col_names'])))
-    for mi, midpt in enumerate(ProgressBar(fit_idxs, mesg='cHPI amplitudes')):
+    n_times = len(sin_fits['times'])
+    n_freqs = len(hpi['freqs'])
+    n_chans = len(sin_fits['proj']['data']['col_names'])
+    if snr:
+        del sin_fits['proj']
+        sin_fits['freqs'] = hpi['freqs']
+        ch_types = raw.get_channel_types()
+        grad_offset = 3 if 'mag' in ch_types else 0
+        for ch_type in ('mag', 'grad'):
+            if ch_type in ch_types:
+                for key in ('snr', 'power', 'resid'):
+                    cols = 1 if key == 'resid' else n_freqs
+                    sin_fits[f'{ch_type}_{key}'] = np.empty((n_times, cols))
+    else:
+        sin_fits['slopes'] = np.empty((n_times, n_freqs, n_chans))
+    message = f"cHPI {'SNRs' if snr else 'amplitudes'}"
+    for mi, midpt in enumerate(ProgressBar(fit_idxs, mesg=message)):
         #
         # 0. determine samples to fit.
         #
@@ -964,7 +1093,24 @@ def compute_chpi_amplitudes(raw, t_step_min=0.01, t_window='auto',
         #
         # 1. Fit amplitudes for each channel from each of the N sinusoids
         #
-        sin_fits['slopes'][mi] = _fit_chpi_amplitudes(raw, time_sl, hpi)
+        amps_or_snrs = _fit_chpi_amplitudes(raw, time_sl, hpi, snr)
+        if snr:
+            # unpack the SNR estimates. mag & grad are returned in one array
+            # (because of Numba) so take care with which column is which.
+            # note that mean residual is a scalar (same for all HPI freqs) but
+            # is returned as a (tiled) vector (again, because Numba) so that's
+            # why below we take amps_or_snrs[0, 2] instead of [:, 2]
+            ch_types = raw.get_channel_types()
+            if 'mag' in ch_types:
+                sin_fits['mag_snr'][mi] = amps_or_snrs[:, 0]    # SNR
+                sin_fits['mag_power'][mi] = amps_or_snrs[:, 1]  # mean power
+                sin_fits['mag_resid'][mi] = amps_or_snrs[0, 2]  # mean resid
+            if 'grad' in ch_types:
+                sin_fits['grad_snr'][mi] = amps_or_snrs[:, grad_offset]
+                sin_fits['grad_power'][mi] = amps_or_snrs[:, grad_offset + 1]
+                sin_fits['grad_resid'][mi] = amps_or_snrs[0, grad_offset + 2]
+        else:
+            sin_fits['slopes'][mi] = amps_or_snrs
     return sin_fits
 
 
@@ -975,8 +1121,7 @@ def compute_chpi_locs(info, chpi_amplitudes, t_step_max=1., too_close='raise',
 
     Parameters
     ----------
-    info : instance of Info
-        The measurement information.
+    %(info_not_none)s
     %(chpi_amplitudes)s
         Typically obtained by :func:`mne.chpi.compute_chpi_amplitudes`.
     t_step_max : float
@@ -984,7 +1129,7 @@ def compute_chpi_locs(info, chpi_amplitudes, t_step_max=1., too_close='raise',
     too_close : str
         How to handle HPI positions too close to the sensors,
         can be 'raise' (default), 'warning', or 'info'.
-    %(chpi_adjust_dig)s
+    %(adjust_dig_chpi)s
     %(verbose)s
 
     Returns
@@ -1025,7 +1170,8 @@ def compute_chpi_locs(info, chpi_amplitudes, t_step_max=1., too_close='raise',
     meg_picks = pick_channels(
         info['ch_names'], proj['data']['col_names'], ordered=True)
     info = pick_info(info, meg_picks)  # makes a copy
-    info['projs'] = [proj]
+    with info._unlock():
+        info['projs'] = [proj]
     del meg_picks, proj
     meg_coils = _concatenate_coils(_create_meg_coils(info['chs'], 'accurate'))
 
@@ -1129,8 +1275,8 @@ def filter_chpi(raw, include_line=True, t_step=0.01, t_window='auto',
         If True, also filter line noise.
     t_step : float
         Time step to use for estimation, default is 0.01 (10 ms).
-    %(chpi_t_window)s
-    %(chpi_ext_order)s
+    %(t_window_chpi_t)s
+    %(ext_order_chpi)s
     allow_line_only : bool
         If True, allow filtering line noise only. The default is False,
         which only allows the function to run when cHPI information is present.
