@@ -3,13 +3,14 @@
 #          Martin Luessi <mluessi@nmr.mgh.harvard.edu>
 #          Eric Larson <larsoner@uw.edu>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 # The computations in this code were primarily derived from Matti Hämäläinen's
 # C code.
 
 from copy import deepcopy
 from contextlib import contextmanager
+from pathlib import Path
 import os
 import os.path as op
 
@@ -21,13 +22,12 @@ from ..io.pick import _has_kit_refs, pick_types, pick_info
 from ..io.constants import FIFF, FWD
 from ..transforms import (_ensure_trans, transform_surface_to, apply_trans,
                           _get_trans, _print_coord_trans, _coord_frame_name,
-                          Transform)
-from ..utils import logger, verbose, warn, _pl
-from ..parallel import check_n_jobs
+                          Transform, invert_transform)
+from ..utils import logger, verbose, warn, _pl, _validate_type, _check_fname
 from ..source_space import (_ensure_src, _filter_source_spaces,
                             _make_discrete_source_space, _complete_vol_src)
 from ..source_estimate import VolSourceEstimate
-from ..surface import _normalize_vectors
+from ..surface import _normalize_vectors, _CheckInside
 from ..bem import read_bem_solution, _bem_find_surface, ConductorModel
 
 from .forward import Forward, _merge_meg_eeg_fwds, convert_forward_solution
@@ -102,10 +102,13 @@ def _read_coil_def_file(fname, use_registry=True):
             for p in range(npts):
                 # get next non-comment line
                 line = lines.pop()
-                while(line[0] == '#'):
+                while line[0] == '#':
                     line = lines.pop()
                 vals = np.fromstring(line, sep=' ')
-                assert len(vals) == 7
+                if len(vals) != 7:
+                    raise RuntimeError(
+                        f'Could not interpret line {p + 1} as 7 points:\n'
+                        f'{line}')
                 # Read and verify data for each integration point
                 w.append(vals[0])
                 rmag.append(vals[[1, 2, 3]])
@@ -230,12 +233,11 @@ def _setup_bem(bem, bem_extra, neeg, mri_head_t, allow_none=False,
     if allow_none and bem is None:
         return None
     logger.info('')
-    if isinstance(bem, str):
+    _validate_type(bem, ('path-like', ConductorModel), bem)
+    if not isinstance(bem, ConductorModel):
         logger.info('Setting up the BEM model using %s...\n' % bem_extra)
         bem = read_bem_solution(bem)
     else:
-        if not isinstance(bem, ConductorModel):
-            raise TypeError('bem must be a string or ConductorModel')
         bem = bem.copy()
     if bem['is_sphere']:
         logger.info('Using the sphere model.\n')
@@ -263,18 +265,16 @@ def _setup_bem(bem, bem_extra, neeg, mri_head_t, allow_none=False,
 
 
 @verbose
-def _prep_meg_channels(info, accurate=True, exclude=(), ignore_ref=False,
+def _prep_meg_channels(info, accuracy='accurate', exclude=(), ignore_ref=False,
                        head_frame=True, do_es=False, do_picking=True,
                        verbose=None):
     """Prepare MEG coil definitions for forward calculation.
 
     Parameters
     ----------
-    info : instance of Info
-        The measurement information dictionary
-    accurate : bool
-        If true (default) then use `accurate` coil definitions (more
-        integration points)
+    %(info_not_none)s
+    accuracy : str
+        Can be "normal" or "accurate" (default).
     exclude : list of str | str
         List of channels to exclude. If 'bads', exclude channels in
         info['bads']
@@ -299,7 +299,6 @@ def _prep_meg_channels(info, accurate=True, exclude=(), ignore_ref=False,
     meginfo : instance of Info
         Information subselected for just the set of MEG coils
     """
-    accuracy = 'accurate' if accurate else 'normal'
     info_extra = 'info'
     megnames, megcoils, compcoils = [], [], []
 
@@ -378,8 +377,7 @@ def _prep_eeg_channels(info, exclude=(), verbose=None):
 
     Parameters
     ----------
-    info : instance of Info
-        The measurement information dictionary
+    %(info_not_none)s
     exclude : list of str | str
         List of channels to exclude. If 'bads', exclude channels in
         info['bads']
@@ -446,8 +444,10 @@ def _prepare_for_forward(src, mri_head_t, info, bem, mindist, n_jobs,
     cmd = 'make_forward_solution(%s)' % (', '.join([str(a) for a in arg_list]))
     mri_id = dict(machid=np.zeros(2, np.int32), version=0, secs=0, usecs=0)
 
+    info_trans = str(trans) if isinstance(trans, Path) else trans
     info = Info(chs=info['chs'], comps=info['comps'],
-                dev_head_t=info['dev_head_t'], mri_file=trans, mri_id=mri_id,
+                dev_head_t=info['dev_head_t'], mri_file=info_trans,
+                mri_id=mri_id,
                 meas_file=info_extra, meas_id=None, working_dir=os.getcwd(),
                 command_line=cmd, bads=info['bads'], mri_head_t=mri_head_t)
     info._update_redundant()
@@ -483,11 +483,38 @@ def _prepare_for_forward(src, mri_head_t, info, bem, mindist, n_jobs,
     bem = _setup_bem(bem, bem_extra, len(eegnames), mri_head_t,
                      allow_none=allow_bem_none)
 
-    # Circumvent numerical problems by excluding points too close to the skull
-    if bem is not None and not bem['is_sphere']:
-        inner_skull = _bem_find_surface(bem, 'inner_skull')
-        _filter_source_spaces(inner_skull, mindist, mri_head_t, src, n_jobs)
-        logger.info('')
+    # Circumvent numerical problems by excluding points too close to the skull,
+    # and check that sensors are not inside any BEM surface
+    if bem is not None:
+        if not bem['is_sphere']:
+            check_surface = 'inner skull surface'
+            inner_skull = _bem_find_surface(bem, 'inner_skull')
+            check_inside = _filter_source_spaces(
+                inner_skull, mindist, mri_head_t, src, n_jobs)
+            logger.info('')
+            if len(bem['surfs']) == 3:
+                check_surface = 'scalp surface'
+                check_inside = _CheckInside(
+                    _bem_find_surface(bem, 'head'))
+        else:
+            check_surface = 'outermost sphere shell'
+            if len(bem['layers']) == 0:
+                def check_inside(x):
+                    return np.zeros(len(x), bool)
+            else:
+                def check_inside(x):
+                    return (np.linalg.norm(x - bem['r0'], axis=1) <
+                            bem['layers'][-1]['rad'])
+        if len(megcoils):
+            meg_loc = apply_trans(
+                invert_transform(mri_head_t),
+                np.array([coil['r0'] for coil in megcoils]))
+            n_inside = check_inside(meg_loc).sum()
+            if n_inside:
+                raise RuntimeError(
+                    f'Found {n_inside} MEG sensor{_pl(n_inside)} inside the '
+                    f'{check_surface}, perhaps coordinate frames and/or '
+                    'coregistration must be incorrect')
 
     rr = np.concatenate([s['rr'][s['vertno']] for s in src])
     if len(rr) < 1:
@@ -505,21 +532,18 @@ def _prepare_for_forward(src, mri_head_t, info, bem, mindist, n_jobs,
 
 @verbose
 def make_forward_solution(info, trans, src, bem, meg=True, eeg=True,
-                          mindist=0.0, ignore_ref=False, n_jobs=1,
+                          mindist=0.0, ignore_ref=False, n_jobs=None, *,
                           verbose=None):
     """Calculate a forward solution for a subject.
 
     Parameters
     ----------
-    info : instance of mne.Info | str
-        If str, then it should be a filename to a Raw, Epochs, or Evoked
-        file with measurement information. If dict, should be an info
-        dict (such as one from Raw, Epochs, or Evoked).
+    %(info_str)s
     %(trans)s
-    src : str | instance of SourceSpaces
+    src : path-like | instance of SourceSpaces
         If string, should be a source space filename. Can also be an
         instance of loaded or generated SourceSpaces.
-    bem : dict | str
+    bem : path-like | dict
         Filename of the BEM (e.g., "sample-5120-5120-5120-bem-sol.fif") to
         use, or a loaded sphere model (dict).
     meg : bool
@@ -564,14 +588,14 @@ def make_forward_solution(info, trans, src, bem, meg=True, eeg=True,
         bem_extra = 'instance of ConductorModel'
     else:
         bem_extra = bem
-    if not isinstance(info, (Info, str)):
-        raise TypeError('info should be an instance of Info or string')
-    if isinstance(info, str):
+    _validate_type(info, ('path-like', Info), 'info')
+    if not isinstance(info, Info):
         info_extra = op.split(info)[1]
+        info = _check_fname(info, must_exist=True, overwrite='read',
+                            name='info')
         info = read_info(info, verbose=False)
     else:
         info_extra = 'instance of Info'
-    n_jobs = check_n_jobs(n_jobs)
 
     # Report the setup
     logger.info('Source space          : %s' % src)
@@ -618,7 +642,8 @@ def make_forward_solution(info, trans, src, bem, meg=True, eeg=True,
 
 
 @verbose
-def make_forward_dipole(dipole, bem, info, trans=None, n_jobs=1, verbose=None):
+def make_forward_dipole(dipole, bem, info, trans=None, n_jobs=None, *,
+                        verbose=None):
     """Convert dipole object to source estimate and calculate forward operator.
 
     The instance of Dipole is converted to a discrete source space,
@@ -634,10 +659,7 @@ def make_forward_dipole(dipole, bem, info, trans=None, n_jobs=1, verbose=None):
 
     Parameters
     ----------
-    dipole : instance of Dipole
-        Dipole object containing position, orientation and amplitude of
-        one or more dipoles. Multiple simultaneous dipoles may be defined by
-        assigning them identical times.
+    %(dipole)s
     bem : str | dict
         The BEM filename (str) or a loaded sphere model (dict).
     info : instance of Info
@@ -666,6 +688,10 @@ def make_forward_dipole(dipole, bem, info, trans=None, n_jobs=1, verbose=None):
     -----
     .. versionadded:: 0.12.0
     """
+    if isinstance(dipole, list):
+        from ..dipole import _concatenate_dipoles  # To avoid circular import
+        dipole = _concatenate_dipoles(dipole)
+
     # Make copies to avoid mangling original dipole
     times = dipole.times.copy()
     pos = dipole.pos.copy()
