@@ -9,7 +9,8 @@ from math import sqrt
 import numpy as np
 
 from .mxne_debiasing import compute_bias
-from ..utils import logger, verbose, sum_squared, warn, _get_blas_funcs
+from ..utils import (logger, verbose, sum_squared, warn, _get_blas_funcs,
+                     _validate_type, _check_option)
 from ..time_frequency._stft import stft_norm1, stft_norm2, stft, istft
 
 
@@ -42,135 +43,43 @@ def norm_l21(A, n_orient, copy=True):
     return np.sum(np.sqrt(groups_norm2(A, n_orient)))
 
 
-def prox_l21(Y, alpha, n_orient, shape=None, is_stft=False):
-    """Proximity operator for l21 norm.
+def _primal_l21(M, G, X, active_set, alpha, n_orient):
+    """Primal objective for the mixed-norm inverse problem.
 
-    L2 over columns and L1 over rows => groups contain n_orient rows.
-
-    It can eventually take into account the negative frequencies
-    when a complex value is passed and is_stft=True.
+    See :footcite:`GramfortEtAl2012`.
 
     Parameters
     ----------
-    Y : array, shape (n_sources, n_coefs)
-        The input data.
-    alpha : float
-        The regularization parameter.
-    n_orient : int
-        Number of dipoles per locations (typically 1 or 3).
-    shape : None | tuple
-        Shape of TF coefficients matrix.
-    is_stft : bool
-        If True, Y contains TF coefficients.
-
-    Returns
-    -------
-    Y : array, shape (n_sources, n_coefs)
-        The output data.
-    active_set : array of bool, shape (n_sources, )
-        Mask of active sources
-
-    Examples
-    --------
-    >>> Y = np.tile(np.array([0, 4, 3, 0, 0], dtype=np.float64), (2, 1))
-    >>> Y = np.r_[Y, np.zeros_like(Y)]
-    >>> print(Y)  # doctest:+SKIP
-    [[ 0.  4.  3.  0.  0.]
-     [ 0.  4.  3.  0.  0.]
-     [ 0.  0.  0.  0.  0.]
-     [ 0.  0.  0.  0.  0.]]
-    >>> Yp, active_set = prox_l21(Y, 2, 2)
-    >>> print(Yp)  # doctest:+SKIP
-    [[0.         2.86862915 2.15147186 0.         0.        ]
-     [0.         2.86862915 2.15147186 0.         0.        ]]
-    >>> print(active_set)
-    [ True  True False False]
-    """
-    if len(Y) == 0:
-        return np.zeros_like(Y), np.zeros((0,), dtype=bool)
-    if shape is not None:
-        shape_init = Y.shape
-        Y = Y.reshape(*shape)
-    n_positions = Y.shape[0] // n_orient
-
-    if is_stft:
-        rows_norm = np.sqrt(stft_norm2(Y).reshape(n_positions, -1).sum(axis=1))
-    else:
-        rows_norm = np.sqrt((Y * Y.conj()).real.reshape(n_positions,
-                                                        -1).sum(axis=1))
-    # Ensure shrink is >= 0 while avoiding any division by zero
-    shrink = np.maximum(1.0 - alpha / np.maximum(rows_norm, alpha), 0.0)
-    active_set = shrink > 0.0
-    if n_orient > 1:
-        active_set = np.tile(active_set[:, None], [1, n_orient]).ravel()
-        shrink = np.tile(shrink[:, None], [1, n_orient]).ravel()
-    Y = Y[active_set]
-    if shape is None:
-        Y *= shrink[active_set][:, np.newaxis]
-    else:
-        Y *= shrink[active_set][:, np.newaxis, np.newaxis]
-        Y = Y.reshape(-1, *shape_init[1:])
-    return Y, active_set
-
-
-def prox_l1(Y, alpha, n_orient):
-    """Proximity operator for l1 norm with multiple orientation support.
-
-    Please note that this function computes a soft-thresholding if
-    n_orient == 1 and a block soft-thresholding (L2 over orientation and
-    L1 over position (space + time)) if n_orient == 3. See also
-    :footcite:`GramfortEtAl2013b`.
-
-    Parameters
-    ----------
-    Y : array, shape (n_sources, n_coefs)
-        The input data.
-    alpha : float
-        The regularization parameter.
-    n_orient : int
-        Number of dipoles per locations (typically 1 or 3).
-
-    Returns
-    -------
-    Y : array, shape (n_sources, n_coefs)
-        The output data.
-    active_set : array of bool, shape (n_sources, )
+    M : array, shape (n_sensors, n_times)
+        The data.
+    G : array, shape (n_sensors, n_active)
+        The gain matrix a.k.a. lead field.
+    X : array, shape (n_active, n_times)
+        Sources.
+    active_set : array of bool, shape (n_sources,)
         Mask of active sources.
+    alpha : float
+        The regularization parameter.
+    n_orient : int
+        Number of dipoles per locations (typically 1 or 3).
 
-    References
-    ----------
-    .. footbibliography::
-
-    Examples
-    --------
-    >>> Y = np.tile(np.array([1, 2, 3, 2, 0], dtype=np.float64), (2, 1))
-    >>> Y = np.r_[Y, np.zeros_like(Y)]
-    >>> print(Y)  # doctest:+SKIP
-    [[ 1.  2.  3.  2.  0.]
-     [ 1.  2.  3.  2.  0.]
-     [ 0.  0.  0.  0.  0.]
-     [ 0.  0.  0.  0.  0.]]
-    >>> Yp, active_set = prox_l1(Y, 2, 2)
-    >>> print(Yp)  # doctest:+SKIP
-    [[0.         0.58578644 1.58578644 0.58578644 0.        ]
-     [0.         0.58578644 1.58578644 0.58578644 0.        ]]
-    >>> print(active_set)
-    [ True  True False False]
+    Returns
+    -------
+    p_obj : float
+        Primal objective.
+    R : array, shape (n_sensors, n_times)
+        Current residual (M - G * X).
+    nR2 : float
+        Data-fitting term.
+    GX : array, shape (n_sensors, n_times)
+        Forward prediction.
     """
-    n_positions = Y.shape[0] // n_orient
-    norms = np.sqrt((Y * Y.conj()).real.T.reshape(-1, n_orient).sum(axis=1))
-    # Ensure shrink is >= 0 while avoiding any division by zero
-    shrink = np.maximum(1.0 - alpha / np.maximum(norms, alpha), 0.0)
-    shrink = shrink.reshape(-1, n_positions).T
-    active_set = np.any(shrink > 0.0, axis=1)
-    shrink = shrink[active_set]
-    if n_orient > 1:
-        active_set = np.tile(active_set[:, None], [1, n_orient]).ravel()
-    Y = Y[active_set]
-    if len(Y) > 0:
-        for o in range(n_orient):
-            Y[o::n_orient] *= shrink
-    return Y, active_set
+    GX = np.dot(G[:, active_set], X)
+    R = M - GX
+    penalty = norm_l21(X, n_orient, copy=True)
+    nR2 = sum_squared(R)
+    p_obj = 0.5 * nR2 + alpha * penalty
+    return p_obj, R, nR2, GX
 
 
 def dgap_l21(M, G, X, active_set, alpha, n_orient):
@@ -208,12 +117,7 @@ def dgap_l21(M, G, X, active_set, alpha, n_orient):
     ----------
     .. footbibilography::
     """
-    GX = np.dot(G[:, active_set], X)
-    R = M - GX
-    penalty = norm_l21(X, n_orient, copy=True)
-    nR2 = sum_squared(R)
-    p_obj = 0.5 * nR2 + alpha * penalty
-
+    p_obj, R, nR2, GX = _primal_l21(M, G, X, active_set, alpha, n_orient)
     dual_norm = norm_l2inf(np.dot(G.T, R), n_orient, copy=False)
     scaling = alpha / dual_norm
     scaling = min(scaling, 1.0)
@@ -223,73 +127,6 @@ def dgap_l21(M, G, X, active_set, alpha, n_orient):
     return gap, p_obj, d_obj, R
 
 
-@verbose
-def _mixed_norm_solver_prox(M, G, alpha, lipschitz_constant, maxit=200,
-                            tol=1e-8, verbose=None, init=None, n_orient=1,
-                            dgap_freq=10):
-    """Solve L21 inverse problem with proximal iterations and FISTA."""
-    n_sensors, n_times = M.shape
-    _, n_sources = G.shape
-
-    if n_sources < n_sensors:
-        gram = np.dot(G.T, G)
-        GTM = np.dot(G.T, M)
-    else:
-        gram = None
-
-    if init is None:
-        X = 0.0
-        R = M.copy()
-        if gram is not None:
-            R = np.dot(G.T, R)
-    else:
-        X = init
-        if gram is None:
-            R = M - np.dot(G, X)
-        else:
-            R = GTM - np.dot(gram, X)
-
-    t = 1.0
-    Y = np.zeros((n_sources, n_times))  # FISTA aux variable
-    E = []  # track primal objective function
-    highest_d_obj = - np.inf
-    active_set = np.ones(n_sources, dtype=bool)  # start with full AS
-
-    for i in range(maxit):
-        X0, active_set_0 = X, active_set  # store previous values
-        if gram is None:
-            Y += np.dot(G.T, R) / lipschitz_constant  # ISTA step
-        else:
-            Y += R / lipschitz_constant  # ISTA step
-        X, active_set = prox_l21(Y, alpha / lipschitz_constant, n_orient)
-
-        t0 = t
-        t = 0.5 * (1.0 + sqrt(1.0 + 4.0 * t ** 2))
-        Y.fill(0.0)
-        dt = ((t0 - 1.0) / t)
-        Y[active_set] = (1.0 + dt) * X
-        Y[active_set_0] -= dt * X0
-        Y_as = active_set_0 | active_set
-
-        if gram is None:
-            R = M - np.dot(G[:, Y_as], Y[Y_as])
-        else:
-            R = GTM - np.dot(gram[:, Y_as], Y[Y_as])
-
-        if (i + 1) % dgap_freq == 0:
-            _, p_obj, d_obj, _ = dgap_l21(M, G, X, active_set, alpha,
-                                          n_orient)
-            highest_d_obj = max(d_obj, highest_d_obj)
-            gap = p_obj - highest_d_obj
-            E.append(p_obj)
-            logger.debug("p_obj : %s -- gap : %s" % (p_obj, gap))
-            if gap < tol:
-                logger.debug('Convergence reached ! (gap: %s < %s)'
-                             % (gap, tol))
-                break
-    return X, active_set, E
-
-
 @verbose
 def _mixed_norm_solver_cd(M, G, alpha, lipschitz_constant, maxit=10000,
                           tol=1e-8, verbose=None, init=None, n_orient=1,
@@ -300,8 +137,7 @@ def _mixed_norm_solver_cd(M, G, alpha, lipschitz_constant, maxit=10000,
     assert M.ndim == G.ndim and M.shape[0] == G.shape[0]
 
     clf = MultiTaskLasso(alpha=alpha / len(M), tol=tol / sum_squared(M),
-                         normalize=False, fit_intercept=False, max_iter=maxit,
-                         warm_start=True)
+                         fit_intercept=False, max_iter=maxit, warm_start=True)
     if init is not None:
         clf.coef_ = init.T
     else:
@@ -318,10 +154,10 @@ def _mixed_norm_solver_cd(M, G, alpha, lipschitz_constant, maxit=10000,
 @verbose
 def _mixed_norm_solver_bcd(M, G, alpha, lipschitz_constant, maxit=200,
                            tol=1e-8, verbose=None, init=None, n_orient=1,
-                           dgap_freq=10):
+                           dgap_freq=10, use_accel=True, K=5):
     """Solve L21 inverse problem with block coordinate descent."""
-    n_sensors, n_times = M.shape
-    n_sensors, n_sources = G.shape
+    _, n_times = M.shape
+    _, n_sources = G.shape
     n_positions = n_sources // n_orient
 
     if init is None:
@@ -337,6 +173,10 @@ def _mixed_norm_solver_bcd(M, G, alpha, lipschitz_constant, maxit=200,
 
     alpha_lc = alpha / lipschitz_constant
 
+    if use_accel:
+        last_K_X = np.empty((K + 1, n_sources, n_times))
+        U = np.zeros((K, n_sources * n_times))
+
     # First make G fortran for faster access to blocks of columns
     G = np.asfortranarray(G)
     # Ensure these are correct for dgemm
@@ -355,8 +195,7 @@ def _mixed_norm_solver_bcd(M, G, alpha, lipschitz_constant, maxit=200,
         list_G_j_c.append(np.ascontiguousarray(G[:, idx]))
 
     for i in range(maxit):
-        _bcd(G, X, R, active_set, one_ovr_lc, n_orient, n_positions,
-             alpha_lc, list_G_j_c)
+        _bcd(G, X, R, active_set, one_ovr_lc, n_orient, alpha_lc, list_G_j_c)
 
         if (i + 1) % dgap_freq == 0:
             _, p_obj, d_obj, _ = dgap_l21(M, G, X[active_set], active_set,
@@ -372,13 +211,49 @@ def _mixed_norm_solver_bcd(M, G, alpha, lipschitz_constant, maxit=200,
                              % (gap, tol))
                 break
 
+        # using Anderson acceleration of the primal variable for faster
+        # convergence
+        if use_accel:
+            last_K_X[i % (K + 1)] = X
+
+            if i % (K + 1) == K:
+                for k in range(K):
+                    U[k] = last_K_X[k + 1].ravel() - last_K_X[k].ravel()
+                C = U @ U.T
+                # at least on ARM64 we can't rely on np.linalg.solve to
+                # reliably raise LinAlgError here, so use SVD instead
+                # equivalent to:
+                # z = np.linalg.solve(C, np.ones(K))
+                u, s, _ = np.linalg.svd(C, hermitian=True)
+                if s[-1] <= 1e-6 * s[0]:
+                    logger.debug("Iteration %d: LinAlg Error" % (i + 1))
+                    continue
+                z = ((u * 1 / s) @ u.T).sum(0)
+                c = z / z.sum()
+                X_acc = np.sum(
+                    last_K_X[:-1] * c[:, None, None], axis=0
+                )
+                _grp_norm2_acc = groups_norm2(X_acc, n_orient)
+                active_set_acc = _grp_norm2_acc != 0
+                if n_orient > 1:
+                    active_set_acc = np.kron(
+                        active_set_acc, np.ones(n_orient, dtype=bool)
+                    )
+                p_obj = _primal_l21(M, G, X[active_set], active_set, alpha,
+                                    n_orient)[0]
+                p_obj_acc = _primal_l21(M, G, X_acc[active_set_acc],
+                                        active_set_acc, alpha, n_orient)[0]
+                if p_obj_acc < p_obj:
+                    X = X_acc
+                    active_set = active_set_acc
+                    R = M - G[:, active_set] @ X[active_set]
+
     X = X[active_set]
 
     return X, active_set, E
 
 
-def _bcd(G, X, R, active_set, one_ovr_lc, n_orient, n_positions,
-         alpha_lc, list_G_j_c):
+def _bcd(G, X, R, active_set, one_ovr_lc, n_orient, alpha_lc, list_G_j_c):
     """Implement one full pass of BCD.
 
     BCD stands for Block Coordinate Descent.
@@ -403,7 +278,7 @@ def _bcd(G, X, R, active_set, one_ovr_lc, n_orient, n_positions,
     alpha_lc: array, shape (n_positions, )
         alpha * (Lipschitz constants).
     """
-    X_j_new = np.zeros_like(X[0:n_orient, :], order='C')
+    X_j_new = np.zeros_like(X[:n_orient, :], order='C')
     dgemm = _get_dgemm()
 
     for j, G_j_c in enumerate(list_G_j_c):
@@ -438,10 +313,12 @@ def _bcd(G, X, R, active_set, one_ovr_lc, n_orient, n_positions,
 @verbose
 def mixed_norm_solver(M, G, alpha, maxit=3000, tol=1e-8, verbose=None,
                       active_set_size=50, debias=True, n_orient=1,
-                      solver='auto', return_gap=False, dgap_freq=10):
+                      solver='auto', return_gap=False, dgap_freq=10,
+                      active_set_init=None, X_init=None):
     """Solve L1/L2 mixed-norm inverse problem with active set strategy.
 
-    See references :footcite:`GramfortEtAl2012,StrohmeierEtAl2016`.
+    See references :footcite:`GramfortEtAl2012,StrohmeierEtAl2016,
+    BertrandEtAl2020`.
 
     Parameters
     ----------
@@ -463,20 +340,28 @@ def mixed_norm_solver(M, G, alpha, maxit=3000, tol=1e-8, verbose=None,
         Debias source estimates.
     n_orient : int
         The number of orientation (1 : fixed or 3 : free or loose).
-    solver : 'prox' | 'cd' | 'bcd' | 'auto'
-        The algorithm to use for the optimization.
+    solver : 'cd' | 'bcd' | 'auto'
+        The algorithm to use for the optimization. Block Coordinate Descent
+        (BCD) uses Anderson acceleration for faster convergence.
     return_gap : bool
         Return final duality gap.
     dgap_freq : int
         The duality gap is computed every dgap_freq iterations of the solver on
         the active set.
+    active_set_init : array, shape (n_dipoles,) or None
+        The initial active set (boolean array) used at the first iteration.
+        If None, the usual active set strategy is applied.
+    X_init : array, shape (n_dipoles, n_times) or None
+        The initial weight matrix used for warm starting the solver. If None,
+        the weights are initialized at zero.
 
     Returns
     -------
     X : array, shape (n_active, n_times)
         The source estimates.
-    active_set : array
-        The mask of active sources.
+    active_set : array, shape (new_active_set_size,)
+        The mask of active sources. Note that new_active_set_size is the size
+        of the active set after convergence of the solver.
     E : list
         The value of the objective function over the iterations.
     gap : float
@@ -488,10 +373,11 @@ def mixed_norm_solver(M, G, alpha, maxit=3000, tol=1e-8, verbose=None,
     """
     n_dipoles = G.shape[1]
     n_positions = n_dipoles // n_orient
-    n_sensors, n_times = M.shape
+    _, n_times = M.shape
     alpha_max = norm_l2inf(np.dot(G.T, M), n_orient, copy=False)
     logger.info("-- ALPHA MAX : %s" % alpha_max)
     alpha = float(alpha)
+    X = np.zeros((n_dipoles, n_times), dtype=G.dtype)
 
     has_sklearn = True
     try:
@@ -499,6 +385,8 @@ def mixed_norm_solver(M, G, alpha, maxit=3000, tol=1e-8, verbose=None,
     except ImportError:
         has_sklearn = False
 
+    _validate_type(solver, str, 'solver')
+    _check_option('solver', solver, ('cd', 'bcd', 'auto'))
     if solver == 'auto':
         if has_sklearn and (n_orient == 1):
             solver = 'cd'
@@ -520,7 +408,8 @@ def mixed_norm_solver(M, G, alpha, maxit=3000, tol=1e-8, verbose=None,
         logger.info("Using coordinate descent")
         l21_solver = _mixed_norm_solver_cd
         lc = None
-    elif solver == 'bcd':
+    else:
+        assert solver == 'bcd'
         logger.info("Using block coordinate descent")
         l21_solver = _mixed_norm_solver_bcd
         G = np.asfortranarray(G)
@@ -531,16 +420,14 @@ def mixed_norm_solver(M, G, alpha, maxit=3000, tol=1e-8, verbose=None,
             for j in range(n_positions):
                 G_tmp = G[:, (j * n_orient):((j + 1) * n_orient)]
                 lc[j] = np.linalg.norm(np.dot(G_tmp.T, G_tmp), ord=2)
-    else:
-        logger.info("Using proximal iterations")
-        l21_solver = _mixed_norm_solver_prox
-        lc = 1.01 * np.linalg.norm(G, ord=2) ** 2
 
     if active_set_size is not None:
         E = list()
         highest_d_obj = - np.inf
-        X_init = None
-        active_set = np.zeros(n_dipoles, dtype=bool)
+        if X_init is not None and X_init.shape != (n_dipoles, n_times):
+            raise ValueError('Wrong dim for initialized coefficients.')
+        active_set = (active_set_init if active_set_init is not None else
+                      np.zeros(n_dipoles, dtype=bool))
         idx_large_corr = np.argsort(groups_norm2(np.dot(G.T, M), n_orient))
         new_active_idx = idx_large_corr[-active_set_size:]
         if n_orient > 1:
@@ -548,6 +435,7 @@ def mixed_norm_solver(M, G, alpha, maxit=3000, tol=1e-8, verbose=None,
                               np.arange(n_orient)[None, :]).ravel()
         active_set[new_active_idx] = True
         as_size = np.sum(active_set)
+        gap = np.inf
         for k in range(maxit):
             if solver == 'bcd':
                 lc_tmp = lc[active_set[::n_orient]]
@@ -566,7 +454,7 @@ def mixed_norm_solver(M, G, alpha, maxit=3000, tol=1e-8, verbose=None,
             highest_d_obj = max(d_obj, highest_d_obj)
             gap = p_obj - highest_d_obj
             E.append(p_obj)
-            logger.info("Iteration %d :: p_obj %f :: dgap %f ::"
+            logger.info("Iteration %d :: p_obj %f :: dgap %f :: "
                         "n_active_start %d :: n_active_end %d" % (
                             k + 1, p_obj, gap, as_size // n_orient,
                             np.sum(active_set) // n_orient))
@@ -614,7 +502,7 @@ def mixed_norm_solver(M, G, alpha, maxit=3000, tol=1e-8, verbose=None,
 def iterative_mixed_norm_solver(M, G, alpha, n_mxne_iter, maxit=3000,
                                 tol=1e-8, verbose=None, active_set_size=50,
                                 debias=True, n_orient=1, dgap_freq=10,
-                                solver='auto'):
+                                solver='auto', weight_init=None):
     """Solve L0.5/L2 mixed-norm inverse problem with active set strategy.
 
     See reference :footcite:`StrohmeierEtAl2016`.
@@ -644,8 +532,11 @@ def iterative_mixed_norm_solver(M, G, alpha, n_mxne_iter, maxit=3000,
         The number of orientation (1 : fixed or 3 : free or loose).
     dgap_freq : int or np.inf
         The duality gap is evaluated every dgap_freq iterations.
-    solver : 'prox' | 'cd' | 'bcd' | 'auto'
+    solver : 'cd' | 'bcd' | 'auto'
         The algorithm to use for the optimization.
+    weight_init : array, shape (n_dipoles,) or None
+        The initial weight used for reweighting the gain matrix. If None, the
+        weights are initialized with ones.
 
     Returns
     -------
@@ -668,8 +559,13 @@ def iterative_mixed_norm_solver(M, G, alpha, n_mxne_iter, maxit=3000,
 
     E = list()
 
-    active_set = np.ones(G.shape[1], dtype=bool)
-    weights = np.ones(G.shape[1])
+    if weight_init is not None and weight_init.shape != (G.shape[1],):
+        raise ValueError('Wrong dimension for weight initialization. Got %s. '
+                         'Expected %s.' % (weight_init.shape, (G.shape[1],)))
+
+    weights = weight_init if weight_init is not None else np.ones(G.shape[1])
+    active_set = (weights != 0)
+    weights = weights[active_set]
     X = np.zeros((G.shape[1], M.shape[1]))
 
     for k in range(n_mxne_iter):
@@ -698,7 +594,6 @@ def iterative_mixed_norm_solver(M, G, alpha, n_mxne_iter, maxit=3000,
 
         if _active_set.sum() > 0:
             active_set[active_set] = _active_set
-
             # Reapply weights to have correct unit
             X *= weights[_active_set][:, np.newaxis]
             weights = gprime(X)
@@ -1472,9 +1367,7 @@ def iterative_tf_mixed_norm_solver(M, G, alpha_space, alpha_time,
         The number of orientation (1 : fixed or 3 : free or loose).
     dgap_freq : int or np.inf
         The duality gap is evaluated every dgap_freq iterations.
-    verbose : bool, str, int, or None
-        If not None, override default verbose level (see :func:`mne.verbose`
-        and :ref:`Logging documentation <tut_logging>` for more).
+    %(verbose)s
 
     Returns
     -------
