@@ -8,7 +8,7 @@
 #          Okba Bekhelifi <okba.bekhelifi@gmail.com>
 #          Stefan Appelhoff <stefan.appelhoff@mailbox.org>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import configparser
 import os
@@ -26,6 +26,7 @@ from ..base import BaseRaw
 from ..utils import _read_segments_file, _mult_cal_one
 from ...annotations import Annotations, read_annotations
 from ...channels import make_dig_montage
+from ...defaults import HEAD_SIZE_DEFAULT
 
 
 @fill_doc
@@ -38,13 +39,13 @@ class RawBrainVision(BaseRaw):
         Path to the EEG header file.
     eog : list or tuple
         Names of channels or list of indices that should be designated
-        EOG channels. Values should correspond to the vhdr file.
+        EOG channels. Values should correspond to the header file.
         Default is ``('HEOGL', 'HEOGR', 'VEOGb')``.
     misc : list or tuple of str | 'auto'
         Names of channels or list of indices that should be designated
         MISC channels. Values should correspond to the electrodes
-        in the vhdr file. If 'auto', units in vhdr file are used for inferring
-        misc channels. Default is ``'auto'``.
+        in the header file. If 'auto', units in header file are used for
+        inferring misc channels. Default is ``'auto'``.
     scale : float
         The scaling factor for EEG data. Unless specified otherwise by
         header file, units are in microvolts. Default scale factor is 1.
@@ -67,9 +68,11 @@ class RawBrainVision(BaseRaw):
                  scale=1., preload=False, verbose=None):  # noqa: D107
         # Channel info and events
         logger.info('Extracting parameters from %s...' % vhdr_fname)
-        vhdr_fname = op.abspath(vhdr_fname)
+        hdr_fname = op.abspath(vhdr_fname)
+        ext = op.splitext(hdr_fname)[-1]
+        ahdr_format = True if ext == '.ahdr' else False
         (info, data_fname, fmt, order, n_samples, mrk_fname, montage,
-         orig_units) = _get_vhdr_info(vhdr_fname, eog, misc, scale)
+         orig_units) = _get_hdr_info(hdr_fname, eog, misc, scale)
 
         with open(data_fname, 'rb') as f:
             if isinstance(fmt, dict):  # ASCII, this will be slow :(
@@ -89,24 +92,29 @@ class RawBrainVision(BaseRaw):
                 offsets = None
                 n_samples = n_samples // (dtype_bytes * n_data_ch)
 
+        orig_format = "single" if isinstance(fmt, dict) else fmt
         raw_extras = dict(
             offsets=offsets, fmt=fmt, order=order, n_samples=n_samples)
         super(RawBrainVision, self).__init__(
             info, last_samps=[n_samples - 1], filenames=[data_fname],
-            orig_format=fmt, preload=preload, verbose=verbose,
+            orig_format=orig_format, preload=preload, verbose=verbose,
             raw_extras=[raw_extras], orig_units=orig_units)
 
         self.set_montage(montage)
 
-        settings, cfg, cinfo, _ = _aux_vhdr_info(vhdr_fname)
+        settings, cfg, cinfo, _ = _aux_hdr_info(hdr_fname)
         split_settings = settings.splitlines()
         self.impedances = _parse_impedance(split_settings,
                                            self.info['meas_date'])
 
-        # Get annotations from vmrk file
+        # Get annotations from marker file
         annots = read_annotations(mrk_fname, info['sfreq'])
         self.set_annotations(annots)
 
+        # Drop the fake ahdr channel if needed
+        if ahdr_format:
+            self.drop_channels(_AHDR_CHANNEL_NAME)
+
     def _read_segment_file(self, data, idx, fi, start, stop, cals, mult):
         """Read a chunk of raw data."""
         # read data
@@ -125,8 +133,28 @@ class RawBrainVision(BaseRaw):
                 block = np.empty((n_data_ch, stop - start))
                 for ii in range(stop - start):
                     line = fid.readline().decode('ASCII')
-                    line = line.strip().replace(',', '.').split()
-                    block[:n_data_ch, ii] = [float(part) for part in line]
+                    line = line.strip()
+
+                    # Not sure why we special-handle the "," character here,
+                    # but let's just keep this for historical and backward-
+                    # compat reasons
+                    if (isinstance(fmt, dict) and
+                            'decimalsymbol' in fmt and
+                            fmt['decimalsymbol'] != '.'):
+                        line = line.replace(',', '.')
+
+                    if ' ' in line:
+                        line_data = line.split()
+                    elif ',' in line:
+                        # likely exported from BrainVision Analyzer?
+                        line_data = line.split(',')
+                    else:
+                        raise RuntimeError(
+                            'Unknown BrainVision data format encountered. '
+                            'Please contact the MNE-Python developers.'
+                        )
+
+                    block[:n_data_ch, ii] = [float(part) for part in line_data]
             _mult_cal_one(data, block, idx, cals, mult)
 
 
@@ -146,13 +174,13 @@ def _read_segments_c(raw, data, idx, fi, start, stop, cals, mult):
     _mult_cal_one(data, block, idx, cals, mult)
 
 
-def _read_vmrk(fname):
-    """Read annotations from a vmrk file.
+def _read_mrk(fname):
+    """Read annotations from a vmrk/amrk file.
 
     Parameters
     ----------
     fname : str
-        vmrk file to be read.
+        vmrk/amrk file to be read.
 
     Returns
     -------
@@ -166,7 +194,7 @@ def _read_vmrk(fname):
         The recording time as a string. Defaults to empty string if no
         recording time is found.
     """
-    # read vmrk file
+    # read marker file
     with open(fname, 'rb') as fid:
         txt = fid.read()
 
@@ -239,35 +267,39 @@ def _read_vmrk(fname):
 
 
 def _read_annotations_brainvision(fname, sfreq='auto'):
-    """Create Annotations from BrainVision vrmk.
+    """Create Annotations from BrainVision vmrk/amrk.
 
-    This function reads a .vrmk file and makes an
+    This function reads a .vmrk or .amrk file and makes an
     :class:`mne.Annotations` object.
 
     Parameters
     ----------
     fname : str | object
-        The path to the .vmrk file.
+        The path to the .vmrk/.amrk file.
     sfreq : float | 'auto'
         The sampling frequency in the file. It's necessary
-        as Annotations are expressed in seconds and vmrk
+        as Annotations are expressed in seconds and vmrk/amrk
         files are in samples. If set to 'auto' then
-        the sfreq is taken from the .vhdr file that
+        the sfreq is taken from the .vhdr/.ahdr file that
         has the same name (without file extension). So
-        data.vrmk looks for sfreq in data.vhdr.
+        data.vmrk/amrk looks for sfreq in data.vhdr or,
+        if it does not exist, in data.ahdr.
 
     Returns
     -------
     annotations : instance of Annotations
         The annotations present in the file.
     """
-    onset, duration, description, date_str = _read_vmrk(fname)
+    onset, duration, description, date_str = _read_mrk(fname)
     orig_time = _str_to_meas_date(date_str)
 
     if sfreq == 'auto':
-        vhdr_fname = op.splitext(fname)[0] + '.vhdr'
-        logger.info("Finding 'sfreq' from header file: %s" % vhdr_fname)
-        _, _, _, info = _aux_vhdr_info(vhdr_fname)
+        hdr_fname = op.splitext(fname)[0] + '.vhdr'
+        # if vhdr file does not exist assume that the format is ahdr
+        if not op.exists(hdr_fname):
+            hdr_fname = op.splitext(fname)[0] + '.ahdr'
+        logger.info("Finding 'sfreq' from header file: %s" % hdr_fname)
+        _, _, _, info = _aux_hdr_info(hdr_fname)
         sfreq = info['sfreq']
 
     onset = np.array(onset, dtype=float) / sfreq
@@ -278,22 +310,26 @@ def _read_annotations_brainvision(fname, sfreq='auto'):
     return annotations
 
 
-_data_err = """\
-MNE-Python currently only supports %s versions 1.0 and 2.0, got unparsable \
-%r. Contact MNE-Python developers for support."""
-# optional space, optional Core, Version/Header, optional comma, 1/2
-_data_re = r'Brain ?Vision( Core)? Data Exchange %s File,? Version %s\.0'
-
-
 def _check_bv_version(header, kind):
     """Check the header version."""
+    _data_err = """\
+    MNE-Python currently only supports %s versions 1.0 and 2.0, got unparsable\
+     %r. Contact MNE-Python developers for support."""
+    # optional space, optional Core or V-Amp, optional Exchange,
+    # Version/Header, optional comma, 1/2
+    _data_re = (r"Brain ?Vision( Core| V-Amp)? Data( Exchange)? "
+                r"%s File,? Version %s\.0")
+
     assert kind in ('header', 'marker')
+
+    if header == '':
+        warn(f'Missing header in {kind} file.')
     for version in range(1, 3):
         this_re = _data_re % (kind.capitalize(), version)
         if re.search(this_re, header) is not None:
             return version
     else:
-        raise ValueError(_data_err % (kind, header))
+        warn(_data_err % (kind, header))
 
 
 _orientation_dict = dict(MULTIPLEXED='F', VECTORIZED='C')
@@ -305,7 +341,9 @@ _unit_dict = {'V': 1.,  # V stands for Volt
               'uV': 1e-6,
               'mV': 1e-3,
               'nV': 1e-9,
-              'C': 1,  # C stands for celsius
+              'C': 1,  # C stands for Celsius
+              '°C': 1,  # degrees Celsius
+              'n/a': 1,  # unit "not available" (or applicable)
               'µS': 1e-6,  # S stands for Siemens
               'uS': 1e-6,
               'ARU': 1,  # ARU is the unity for the breathing data
@@ -333,9 +371,9 @@ def _str_to_meas_date(date_str):
     return meas_date
 
 
-def _aux_vhdr_info(vhdr_fname):
-    """Aux function for _get_vhdr_info."""
-    with open(vhdr_fname, 'rb') as f:
+def _aux_hdr_info(hdr_fname):
+    """Aux function for _get_hdr_info."""
+    with open(hdr_fname, 'rb') as f:
         # extract the first section to resemble a cfg
         header = f.readline()
         codepage = 'utf-8'
@@ -384,23 +422,25 @@ def _aux_vhdr_info(vhdr_fname):
     # Sampling interval is given in microsec
     sfreq = 1e6 / cfg.getfloat(cinfostr, 'SamplingInterval')
     info = _empty_info(sfreq)
+    info._unlocked = False
     return settings, cfg, cinfostr, info
 
 
-def _get_vhdr_info(vhdr_fname, eog, misc, scale):
+@fill_doc
+def _get_hdr_info(hdr_fname, eog, misc, scale):
     """Extract all the information from the header file.
 
     Parameters
     ----------
-    vhdr_fname : str
+    hdr_fname : str
         Raw EEG header to be read.
     eog : list of str
         Names of channels that should be designated EOG channels. Names should
-        correspond to the vhdr file.
+        correspond to the header file.
     misc : list or tuple of str | 'auto'
         Names of channels or list of indices that should be designated
-        MISC channels. Values should correspond to the electrodes
-        in the vhdr file. If 'auto', units in vhdr file are used for inferring
+        MISC channels. Values should correspond to the electrodes in the
+        header file. If 'auto', units in header file are used for inferring
         misc channels. Default is ``'auto'``.
     scale : float
         The scaling factor for EEG data. Unless specified otherwise by
@@ -408,8 +448,7 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale):
 
     Returns
     -------
-    info : Info
-        The measurement info.
+    %(info_not_none)s
     data_fname : str
         Path to the binary data file.
     fmt : str
@@ -427,12 +466,14 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale):
         the header file. Example: {'FC1': 'nV'}
     """
     scale = float(scale)
-    ext = op.splitext(vhdr_fname)[-1]
-    if ext != '.vhdr':
+    ext = op.splitext(hdr_fname)[-1]
+    ahdr_format = (ext == '.ahdr')
+    if ext not in ('.vhdr', '.ahdr'):
         raise IOError("The header file must be given to read the data, "
                       "not a file with extension '%s'." % ext)
 
-    settings, cfg, cinfostr, info = _aux_vhdr_info(vhdr_fname)
+    settings, cfg, cinfostr, info = _aux_hdr_info(hdr_fname)
+    info._unlocked = True
 
     order = cfg.get(cinfostr, 'DataOrientation')
     if order not in _orientation_dict:
@@ -455,7 +496,7 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale):
                for key in cfg.options('ASCII Infos')}
 
     # locate EEG binary file and marker file for the stim channel
-    path = op.dirname(vhdr_fname)
+    path = op.dirname(hdr_fname)
     data_fname = op.join(path, cfg.get(cinfostr, 'DataFile'))
     mrk_fname = op.join(path, cfg.get(cinfostr, 'MarkerFile'))
 
@@ -467,25 +508,26 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale):
 
     for line in lines:
         match = re.findall(regexp, line.strip())
-
         # Always take first measurement date we find
         if match:
             date_str = match[0]
             info['meas_date'] = _str_to_meas_date(date_str)
             break
-
     else:
         info['meas_date'] = None
 
     # load channel labels
     nchan = cfg.getint(cinfostr, 'NumberOfChannels')
+    if ahdr_format:
+        # add one fake channel for ahdr format
+        nchan += 1
     n_samples = None
     if order == 'C':
         try:
             n_samples = cfg.getint(cinfostr, 'DataPoints')
         except configparser.NoOptionError:
-            logger.warning('No info on DataPoints found. Inferring number of '
-                           'samples from the data file size.')
+            warn('No info on DataPoints found. Inferring number of '
+                 'samples from the data file size.')
             with open(data_fname, 'rb') as fid:
                 fid.seek(0, 2)
                 n_bytes = fid.tell()
@@ -518,7 +560,7 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale):
         ch_dict[chan] = name
         ch_names[n] = name
         if resolution == "":
-            if not(unit):  # For truncated vhdrs (e.g. EEGLAB export)
+            if not unit:  # For truncated vhdrs (e.g. EEGLAB export)
                 resolution = 0.000001
             else:
                 resolution = 1.  # for files with units specified, but not res
@@ -529,13 +571,21 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale):
         if unit not in ('V', 'mV', 'µV', 'uV', 'nV'):
             misc_chs[name] = (FIFF.FIFF_UNIT_CEL if unit == 'C'
                               else FIFF.FIFF_UNIT_NONE)
+    if ahdr_format:
+        ch_dict[_AHDR_CHANNEL_NAME] = _AHDR_CHANNEL_NAME
+        ch_names[-1] = _AHDR_CHANNEL_NAME
+        orig_units[_AHDR_CHANNEL_NAME] = 'V'
+        cals[-1] = 1.
+        ranges[-1] = 1
+
     misc = list(misc_chs.keys()) if misc == 'auto' else misc
 
-    # create montage: 'Coordinates' section in VHDR file corresponds to "BVEF"
-    # BrainVision Electrode File. The data are based on BrainVision Analyzer
-    # coordinate system: Defined between standard electrode positions: X-axis
-    # from T7 to T8, Y-axis from Oz to Fpz, Z-axis orthogonal from XY-plane
-    # through Cz, fit to a sphere if idealized (when radius=1), specified in mm
+    # create montage: 'Coordinates' section in VHDR/AHDR file corresponds to
+    # "BVEF" BrainVision Electrode File. The data are based on BrainVision
+    # Analyzer coordinate system: Defined between standard electrode positions:
+    # X-axis from T7 to T8, Y-axis from Oz to Fpz, Z-axis orthogonal from
+    # XY-plane through Cz, fit to a sphere if idealized (when radius=1),
+    # specified in mm
     montage = None
     if cfg.has_section('Coordinates'):
         from ...transforms import _sph_to_cart
@@ -552,8 +602,8 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale):
             az = np.deg2rad(phi)
             # Coordinates could be "idealized" (spherical head model)
             if rad == 1:
-                # scale up to realistic head radius (8.5cm == 85mm)
-                rad *= 85.
+                # scale up to realistic head radius: *1000 to convert m to mm
+                rad *= HEAD_SIZE_DEFAULT * 1000
             pos = _sph_to_cart(np.array([[rad, az, pol]]))[0]
             if (pos == 0).all() and ch_name not in list(eog) + misc:
                 to_misc.append(ch_name)
@@ -634,6 +684,8 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale):
         lp_s = '[s]' in header[lp_col]
 
         for i, ch in enumerate(ch_names, 1):
+            if ahdr_format and i == len(ch_names) and ch == _AHDR_CHANNEL_NAME:
+                break
             # double check alignment with channel by using the hw settings
             if idx == idx_amp:
                 line_amp = settings[idx + i]
@@ -652,6 +704,7 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale):
             line = re.split(divider, settings[idx + i])
             highpass.append(line[hp_col + real_shift])
             lowpass.append(line[lp_col + real_shift])
+
         if len(highpass) == 0:
             pass
         elif len(set(highpass)) == 1:
@@ -701,7 +754,7 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale):
         if len(lowpass) == 0:
             pass
         elif len(set(lowpass)) == 1:
-            if lowpass[0] in ('NaN', 'Off'):
+            if lowpass[0] in ('NaN', 'Off', '0'):
                 pass  # Placeholder for future use. Lowpass set in _empty_info
             else:
                 info['lowpass'] = float(lowpass[0])
@@ -714,7 +767,7 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale):
             if lp_s:
                 # We convert channels with disabled filters to having
                 # infinitely relaxed / no filters
-                lowpass = [float(filt) if filt not in ('NaN', 'Off')
+                lowpass = [float(filt) if filt not in ('NaN', 'Off', '0')
                            else 0.0 for filt in lowpass]
                 info['lowpass'] = np.min(np.array(lowpass, dtype=np.float64))
                 try:
@@ -736,7 +789,7 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale):
             else:
                 # We convert channels with disabled filters to having
                 # infinitely relaxed / no filters
-                lowpass = [float(filt) if filt not in ('NaN', 'Off')
+                lowpass = [float(filt) if filt not in ('NaN', 'Off', '0')
                            else np.Inf for filt in lowpass]
                 info['lowpass'] = np.max(np.array(lowpass, dtype=np.float64))
 
@@ -792,6 +845,7 @@ def _get_vhdr_info(vhdr_fname, eog, misc, scale):
             unit=unit, unit_mul=FIFF.FIFF_UNITM_NONE,
             coord_frame=FIFF.FIFFV_COORD_HEAD))
 
+    info._unlocked = False
     info._update_redundant()
     return (info, data_fname, fmt, order, n_samples, mrk_fname, montage,
             orig_units)
@@ -809,12 +863,12 @@ def read_raw_brainvision(vhdr_fname,
         Path to the EEG header file.
     eog : list or tuple of str
         Names of channels or list of indices that should be designated
-        EOG channels. Values should correspond to the vhdr file
+        EOG channels. Values should correspond to the header file
         Default is ``('HEOGL', 'HEOGR', 'VEOGb')``.
     misc : list or tuple of str | 'auto'
         Names of channels or list of indices that should be designated
-        MISC channels. Values should correspond to the electrodes
-        in the vhdr file. If 'auto', units in vhdr file are used for inferring
+        MISC channels. Values should correspond to the electrodes in the
+        header file. If 'auto', units in header file are used for inferring
         misc channels. Default is ``'auto'``.
     scale : float
         The scaling factor for EEG data. Unless specified otherwise by
@@ -842,6 +896,7 @@ _OTHER_ACCEPTED_MARKERS = {
     'New Segment/': 99999, 'SyncStatus/Sync On': 99998
 }
 _OTHER_OFFSET = 10001  # where to start "unknown" event_ids
+_AHDR_CHANNEL_NAME = "AHDR_CHANNEL"
 
 
 class _BVEventParser(_DefaultEventParser):
@@ -876,9 +931,9 @@ def _parse_impedance(settings, recording_date=None):
     Parameters
     ----------
     settings : list
-        The header settings lines fom the VHDR file.
+        The header settings lines fom the VHDR/AHDR file.
     recording_date : datetime.datetime | None
-        The date of the recording as extracted from the VMRK file.
+        The date of the recording as extracted from the VMRK/AMRK file.
 
     Returns
     -------
@@ -938,7 +993,7 @@ def _parse_impedance_ranges(settings):
     Parameters
     ----------
     settings : list
-        The header settings lines fom the VHDR file.
+        The header settings lines fom the VHDR/AHDR file.
 
     Returns
     -------
