@@ -3,14 +3,14 @@
 # Authors: Dirk Gütlin <dirk.guetlin@stud.sbg.ac.at>
 #
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
-import os.path as op
-import numpy as np
-from shutil import copyfile
 from datetime import datetime, timezone
+import os.path as op
+from shutil import copyfile
 
 import pytest
+import numpy as np
 from numpy.testing import assert_allclose, assert_array_equal
 
 from mne.annotations import events_from_annotations
@@ -22,7 +22,7 @@ from mne.io.constants import FIFF
 from mne.io.edf import read_raw_bdf
 from mne.io.bti import read_raw_bti
 from mne.io.curry import read_raw_curry
-from mne.utils import check_version, run_tests_if_main, catch_logging
+from mne.utils import check_version, catch_logging, _record_warnings
 from mne.annotations import read_annotations
 from mne.io.curry.curry import (_get_curry_version, _get_curry_file_structure,
                                 _read_events_curry, FILE_EXTENSIONS)
@@ -70,7 +70,7 @@ def bdf_curry_ref():
 @pytest.mark.parametrize('preload', [True, False])
 def test_read_raw_curry(fname, tol, preload, bdf_curry_ref):
     """Test reading CURRY files."""
-    with pytest.warns(None) as wrn:
+    with _record_warnings() as wrn:
         raw = read_raw_curry(fname, preload=preload)
 
     if not check_version('numpy', '1.16') and preload and fname.endswith(
@@ -87,12 +87,12 @@ def test_read_raw_curry(fname, tol, preload, bdf_curry_ref):
         assert_array_equal([ch[field] for ch in raw.info['chs']],
                            [ch[field] for ch in bdf_curry_ref.info['chs']])
 
-    raw.verbose = 'error'  # don't emit warnings about slow reading
-    assert_allclose(raw.get_data(), bdf_curry_ref.get_data(), atol=tol)
+    assert_allclose(raw.get_data(verbose='error'),
+                    bdf_curry_ref.get_data(), atol=tol)
 
     picks, start, stop = ["C3", "C4"], 200, 800
     assert_allclose(
-        raw.get_data(picks=picks, start=start, stop=stop),
+        raw.get_data(picks=picks, start=start, stop=stop, verbose='error'),
         bdf_curry_ref.get_data(picks=picks, start=start, stop=stop),
         rtol=tol)
     assert raw.info['dev_head_t'] is None
@@ -205,21 +205,21 @@ WANT_TRANS = np.array(
     pytest.param(curry8_rfDC_file, 1e-3, id='curry 8'),
 ])
 @pytest.mark.parametrize('mock_dev_head_t', [True, False])
-def test_read_raw_curry_rfDC(fname, tol, mock_dev_head_t, tmpdir):
+def test_read_raw_curry_rfDC(fname, tol, mock_dev_head_t, tmp_path):
     """Test reading CURRY files."""
     if mock_dev_head_t:
         if 'Curry 7' in fname:  # not supported yet
             return
-        # copy files to tmpdir
+        # copy files to tmp_path
         base = op.splitext(fname)[0]
         for ext in ('.cdt', '.cdt.dpa'):
             src = base + ext
-            dst = op.join(tmpdir, op.basename(base) + ext)
+            dst = op.join(tmp_path, op.basename(base) + ext)
             copyfile(src, dst)
             if ext == '.cdt.dpa':
                 with open(dst, 'a') as fid:
                     fid.write(LM_CONTENT)
-        fname = op.join(tmpdir, op.basename(fname))
+        fname = op.join(tmp_path, op.basename(fname))
         with open(fname + '.hpi', 'w') as fid:
             fid.write(HPI_CONTENT)
 
@@ -331,12 +331,12 @@ def _mock_info_file(src, dst, sfreq, time_step):
     pytest.param(dict(sfreq=500, time_step=42), id='mismatch',
                  marks=pytest.mark.xfail(raises=ValueError)),
 ])
-def sfreq_testing_data(tmpdir, request):
+def sfreq_testing_data(tmp_path, request):
     """Generate different sfreq, time_step scenarios to be tested."""
     sfreq, time_step = request.param['sfreq'], request.param['time_step']
 
     in_base_name = curry7_bdf_file.strip('dat')
-    out_base_name = str(tmpdir.join('curry.'))
+    out_base_name = str(tmp_path / 'curry.')
 
     # create dummy empty files for 'dat' and 'rs3'
     for fname in [out_base_name + ext for ext in ['dat', 'rs3']]:
@@ -391,8 +391,9 @@ def _get_read_annotations_mock_info(name_part, mock_dir):
     version = _get_curry_version(ext)
     original['info'] = original['base'] + FILE_EXTENSIONS[version]["info"]
 
-    modified['base'] = str(mock_dir.join('curry'))
-    modified['event'] = modified['base'] + FILE_EXTENSIONS[version]["events"]
+    modified['base'] = str(mock_dir / 'curry')
+    modified['event'] = (modified['base'] +
+                         FILE_EXTENSIONS[version]["events_cef"])
     modified['info'] = modified['base'] + FILE_EXTENSIONS[version]["info"]
 
     return original, modified
@@ -405,7 +406,7 @@ def _get_read_annotations_mock_info(name_part, mock_dir):
     pytest.param('7 ASCII.cef', id='7 (ascii)'),
     pytest.param('8 ASCII.cdt.cef', id='8 (ascii)'),
 ])
-def test_read_curry_annotations_using_mocked_info(tmpdir, name_part):
+def test_read_curry_annotations_using_mocked_info(tmp_path, name_part):
     """Test reading for Curry events file."""
     EXPECTED_ONSET = [0.484, 0.486, 0.62, 0.622, 1.904, 1.906, 3.212, 3.214,
                       4.498, 4.5, 5.8, 5.802, 7.074, 7.076, 8.324, 8.326, 9.58,
@@ -416,7 +417,7 @@ def test_read_curry_annotations_using_mocked_info(tmpdir, name_part):
                             '1', '50000', '1', '50000']
 
     original, fname = _get_read_annotations_mock_info("Curry " + name_part,
-                                                      tmpdir)
+                                                      tmp_path)
     copyfile(src=original['event'], dst=fname['event'])
 
     _msg = 'required files cannot be found'
@@ -477,6 +478,3 @@ def test_meas_date(fname, expected_meas_date):
     # If the information is not valid, raw.info['meas_date'] should be None
     raw = read_raw_curry(fname, preload=False)
     assert raw.info['meas_date'] == expected_meas_date
-
-
-run_tests_if_main()
