@@ -9,7 +9,7 @@
 #          Clemens Brunner <clemens.brunner@gmail.com>
 #          Jeroen Van Der Donckt (IDlab - imec) <jeroen.vanderdonckt@ugent.be>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 from datetime import datetime, timezone, timedelta
 import os
@@ -27,6 +27,22 @@ from ...utils import fill_doc
 from ...annotations import Annotations
 
 
+# common channel type names mapped to internal ch types
+CH_TYPE_MAPPING = {
+    'EEG': FIFF.FIFFV_EEG_CH,
+    'SEEG': FIFF.FIFFV_SEEG_CH,
+    'ECOG': FIFF.FIFFV_ECOG_CH,
+    'DBS': FIFF.FIFFV_DBS_CH,
+    'EOG': FIFF.FIFFV_EOG_CH,
+    'ECG': FIFF.FIFFV_ECG_CH,
+    'EMG': FIFF.FIFFV_EMG_CH,
+    'BIO': FIFF.FIFFV_BIO_CH,
+    'RESP': FIFF.FIFFV_RESP_CH,
+    'MISC': FIFF.FIFFV_MISC_CH,
+    'SAO2': FIFF.FIFFV_BIO_CH,
+}
+
+
 @fill_doc
 class RawEDF(BaseRaw):
     """Raw object from EDF, EDF+ or BDF file.
@@ -48,17 +64,24 @@ class RawEDF(BaseRaw):
         'trigger' (case insensitive) are set to STIM. If str (or list of str),
         all channels matching the name(s) are set to STIM. If int (or list of
         ints), the channels corresponding to the indices are set to STIM.
-
-        .. warning:: 0.18 does not allow for stim channel synthesis from TAL
-                     channels called 'EDF Annotations' or 'BDF Annotations'
-                     anymore. Instead, TAL channels are parsed and extracted
-                     annotations are stored in raw.annotations. Use
-                     :func:`mne.events_from_annotations` to obtain events from
-                     these annotations.
-
     exclude : list of str
         Channel names to exclude. This can help when reading data with
         different sampling rates to avoid unnecessary resampling.
+    infer_types : bool
+        If True, try to infer channel types from channel labels. If a channel
+        label starts with a known type (such as 'EEG') followed by a space and
+        a name (such as 'Fp1'), the channel type will be set accordingly, and
+        the channel will be renamed to the original label without the prefix.
+        For unknown prefixes, the type will be 'EEG' and the name will not be
+        modified. If False, do not infer types and assume all channels are of
+        type 'EEG'.
+
+        .. versionadded:: 0.24.1
+    include : list of str | str
+        Channel names to be included. A str is interpreted as a regular
+        expression. 'exclude' must be empty if include is assigned.
+
+        .. versionadded:: 1.1
     %(preload)s
     %(verbose)s
 
@@ -107,13 +130,14 @@ class RawEDF(BaseRaw):
     """
 
     @verbose
-    def __init__(self, input_fname, eog=None, misc=None,
-                 stim_channel='auto', exclude=(), preload=False, verbose=None):
+    def __init__(self, input_fname, eog=None, misc=None, stim_channel='auto',
+                 exclude=(), infer_types=False, preload=False, include=None,
+                 verbose=None):
         logger.info('Extracting EDF parameters from {}...'.format(input_fname))
         input_fname = os.path.abspath(input_fname)
-        info, edf_info, orig_units = _get_info(input_fname,
-                                               stim_channel, eog, misc,
-                                               exclude, preload)
+        info, edf_info, orig_units = _get_info(input_fname, stim_channel, eog,
+                                               misc, exclude, infer_types,
+                                               preload, include)
         logger.info('Creating raw.info structure...')
 
         # Raw attributes
@@ -167,6 +191,13 @@ class RawGDF(BaseRaw):
     exclude : list of str
         Channel names to exclude. This can help when reading data with
         different sampling rates to avoid unnecessary resampling.
+
+        .. versionadded:: 0.24.1
+    include : list of str | str
+        Channel names to be included. A str is interpreted as a regular
+        expression. 'exclude' must be empty if include is assigned.
+
+        .. versionadded:: 1.1
     %(preload)s
     %(verbose)s
 
@@ -184,12 +215,13 @@ class RawGDF(BaseRaw):
 
     @verbose
     def __init__(self, input_fname, eog=None, misc=None,
-                 stim_channel='auto', exclude=(), preload=False, verbose=None):
+                 stim_channel='auto', exclude=(), preload=False, include=None,
+                 verbose=None):
         logger.info('Extracting EDF parameters from {}...'.format(input_fname))
         input_fname = os.path.abspath(input_fname)
-        info, edf_info, orig_units = _get_info(input_fname,
-                                               stim_channel, eog, misc,
-                                               exclude, preload)
+        info, edf_info, orig_units = _get_info(input_fname, stim_channel, eog,
+                                               misc, exclude, True, preload,
+                                               include)
         logger.info('Creating raw.info structure...')
 
         # Raw attributes
@@ -283,7 +315,8 @@ def _read_segment_file(data, idx, fi, start, stop, raw_extras, filenames,
             one = np.zeros((len(orig_sel), d_eidx - d_sidx), dtype=data.dtype)
             for ii, ci in enumerate(read_sel):
                 # This now has size (n_chunks_read, n_samp[ci])
-                ch_data = many_chunk[:, ch_offsets[ci]:ch_offsets[ci + 1]]
+                ch_data = many_chunk[:,
+                                     ch_offsets[ci]:ch_offsets[ci + 1]].copy()
 
                 if ci in tal_idx:
                     tal_data.append(ch_data)
@@ -324,16 +357,28 @@ def _read_segment_file(data, idx, fi, start, stop, raw_extras, filenames,
     return tal_data
 
 
-def _read_header(fname, exclude):
-    """Unify edf, bdf and gdf _read_header call.
+def _read_header(fname, exclude, infer_types, include=None):
+    """Unify EDF, BDF and GDF _read_header call.
 
     Parameters
     ----------
     fname : str
         Path to the EDF+, BDF, or GDF file.
-    exclude : list of str
+    exclude : list of str | str
         Channel names to exclude. This can help when reading data with
-        different sampling rates to avoid unnecessary resampling.
+        different sampling rates to avoid unnecessary resampling. A str is
+        interpreted as a regular expression.
+    infer_types : bool
+        If True, try to infer channel types from channel labels. If a channel
+        label starts with a known type (such as 'EEG') followed by a space and
+        a name (such as 'Fp1'), the channel type will be set accordingly, and
+        the channel will be renamed to the original label without the prefix.
+        For unknown prefixes, the type will be 'EEG' and the name will not be
+        modified. If False, do not infer types and assume all channels are of
+        type 'EEG'.
+    include : list of str | str
+        Channel names to be included. A str is interpreted as a regular
+        expression. 'exclude' must be empty if include is assigned.
 
     Returns
     -------
@@ -342,20 +387,21 @@ def _read_header(fname, exclude):
     ext = os.path.splitext(fname)[1][1:].lower()
     logger.info('%s file detected' % ext.upper())
     if ext in ('bdf', 'edf'):
-        return _read_edf_header(fname, exclude)
+        return _read_edf_header(fname, exclude, infer_types, include)
     elif ext == 'gdf':
-        return _read_gdf_header(fname, exclude), None
+        return _read_gdf_header(fname, exclude, include), None
     else:
         raise NotImplementedError(
             f'Only GDF, EDF, and BDF files are supported, got {ext}.')
 
 
-def _get_info(fname, stim_channel, eog, misc, exclude, preload):
-    """Extract all the information from the EDF+, BDF or GDF file."""
+def _get_info(fname, stim_channel, eog, misc, exclude, infer_types, preload,
+              include=None):
+    """Extract information from EDF+, BDF or GDF file."""
     eog = eog if eog is not None else []
     misc = misc if misc is not None else []
 
-    edf_info, orig_units = _read_header(fname, exclude)
+    edf_info, orig_units = _read_header(fname, exclude, infer_types, include)
 
     # XXX: `tal_ch_names` to pass to `_check_stim_channel` should be computed
     #      from `edf_info['ch_names']` and `edf_info['tal_idx']` but 'tal_idx'
@@ -365,7 +411,14 @@ def _get_info(fname, stim_channel, eog, misc, exclude, preload):
 
     sel = edf_info['sel']  # selection of channels not excluded
     ch_names = edf_info['ch_names']  # of length len(sel)
-    n_samps = edf_info['n_samps'][sel]
+    if 'ch_types' in edf_info:
+        ch_types = edf_info['ch_types']  # of length len(sel)
+    else:
+        ch_types = [None] * len(sel)
+    if len(sel) == 0:  # only want stim channels
+        n_samps = edf_info['n_samps'][[0]]
+    else:
+        n_samps = edf_info['n_samps'][sel]
     nchan = edf_info['nchan']
     physical_ranges = edf_info['physical_max'] - edf_info['physical_min']
     cals = edf_info['digital_max'] - edf_info['digital_min']
@@ -385,6 +438,8 @@ def _get_info(fname, stim_channel, eog, misc, exclude, preload):
     chs = list()
     pick_mask = np.ones(len(ch_names))
 
+    chs_without_types = list()
+
     for idx, ch_name in enumerate(ch_names):
         chan_info = {}
         chan_info['cal'] = 1.
@@ -397,7 +452,19 @@ def _get_info(fname, stim_channel, eog, misc, exclude, preload):
         chan_info['coord_frame'] = FIFF.FIFFV_COORD_HEAD
         chan_info['coil_type'] = FIFF.FIFFV_COIL_EEG
         chan_info['kind'] = FIFF.FIFFV_EEG_CH
-        chan_info['loc'] = np.zeros(12)
+        # montage can't be stored in EDF so channel locs are unknown:
+        chan_info['loc'] = np.full(12, np.nan)
+
+        # if the edf info contained channel type information
+        # set it now
+        ch_type = ch_types[idx]
+        if ch_type is not None and ch_type in CH_TYPE_MAPPING:
+            chan_info['kind'] = CH_TYPE_MAPPING.get(ch_type)
+            if ch_type not in ['EEG', 'ECOG', 'SEEG', 'DBS']:
+                chan_info['coil_type'] = FIFF.FIFFV_COIL_NONE
+                pick_mask[idx] = False
+        # if user passes in explicit mapping for eog, misc and stim
+        # channels set them here
         if ch_name in eog or idx in eog or idx - nchan in eog:
             chan_info['coil_type'] = FIFF.FIFFV_COIL_NONE
             chan_info['kind'] = FIFF.FIFFV_EOG_CH
@@ -414,10 +481,17 @@ def _get_info(fname, stim_channel, eog, misc, exclude, preload):
             chan_info['ch_name'] = ch_name
             ch_names[idx] = chan_info['ch_name']
             edf_info['units'][idx] = 1
+        elif ch_type not in CH_TYPE_MAPPING:
+            chs_without_types.append(ch_name)
         chs.append(chan_info)
 
-    edf_info['stim_channel_idxs'] = stim_channel_idxs
+    # warn if channel type was not inferable
+    if len(chs_without_types):
+        msg = ('Could not determine channel type of the following channels, '
+               f'they will be set as EEG:\n{", ".join(chs_without_types)}')
+        logger.info(msg)
 
+    edf_info['stim_channel_idxs'] = stim_channel_idxs
     if any(pick_mask):
         picks = [item for item, mask in zip(range(nchan), pick_mask) if mask]
         edf_info['max_samp'] = max_samp = n_samps[picks].max()
@@ -429,8 +503,11 @@ def _get_info(fname, stim_channel, eog, misc, exclude, preload):
 
     not_stim_ch = [x for x in range(n_samps.shape[0])
                    if x not in stim_channel_idxs]
+    if len(not_stim_ch) == 0:  # only loading stim channels
+        not_stim_ch = list(range(len(n_samps)))
     sfreq = np.take(n_samps, not_stim_ch).max() * \
         edf_info['record_length'][1] / edf_info['record_length'][0]
+    del n_samps
     info = _empty_info(sfreq)
     info['meas_date'] = edf_info['meas_date']
     info['chs'] = chs
@@ -443,7 +520,8 @@ def _get_info(fname, stim_channel, eog, misc, exclude, preload):
         pass
     elif all(highpass):
         if highpass[0] == 'NaN':
-            pass  # Placeholder for future use. Highpass set in _empty_info.
+            # Placeholder for future use. Highpass set in _empty_info.
+            pass
         elif highpass[0] == 'DC':
             info['highpass'] = 0.
         else:
@@ -460,10 +538,12 @@ def _get_info(fname, stim_channel, eog, misc, exclude, preload):
     if np.isnan(info['highpass']):
         info['highpass'] = 0.
     if lowpass.size == 0:
-        pass  # Placeholder for future use. Lowpass set in _empty_info.
+        # Placeholder for future use. Lowpass set in _empty_info.
+        pass
     elif all(lowpass):
         if lowpass[0] in ('NaN', '0', '0.0'):
-            pass  # Placeholder for future use. Lowpass set in _empty_info.
+            # Placeholder for future use. Lowpass set in _empty_info.
+            pass
         else:
             info['lowpass'] = float(lowpass[0])
     else:
@@ -474,8 +554,8 @@ def _get_info(fname, stim_channel, eog, misc, exclude, preload):
         info['lowpass'] = info['sfreq'] / 2.
 
     if info['highpass'] > info['lowpass']:
-        warn(f'Highpass cutoff frequency {info["highpass"]} is greater than '
-             f'lowpass cutoff frequency {info["lowpass"]}, '
+        warn(f'Highpass cutoff frequency {info["highpass"]} is greater '
+             f'than lowpass cutoff frequency {info["lowpass"]}, '
              'setting values to 0 and Nyquist.')
         info['highpass'] = 0.
         info['lowpass'] = info['sfreq'] / 2.
@@ -484,6 +564,7 @@ def _get_info(fname, stim_channel, eog, misc, exclude, preload):
     info['description'] = None
     edf_info['nsamples'] = int(edf_info['n_records'] * max_samp)
 
+    info._unlocked = False
     info._update_redundant()
 
     # Later used for reading
@@ -520,7 +601,11 @@ def _edf_str(x):
     return x.decode('latin-1').split('\x00')[0]
 
 
-def _read_edf_header(fname, exclude):
+def _edf_str_num(x):
+    return _edf_str(x).replace(",", ".")
+
+
+def _read_edf_header(fname, exclude, infer_types, include=None):
     """Read header information from EDF+ or BDF file."""
     edf_info = {'events': []}
 
@@ -595,7 +680,7 @@ def _read_edf_header(fname, exclude):
         record_length = float(_edf_str(fid.read(8)))
         record_length = np.array([record_length, 1.])  # in seconds
         if record_length[0] == 0:
-            record_length = record_length[0] = 1.
+            record_length[0] = 1.
             warn('Header information is incorrect for record length. Default '
                  'record length set to 1.\nIt is possible that this file only'
                  ' contains annotations and no signals. In that case, please '
@@ -603,8 +688,31 @@ def _read_edf_header(fname, exclude):
 
         nchan = int(_edf_str(fid.read(4)))
         channels = list(range(nchan))
-        ch_names = [fid.read(16).strip().decode('latin-1') for ch in channels]
-        exclude = _find_exclude_idx(ch_names, exclude)
+
+        # read in 16 byte labels and strip any extra spaces at the end
+        ch_labels = [fid.read(16).strip().decode('latin-1') for _ in channels]
+
+        # get channel names and optionally channel type
+        # EDF specification contains 16 bytes that encode channel names,
+        # optionally prefixed by a string representing channel type separated
+        # by a space
+        if infer_types:
+            ch_types, ch_names = [], []
+            for ch_label in ch_labels:
+                ch_type, ch_name = 'EEG', ch_label  # default to EEG
+                parts = ch_label.split(' ')
+                if len(parts) > 1:
+                    if parts[0].upper() in CH_TYPE_MAPPING:
+                        ch_type = parts[0].upper()
+                        ch_name = ' '.join(parts[1:])
+                        logger.info(f"Channel '{ch_label}' recognized as type "
+                                    f"{ch_type} (renamed to '{ch_name}').")
+                ch_types.append(ch_type)
+                ch_names.append(ch_name)
+        else:
+            ch_types, ch_names = ['EEG'] * nchan, ch_labels
+
+        exclude = _find_exclude_idx(ch_names, exclude, include)
         tal_idx = _find_tal_idx(ch_names)
         exclude = np.concatenate([exclude, tal_idx])
         sel = np.setdiff1d(np.arange(len(ch_names)), exclude)
@@ -615,8 +723,8 @@ def _read_edf_header(fname, exclude):
         for i, unit in enumerate(units):
             if i in exclude:
                 continue
-            # allow both μ (greek mu) and µ (micro symbol) codepoints
-            if unit in ('\u03BCV', '\u00B5V', 'uV'):
+            # allow μ (greek mu), µ (micro symbol) and μ (sjis mu) codepoints
+            if unit in ('\u03BCV', '\u00B5V', '\x83\xCAV', 'uV'):
                 edf_info['units'].append(1e-6)
             elif unit == 'mV':
                 edf_info['units'].append(1e-3)
@@ -632,13 +740,13 @@ def _read_edf_header(fname, exclude):
         orig_units = dict(zip(ch_names, units))
 
         physical_min = np.array(
-            [float(_edf_str(fid.read(8))) for ch in channels])[sel]
+            [float(_edf_str_num(fid.read(8))) for ch in channels])[sel]
         physical_max = np.array(
-            [float(_edf_str(fid.read(8))) for ch in channels])[sel]
+            [float(_edf_str_num(fid.read(8))) for ch in channels])[sel]
         digital_min = np.array(
-            [float(_edf_str(fid.read(8))) for ch in channels])[sel]
+            [float(_edf_str_num(fid.read(8))) for ch in channels])[sel]
         digital_max = np.array(
-            [float(_edf_str(fid.read(8))) for ch in channels])[sel]
+            [float(_edf_str_num(fid.read(8))) for ch in channels])[sel]
         prefiltering = [_edf_str(fid.read(80)).strip() for ch in channels][:-1]
         highpass, lowpass = _parse_prefilter_string(prefiltering)
 
@@ -647,7 +755,7 @@ def _read_edf_header(fname, exclude):
 
         # Populate edf_info
         edf_info.update(
-            ch_names=ch_names, data_offset=header_nbytes,
+            ch_names=ch_names, ch_types=ch_types, data_offset=header_nbytes,
             digital_max=digital_max, digital_min=digital_min,
             highpass=highpass, sel=sel, lowpass=lowpass, meas_date=meas_date,
             n_records=n_records, n_samps=n_samps, nchan=nchan,
@@ -708,7 +816,7 @@ def _check_dtype_byte(types):
     return dtype_np[0], dtype_byte[0]
 
 
-def _read_gdf_header(fname, exclude):
+def _read_gdf_header(fname, exclude, include=None):
     """Read GDF 1.x and GDF 2.x header info."""
     edf_info = dict()
     events = None
@@ -765,7 +873,7 @@ def _read_gdf_header(fname, exclude):
             nchan = np.fromfile(fid, UINT32, 1)[0]
             channels = list(range(nchan))
             ch_names = [_edf_str(fid.read(16)).strip() for ch in channels]
-            exclude = _find_exclude_idx(ch_names, exclude)
+            exclude = _find_exclude_idx(ch_names, exclude, include)
             sel = np.setdiff1d(np.arange(len(ch_names)), exclude)
             fid.seek(80 * len(channels), 1)  # transducer
             units = [_edf_str(fid.read(8)).strip() for ch in channels]
@@ -945,7 +1053,7 @@ def _read_gdf_header(fname, exclude):
             # Channels (variable header)
             channels = list(range(nchan))
             ch_names = [_edf_str(fid.read(16)).strip() for ch in channels]
-            exclude = _find_exclude_idx(ch_names, exclude)
+            exclude = _find_exclude_idx(ch_names, exclude, include)
             sel = np.setdiff1d(np.arange(len(ch_names)), exclude)
 
             fid.seek(80 * len(channels), 1)  # reserved space
@@ -1133,12 +1241,34 @@ def _check_stim_channel(stim_channel, ch_names,
         return stim_channel_idxs, names
 
 
-def _find_exclude_idx(ch_names, exclude):
-    """Find the index of all channels to exclude.
+def _find_exclude_idx(ch_names, exclude, include=None):
+    """Find indices of all channels to exclude.
 
-    If there are several channels called "A" and we want to exclude "A",
-    then add (the index of) all "A" channels to the exclusion list.
+    If there are several channels called "A" and we want to exclude "A", then
+    add (the index of) all "A" channels to the exclusion list.
     """
+    if include:  # find other than include channels
+        if exclude:
+            raise ValueError(
+                "'exclude' must be empty if 'include' is assigned. "
+                f"Got {exclude}.")
+        if isinstance(include, str):  # regex for channel names
+            indices_include = []
+            for idx, ch in enumerate(ch_names):
+                if re.match(include, ch):
+                    indices_include.append(idx)
+            indices = np.setdiff1d(np.arange(len(ch_names)), indices_include)
+            return indices
+        # list of channel names
+        return [idx for idx, ch in enumerate(ch_names) if ch not in include]
+
+    if isinstance(exclude, str):  # regex for channel names
+        indices = []
+        for idx, ch in enumerate(ch_names):
+            if re.match(exclude, ch):
+                indices.append(idx)
+        return indices
+    # list of channel names
     return [idx for idx, ch in enumerate(ch_names) if ch in exclude]
 
 
@@ -1150,8 +1280,9 @@ def _find_tal_idx(ch_names):
 
 
 @fill_doc
-def read_raw_edf(input_fname, eog=None, misc=None,
-                 stim_channel='auto', exclude=(), preload=False, verbose=None):
+def read_raw_edf(input_fname, eog=None, misc=None, stim_channel='auto',
+                 exclude=(), infer_types=False, include=None, preload=False,
+                 verbose=None):
     """Reader function for EDF or EDF+ files.
 
     Parameters
@@ -1171,17 +1302,25 @@ def read_raw_edf(input_fname, eog=None, misc=None,
         'trigger' (case insensitive) are set to STIM. If str (or list of str),
         all channels matching the name(s) are set to STIM. If int (or list of
         ints), channels corresponding to the indices are set to STIM.
-
-        .. warning:: 0.18 does not allow for stim channel synthesis from TAL
-                     channels called 'EDF Annotations' anymore. Instead, TAL
-                     channels are parsed and extracted annotations are stored
-                     in raw.annotations. Use
-                     :func:`mne.events_from_annotations` to obtain events from
-                     these annotations.
-
-    exclude : list of str
+    exclude : list of str | str
         Channel names to exclude. This can help when reading data with
-        different sampling rates to avoid unnecessary resampling.
+        different sampling rates to avoid unnecessary resampling. A str is
+        interpreted as a regular expression.
+    infer_types : bool
+        If True, try to infer channel types from channel labels. If a channel
+        label starts with a known type (such as 'EEG') followed by a space and
+        a name (such as 'Fp1'), the channel type will be set accordingly, and
+        the channel will be renamed to the original label without the prefix.
+        For unknown prefixes, the type will be 'EEG' and the name will not be
+        modified. If False, do not infer types and assume all channels are of
+        type 'EEG'.
+
+        .. versionadded:: 0.24.1
+    include : list of str | str
+        Channel names to be included. A str is interpreted as a regular
+        expression. 'exclude' must be empty if include is assigned.
+
+        .. versionadded:: 1.1
     %(preload)s
     %(verbose)s
 
@@ -1194,6 +1333,7 @@ def read_raw_edf(input_fname, eog=None, misc=None,
     --------
     mne.io.read_raw_bdf : Reader function for BDF files.
     mne.io.read_raw_gdf : Reader function for GDF files.
+    mne.export.export_raw : Export function for EDF files.
 
     Notes
     -----
@@ -1211,20 +1351,46 @@ def read_raw_edf(input_fname, eog=None, misc=None,
     If channels named 'status' or 'trigger' are present, they are considered as
     STIM channels by default. Use func:`mne.find_events` to parse events
     encoded in such analog stim channels.
+
+    The EDF specification allows optional storage of channel types in the
+    prefix of the signal label for each channel. For example, ``EEG Fz``
+    implies that ``Fz`` is an EEG channel and ``MISC E`` would imply ``E`` is
+    a MISC channel. However, there is no standard way of specifying all
+    channel types. MNE-Python will try to infer the channel type, when such a
+    string exists, defaulting to EEG, when there is no prefix or the prefix is
+    not recognized.
+
+    The following prefix strings are mapped to MNE internal types:
+
+        - 'EEG': 'eeg'
+        - 'SEEG': 'seeg'
+        - 'ECOG': 'ecog'
+        - 'DBS': 'dbs'
+        - 'EOG': 'eog'
+        - 'ECG': 'ecg'
+        - 'EMG': 'emg'
+        - 'BIO': 'bio'
+        - 'RESP': 'resp'
+        - 'MISC': 'misc'
+        - 'SAO2': 'bio'
+
+    The EDF specification allows storage of subseconds in measurement date.
+    However, this reader currently sets subseconds to 0 by default.
     """
     input_fname = os.path.abspath(input_fname)
     ext = os.path.splitext(input_fname)[1][1:].lower()
     if ext != 'edf':
-        raise NotImplementedError(
-            'Only EDF files are supported by read_raw_edf, got %s' % (ext,))
+        raise NotImplementedError(f'Only EDF files are supported, got {ext}.')
     return RawEDF(input_fname=input_fname, eog=eog, misc=misc,
-                  stim_channel=stim_channel, exclude=exclude, preload=preload,
+                  stim_channel=stim_channel, exclude=exclude,
+                  infer_types=infer_types, preload=preload, include=include,
                   verbose=verbose)
 
 
 @fill_doc
-def read_raw_bdf(input_fname, eog=None, misc=None,
-                 stim_channel='auto', exclude=(), preload=False, verbose=None):
+def read_raw_bdf(input_fname, eog=None, misc=None, stim_channel='auto',
+                 exclude=(), infer_types=False, include=None, preload=False,
+                 verbose=None):
     """Reader function for BDF files.
 
     Parameters
@@ -1244,17 +1410,25 @@ def read_raw_bdf(input_fname, eog=None, misc=None,
         'trigger' (case insensitive) are set to STIM. If str (or list of str),
         all channels matching the name(s) are set to STIM. If int (or list of
         ints), channels corresponding to the indices are set to STIM.
-
-        .. warning:: 0.18 does not allow for stim channel synthesis from TAL
-                     channels called 'BDF Annotations' anymore. Instead, TAL
-                     channels are parsed and extracted annotations are stored
-                     in raw.annotations. Use
-                     :func:`mne.events_from_annotations` to obtain events from
-                     these annotations.
-
-    exclude : list of str
+    exclude : list of str | str
         Channel names to exclude. This can help when reading data with
-        different sampling rates to avoid unnecessary resampling.
+        different sampling rates to avoid unnecessary resampling. A str is
+        interpreted as a regular expression.
+    infer_types : bool
+        If True, try to infer channel types from channel labels. If a channel
+        label starts with a known type (such as 'EEG') followed by a space and
+        a name (such as 'Fp1'), the channel type will be set accordingly, and
+        the channel will be renamed to the original label without the prefix.
+        For unknown prefixes, the type will be 'EEG' and the name will not be
+        modified. If False, do not infer types and assume all channels are of
+        type 'EEG'.
+
+        .. versionadded:: 0.24.1
+    include : list of str | str
+        Channel names to be included. A str is interpreted as a regular
+        expression. 'exclude' must be empty if include is assigned.
+
+        .. versionadded:: 1.1
     %(preload)s
     %(verbose)s
 
@@ -1307,16 +1481,16 @@ def read_raw_bdf(input_fname, eog=None, misc=None,
     input_fname = os.path.abspath(input_fname)
     ext = os.path.splitext(input_fname)[1][1:].lower()
     if ext != 'bdf':
-        raise NotImplementedError('Only BDF files are supported, got '
-                                  '{}.'.format(ext))
+        raise NotImplementedError(f'Only BDF files are supported, got {ext}.')
     return RawEDF(input_fname=input_fname, eog=eog, misc=misc,
-                  stim_channel=stim_channel, exclude=exclude, preload=preload,
+                  stim_channel=stim_channel, exclude=exclude,
+                  infer_types=infer_types, preload=preload, include=include,
                   verbose=verbose)
 
 
 @fill_doc
-def read_raw_gdf(input_fname, eog=None, misc=None,
-                 stim_channel='auto', exclude=(), preload=False, verbose=None):
+def read_raw_gdf(input_fname, eog=None, misc=None, stim_channel='auto',
+                 exclude=(), include=None, preload=False, verbose=None):
     """Reader function for GDF files.
 
     Parameters
@@ -1336,9 +1510,13 @@ def read_raw_gdf(input_fname, eog=None, misc=None,
         'trigger' (case insensitive) are set to STIM. If str (or list of str),
         all channels matching the name(s) are set to STIM. If int (or list of
         ints), channels corresponding to the indices are set to STIM.
-    exclude : list of str
+    exclude : list of str | str
         Channel names to exclude. This can help when reading data with
-        different sampling rates to avoid unnecessary resampling.
+        different sampling rates to avoid unnecessary resampling. A str is
+        interpreted as a regular expression.
+    include : list of str | str
+        Channel names to be included. A str is interpreted as a regular
+        expression. 'exclude' must be empty if include is assigned.
     %(preload)s
     %(verbose)s
 
@@ -1361,11 +1539,10 @@ def read_raw_gdf(input_fname, eog=None, misc=None,
     input_fname = os.path.abspath(input_fname)
     ext = os.path.splitext(input_fname)[1][1:].lower()
     if ext != 'gdf':
-        raise NotImplementedError('Only GDF files are supported, got '
-                                  '{}.'.format(ext))
+        raise NotImplementedError(f'Only BDF files are supported, got {ext}.')
     return RawGDF(input_fname=input_fname, eog=eog, misc=misc,
                   stim_channel=stim_channel, exclude=exclude, preload=preload,
-                  verbose=verbose)
+                  include=include, verbose=verbose)
 
 
 def _read_annotations_edf(annotations):
@@ -1410,10 +1587,7 @@ def _read_annotations_edf(annotations):
                 tals.extend(np.uint8([this_chan % 256, this_chan // 256])
                             .flatten('F'))
 
-        # use of latin-1 because characters are only encoded for the first 256
-        # code points and utf-8 can triggers an "invalid continuation byte"
-        # error
-        triggers = re.findall(pat, tals.decode('latin-1'))
+        triggers = re.findall(pat, tals.decode('utf8'))
 
     events = []
     offset = 0.
