@@ -6,8 +6,9 @@
 #          Stefan Appelhoff <stefan.appelhoff@mailbox.org>
 #          Joan Massich <mailsik@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
+from contextlib import nullcontext
 from functools import partial
 import os.path as op
 import inspect
@@ -20,16 +21,16 @@ from scipy.io import loadmat
 import pytest
 
 from mne import pick_types, Annotations
+from mne.annotations import events_from_annotations, read_annotations
 from mne.datasets import testing
-from mne.fixes import nullcontext
-from mne.utils import requires_pandas
+from mne.utils import requires_pandas, _record_warnings
 from mne.io import read_raw_edf, read_raw_bdf, read_raw_fif, edf, read_raw_gdf
 from mne.io.tests.test_raw import _test_raw_reader
 from mne.io.edf.edf import (_get_edf_default_event_id, _read_annotations_edf,
                             _read_ch, _parse_prefilter_string, _edf_str,
                             _read_edf_header, _read_header)
 from mne.io.pick import channel_indices_by_type, get_channel_type_constants
-from mne.annotations import events_from_annotations, read_annotations
+from mne.tests.test_annotations import _assert_annotations_equal
 
 td_mark = testing._pytest_mark()
 
@@ -52,12 +53,15 @@ edf_stim_resamp_path = op.join(data_path, 'EDF', 'test_edf_stim_resamp.edf')
 edf_overlap_annot_path = op.join(data_path, 'EDF',
                                  'test_edf_overlapping_annotations.edf')
 edf_reduced = op.join(data_path, 'EDF', 'test_reduced.edf')
+edf_annot_only = op.join(data_path, 'EDF', 'SC4001EC-Hypnogram.edf')
 bdf_stim_channel_path = op.join(data_path, 'BDF', 'test_bdf_stim_channel.bdf')
 bdf_multiple_annotations_path = op.join(data_path, 'BDF',
                                         'multiple_annotation_chans.bdf')
 test_generator_bdf = op.join(data_path, 'BDF', 'test_generator_2.bdf')
 test_generator_edf = op.join(data_path, 'EDF', 'test_generator_2.edf')
 edf_annot_sub_s_path = op.join(data_path, 'EDF', 'subsecond_starttime.edf')
+edf_chtypes_path = op.join(data_path, 'EDF', 'chtypes_edf.edf')
+edf_utf8_annotations = op.join(data_path, 'EDF', 'test_utf8_annotations.edf')
 
 eog = ['REOG', 'LEOG', 'IEOG']
 misc = ['EXG1', 'EXG5', 'EXG8', 'M1', 'M2']
@@ -73,7 +77,7 @@ def test_orig_units():
     assert orig_units['A1'] == 'µV'  # formerly 'uV' edit by _check_orig_units
 
 
-def test_subject_info(tmpdir):
+def test_subject_info(tmp_path):
     """Test exposure of original channel units."""
     raw = read_raw_edf(edf_path)
     assert raw.info['subject_info'] is None  # XXX this is arguably a bug
@@ -82,7 +86,7 @@ def test_subject_info(tmpdir):
     want = {'id': 'X', 'sex': 'X', 'birthday': 'X', 'name': 'X'}
     for key, val in want.items():
         assert edf_info['subject_info'][key] == val, key
-    fname = tmpdir.join('test_raw.fif')
+    fname = tmp_path / 'test_raw.fif'
     raw.save(fname)
     raw = read_raw_fif(fname)
     assert raw.info['subject_info'] is None  # XXX should eventually round-trip
@@ -122,10 +126,10 @@ def test_bdf_data():
 
 
 @testing.requires_testing_data
-def test_bdf_crop_save_stim_channel(tmpdir):
+def test_bdf_crop_save_stim_channel(tmp_path):
     """Test EDF with various sampling rates."""
     raw = read_raw_bdf(bdf_stim_channel_path)
-    raw.save(tmpdir.join('test-raw.fif'), tmin=1.2, tmax=4.0, overwrite=True)
+    raw.save(tmp_path / 'test-raw.fif', tmin=1.2, tmax=4.0, overwrite=True)
 
 
 @testing.requires_testing_data
@@ -141,7 +145,7 @@ def test_edf_others(fname, stim_channel):
         verbose='error')
 
 
-def test_edf_data_broken(tmpdir):
+def test_edf_data_broken(tmp_path):
     """Test edf files."""
     raw = _test_raw_reader(read_raw_edf, input_fname=edf_path,
                            exclude=['Ergo-Left', 'H10'], verbose='error')
@@ -150,7 +154,7 @@ def test_edf_data_broken(tmpdir):
     assert_equal(len(raw.ch_names) + 2, len(raw_py.ch_names))
 
     # Test with number of records not in header (-1).
-    broken_fname = op.join(tmpdir, 'broken.edf')
+    broken_fname = op.join(tmp_path, 'broken.edf')
     with open(edf_path, 'rb') as fid_in:
         fid_in.seek(0, 2)
         n_bytes = fid_in.tell()
@@ -185,7 +189,7 @@ def test_duplicate_channel_labels_edf():
     assert raw.ch_names == EXPECTED_CHANNEL_NAMES
 
 
-def test_parse_annotation(tmpdir):
+def test_parse_annotation(tmp_path):
     """Test parsing the tal channel."""
     # test the parser
     annot = (b'+180\x14Lights off\x14Close door\x14\x00\x00\x00\x00\x00'
@@ -194,15 +198,16 @@ def test_parse_annotation(tmpdir):
              b'+3.14\x1504.20\x14nothing\x14\x00\x00\x00\x00'
              b'+1800.2\x1525.5\x14Apnea\x14\x00\x00\x00\x00\x00\x00\x00'
              b'+123\x14\x14\x00\x00\x00\x00\x00\x00\x00')
-    annot_file = tmpdir.join('annotations.txt')
-    annot_file.write(annot)
+    annot_file = tmp_path / 'annotations.txt'
+    with open(annot_file, "wb") as f:
+        f.write(annot)
 
     annot = [a for a in bytes(annot)]
     annot[1::2] = [a * 256 for a in annot[1::2]]
     tal_channel_A = np.array(list(map(sum, zip(annot[0::2], annot[1::2]))),
                              dtype=np.int64)
 
-    with open(str(annot_file), 'rb') as fid:
+    with open(annot_file, 'rb') as fid:
         # ch_data = np.fromfile(fid, dtype='<i2', count=len(annot))
         tal_channel_B = _read_ch(fid, subtype='EDF', dtype='<i2',
                                  samp=(len(annot) - 1) // 2,
@@ -235,6 +240,31 @@ def test_find_events_backward_compatibility():
     assert_array_equal(events_from_EFA, EXPECTED_EVENTS)
 
 
+@testing.requires_testing_data
+def test_no_data_channels():
+    """Test that we can load with no data channels."""
+    # analog
+    raw = read_raw_edf(edf_path, preload=True)
+    picks = pick_types(raw.info, stim=True)
+    assert list(picks) == [len(raw.ch_names) - 1]
+    stim_data = raw[picks][0]
+    raw = read_raw_edf(edf_path, exclude=raw.ch_names[:-1])
+    stim_data_2 = raw[0][0]
+    assert_array_equal(stim_data, stim_data_2)
+    raw.plot()  # smoke test
+    # annotations
+    raw = read_raw_edf(edf_overlap_annot_path)
+    picks = pick_types(raw.info, stim=True)
+    assert picks.size == 0
+    annot = raw.annotations
+    raw = read_raw_edf(edf_overlap_annot_path, exclude=raw.ch_names)
+    annot_2 = raw.annotations
+    _assert_annotations_equal(annot, annot_2)
+    # only annotations (should warn)
+    with pytest.warns(RuntimeWarning, match='read_annotations'):
+        read_raw_edf(edf_annot_only)
+
+
 @requires_pandas
 @pytest.mark.parametrize('fname', [edf_path, bdf_path])
 def test_to_data_frame(fname):
@@ -247,7 +277,7 @@ def test_to_data_frame(fname):
     _, times = raw[0, :10]
     df = raw.to_data_frame(index='time')
     assert (df.columns == raw.ch_names).all()
-    assert_array_equal(np.round(times * 1e3), df.index.values[:10])
+    assert_array_equal(times, df.index.values[:10])
     df = raw.to_data_frame(index=None, scalings={'eeg': 1e13})
     assert 'time' in df.columns
     assert_array_equal(df.values[:, 1], raw._data[0] * 1e13)
@@ -256,9 +286,9 @@ def test_to_data_frame(fname):
 def test_read_raw_edf_stim_channel_input_parameters():
     """Test edf raw reader deprecation."""
     _MSG = "`read_raw_edf` is not supposed to trigger a deprecation warning"
-    with pytest.warns(None) as recwarn:
+    with _record_warnings() as recwarn:
         read_raw_edf(edf_path)
-    assert all([w.category != DeprecationWarning for w in recwarn.list]), _MSG
+    assert all(w.category != DeprecationWarning for w in recwarn), _MSG
 
     for invalid_stim_parameter in ['EDF Annotations', 'BDF Annotations']:
         with pytest.raises(ValueError,
@@ -266,14 +296,7 @@ def test_read_raw_edf_stim_channel_input_parameters():
             read_raw_edf(edf_path, stim_channel=invalid_stim_parameter)
 
 
-def _assert_annotations_equal(a, b):
-    assert_array_equal(a.onset, b.onset)
-    assert_array_equal(a.duration, b.duration)
-    assert_array_equal(a.description, b.description)
-    assert a.orig_time == b.orig_time
-
-
-def test_read_annot(tmpdir):
+def test_read_annot(tmp_path):
     """Test parsing the tal channel."""
     EXPECTED_ANNOTATIONS = [[180.0, 0, 'Lights off'], [180.0, 0, 'Close door'],
                             [180.0, 0, 'Lights off'], [180.0, 0, 'Close door'],
@@ -294,8 +317,9 @@ def test_read_annot(tmpdir):
              b'+3.14\x1504.20\x14nothing\x14\x00\x00\x00\x00'
              b'+1800.2\x1525.5\x14Apnea\x14\x00\x00\x00\x00\x00\x00\x00'
              b'+123\x14\x14\x00\x00\x00\x00\x00\x00\x00')
-    annot_file = tmpdir.join('annotations.txt')
-    annot_file.write(annot)
+    annot_file = tmp_path / 'annotations.txt'
+    with open(annot_file, "wb") as f:
+        f.write(annot)
 
     onset, duration, desc = _read_annotations_edf(annotations=str(annot_file))
     annotation = Annotations(onset=onset, duration=duration, description=desc,
@@ -303,7 +327,7 @@ def test_read_annot(tmpdir):
     _assert_annotations_equal(annotation, EXPECTED_ANNOTATIONS)
 
     # Now test when reading from buffer of data
-    with open(str(annot_file), 'rb') as fid:
+    with open(annot_file, 'rb') as fid:
         ch_data = np.fromfile(fid, dtype='<i2', count=len(annot))
     onset, duration, desc = _read_annotations_edf([ch_data])
     annotation = Annotations(onset=onset, duration=duration, description=desc,
@@ -319,6 +343,14 @@ def test_read_annotations(fname, recwarn):
     assert len(annot.onset) == 2
 
 
+@testing.requires_testing_data
+def test_read_utf8_annotations():
+    """Test if UTF8 annotations can be read."""
+    raw = read_raw_edf(edf_utf8_annotations)
+    assert raw.annotations[0]['description'] == 'RECORD START'
+    assert raw.annotations[1]['description'] == '仰卧'
+
+
 def test_edf_prefilter_parse():
     """Test prefilter strings from header are parsed correctly."""
     prefilter_basic = ["HP: 0Hz LP: 0Hz"]
@@ -409,7 +441,6 @@ def test_edf_lowpass_zero():
     """Test if a lowpass filter of 0Hz is mapped to the Nyquist frequency."""
     raw = read_raw_edf(edf_stim_resamp_path)
     assert raw.ch_names[100] == 'EEG LDAMT_01-REF'
-    assert len(raw.ch_names[100]) > 15
     assert_allclose(raw.info["lowpass"], raw.info["sfreq"] / 2)
 
 
@@ -420,7 +451,7 @@ def test_edf_annot_sub_s_onset():
     assert_allclose(raw.annotations.onset, [1.951172, 3.492188])
 
 
-def test_invalid_date(tmpdir):
+def test_invalid_date(tmp_path):
     """Test handling of invalid date in EDF header."""
     with open(edf_path, 'rb') as f:  # read valid test file
         edf = bytearray(f.read())
@@ -433,7 +464,7 @@ def test_invalid_date(tmpdir):
     # one wrong: no warning
     edf[101:104] = b'FEB'
     assert edf[172] == ord('4')
-    fname = op.join(str(tmpdir), "temp.edf")
+    fname = op.join(str(tmp_path), "temp.edf")
     with open(fname, "wb") as f:
         f.write(edf)
     read_raw_edf(fname)
@@ -509,6 +540,72 @@ def test_hp_lp_reversed(fname, lo, hi, warns, monkeypatch):
 def test_degenerate():
     """Test checking of some bad inputs."""
     for func in (read_raw_edf, read_raw_bdf, read_raw_gdf,
-                 partial(_read_header, exclude=())):
+                 partial(_read_header, exclude=(), infer_types=False)):
         with pytest.raises(NotImplementedError, match='Only.*txt.*'):
             func(edf_txt_stim_channel_path)
+
+
+def test_exclude():
+    """Test exclude parameter."""
+    exclude = ["I1", "I2", "I3", "I4"]  # list of excluded channels
+
+    raw = read_raw_edf(edf_path, exclude=["I1", "I2", "I3", "I4"])
+    for ch in exclude:
+        assert ch not in raw.ch_names
+
+    raw = read_raw_edf(edf_path, exclude="I[1-4]")
+    for ch in exclude:
+        assert ch not in raw.ch_names
+
+
+def test_include():
+    """Test include parameter."""
+    raw = read_raw_edf(edf_path, include=["I1", "I2"])
+    assert sorted(raw.ch_names) == ["I1", "I2"]
+
+    raw = read_raw_edf(edf_path, include="I[1-4]")
+    assert sorted(raw.ch_names) == ["I1", "I2", "I3", "I4"]
+
+    with pytest.raises(ValueError) as e:
+        raw = read_raw_edf(edf_path, include=["I1", "I2"], exclude="I[1-4]")
+        assert str(e.value) == "'exclude' must be empty" \
+            "if 'include' is assigned."
+
+
+@testing.requires_testing_data
+def test_ch_types():
+    """Test reading of channel types from EDF channel label."""
+    raw = read_raw_edf(edf_chtypes_path)  # infer_types=False
+
+    labels = ['EEG Fp1-Ref', 'EEG Fp2-Ref', 'EEG F3-Ref', 'EEG F4-Ref',
+              'EEG C3-Ref', 'EEG C4-Ref', 'EEG P3-Ref', 'EEG P4-Ref',
+              'EEG O1-Ref', 'EEG O2-Ref', 'EEG F7-Ref', 'EEG F8-Ref',
+              'EEG T7-Ref', 'EEG T8-Ref', 'EEG P7-Ref', 'EEG P8-Ref',
+              'EEG Fz-Ref', 'EEG Cz-Ref', 'EEG Pz-Ref', 'POL E', 'POL PG1',
+              'POL PG2', 'EEG A1-Ref', 'EEG A2-Ref', 'POL T1', 'POL T2',
+              'ECG ECG1', 'ECG ECG2', 'EEG F9-Ref', 'EEG T9-Ref', 'EEG P9-Ref',
+              'EEG F10-Ref', 'EEG T10-Ref', 'EEG P10-Ref', 'SaO2 X9',
+              'SaO2 X10', 'POL DC01', 'POL DC02', 'POL DC03', 'POL DC04',
+              'POL $A1', 'POL $A2']
+
+    # by default all types are 'eeg'
+    assert all(t == 'eeg' for t in raw.get_channel_types())
+    assert raw.ch_names == labels
+
+    raw = read_raw_edf(edf_chtypes_path, infer_types=True)
+
+    labels = ['Fp1-Ref', 'Fp2-Ref', 'F3-Ref', 'F4-Ref', 'C3-Ref', 'C4-Ref',
+              'P3-Ref', 'P4-Ref', 'O1-Ref', 'O2-Ref', 'F7-Ref', 'F8-Ref',
+              'T7-Ref', 'T8-Ref', 'P7-Ref', 'P8-Ref', 'Fz-Ref', 'Cz-Ref',
+              'Pz-Ref', 'POL E', 'POL PG1', 'POL PG2', 'A1-Ref', 'A2-Ref',
+              'POL T1', 'POL T2', 'ECG1', 'ECG2', 'F9-Ref', 'T9-Ref', 'P9-Ref',
+              'F10-Ref', 'T10-Ref', 'P10-Ref', 'X9', 'X10', 'POL DC01',
+              'POL DC02', 'POL DC03', 'POL DC04', 'POL $A1', 'POL $A2']
+    types = ['eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg',
+             'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg',
+             'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'ecg',
+             'ecg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'bio', 'bio',
+             'eeg', 'eeg', 'eeg', 'eeg', 'eeg', 'eeg']
+
+    assert raw.get_channel_types() == types
+    assert raw.ch_names == labels
