@@ -2,13 +2,11 @@
 #         Mikolaj Magnuski <mmagnuski@swps.edu.pl>
 #         Stefan Appelhoff <stefan.appelhoff@mailbox.org>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 from copy import deepcopy
-from distutils.version import LooseVersion
 import os.path as op
 import shutil
-from unittest import SkipTest
 
 import numpy as np
 from numpy.testing import (assert_array_equal, assert_array_almost_equal,
@@ -17,12 +15,13 @@ import pytest
 from scipy import io
 
 from mne import write_events, read_epochs_eeglab
+from mne.channels import read_custom_montage
 from mne.io import read_raw_eeglab
+from mne.io.eeglab.eeglab import _get_montage_information, _dol_to_lod
 from mne.io.tests.test_raw import _test_raw_reader
 from mne.datasets import testing
-from mne.utils import check_version
+from mne.utils import Bunch
 from mne.annotations import events_from_annotations, read_annotations
-from mne.io.eeglab.tests._utils import _read_eeglab_montage
 
 base_dir = op.join(testing.data_path(download=False), 'EEGLAB')
 
@@ -34,6 +33,7 @@ epochs_fname_onefile_mat = op.join(base_dir, 'test_epochs_onefile.set')
 raw_mat_fnames = [raw_fname_mat, raw_fname_onefile_mat]
 epochs_mat_fnames = [epochs_fname_mat, epochs_fname_onefile_mat]
 raw_fname_chanloc = op.join(base_dir, 'test_raw_chanloc.set')
+raw_fname_chanloc_fids = op.join(base_dir, 'test_raw_chanloc_fids.set')
 raw_fname_2021 = op.join(base_dir, 'test_raw_2021.set')
 raw_fname_h5 = op.join(base_dir, 'test_raw_h5.set')
 raw_fname_onefile_h5 = op.join(base_dir, 'test_raw_onefile_h5.set')
@@ -45,18 +45,18 @@ epochs_h5_fnames = [epochs_fname_h5, epochs_fname_onefile_h5]
 montage_path = op.join(base_dir, 'test_chans.locs')
 
 
-needs_h5 = pytest.mark.skipif(not check_version('h5py'), reason='Needs h5py')
+pymatreader = pytest.importorskip('pymatreader')  # module-level
 
 
 @testing.requires_testing_data
 @pytest.mark.parametrize('fname', [
     raw_fname_mat,
-    pytest.param(raw_fname_h5, marks=needs_h5),
+    raw_fname_h5,
     raw_fname_chanloc,
 ], ids=op.basename)
 def test_io_set_raw(fname):
     """Test importing EEGLAB .set files."""
-    montage = _read_eeglab_montage(montage_path)
+    montage = read_custom_montage(montage_path)
     montage.ch_names = [
         'EEG {0:03d}'.format(ii) for ii in range(len(montage.ch_names))
     ]
@@ -101,14 +101,14 @@ def test_io_set_raw(fname):
 
 
 @testing.requires_testing_data
-def test_io_set_raw_more(tmpdir):
+def test_io_set_raw_more(tmp_path):
     """Test importing EEGLAB .set files."""
-    tmpdir = str(tmpdir)
+    tmp_path = str(tmp_path)
     eeg = io.loadmat(raw_fname_mat, struct_as_record=False,
                      squeeze_me=True)['EEG']
 
     # test reading file with one event (read old version)
-    negative_latency_fname = op.join(tmpdir, 'test_negative_latency.set')
+    negative_latency_fname = op.join(tmp_path, 'test_negative_latency.set')
     evnts = deepcopy(eeg.event[0])
     evnts.latency = 0
     io.savemat(negative_latency_fname,
@@ -137,7 +137,7 @@ def test_io_set_raw_more(tmpdir):
             read_raw_eeglab(input_fname=negative_latency_fname, preload=True)
 
     # test overlapping events
-    overlap_fname = op.join(tmpdir, 'test_overlap_event.set')
+    overlap_fname = op.join(tmp_path, 'test_overlap_event.set')
     io.savemat(overlap_fname,
                {'EEG': {'trials': eeg.trials, 'srate': eeg.srate,
                         'nbchan': eeg.nbchan, 'data': 'test_overlap_event.fdt',
@@ -150,7 +150,7 @@ def test_io_set_raw_more(tmpdir):
     read_raw_eeglab(input_fname=overlap_fname, preload=True)
 
     # test reading file with empty event durations
-    empty_dur_fname = op.join(tmpdir, 'test_empty_durations.set')
+    empty_dur_fname = op.join(tmp_path, 'test_empty_durations.set')
     evnts = deepcopy(eeg.event)
     for ev in evnts:
         ev.duration = np.array([], dtype='float')
@@ -179,7 +179,7 @@ def test_io_set_raw_more(tmpdir):
         read_raw_eeglab(input_fname=overlap_fname, preload=True)
 
     # raise error when both EEG.data and fdt name from set are wrong
-    overlap_fname = op.join(tmpdir, 'test_ovrlap_event.set')
+    overlap_fname = op.join(tmp_path, 'test_ovrlap_event.set')
     io.savemat(overlap_fname,
                {'EEG': {'trials': eeg.trials, 'srate': eeg.srate,
                         'nbchan': eeg.nbchan, 'data': 'test_overla_event.fdt',
@@ -191,7 +191,7 @@ def test_io_set_raw_more(tmpdir):
         read_raw_eeglab(input_fname=overlap_fname, preload=True)
 
     # test reading file with one channel
-    one_chan_fname = op.join(tmpdir, 'test_one_channel.set')
+    one_chan_fname = op.join(tmp_path, 'test_one_channel.set')
     io.savemat(one_chan_fname,
                {'EEG': {'trials': eeg.trials, 'srate': eeg.srate,
                         'nbchan': 1, 'data': np.random.random((1, 3)),
@@ -219,14 +219,9 @@ def test_io_set_raw_more(tmpdir):
     # SciPy writing bug that pops up sometimes:
     # nopos_chanlocs = np.array(chanlocs[['labels', 'Z']])
 
-    if LooseVersion(np.__version__) == '1.14.0':
-        # There is a bug in 1.14.0 (or maybe with SciPy 1.0.0?) that causes
-        # this write to fail!
-        raise SkipTest('Need to fix bug in NumPy 1.14.0!')
-
     # test reading channel names but not positions when there is no X (only Z)
     # field in the EEG.chanlocs structure
-    nopos_fname = op.join(tmpdir, 'test_no_chanpos.set')
+    nopos_fname = op.join(tmp_path, 'test_no_chanpos.set')
     io.savemat(nopos_fname,
                {'EEG': {'trials': eeg.trials, 'srate': eeg.srate, 'nbchan': 3,
                         'data': np.random.random((3, 2)), 'epoch': eeg.epoch,
@@ -247,7 +242,7 @@ def test_io_set_raw_more(tmpdir):
 @testing.requires_testing_data
 @pytest.mark.parametrize('fnames', [
     epochs_mat_fnames,
-    pytest.param(epochs_h5_fnames, marks=[needs_h5, pytest.mark.slowtest]),
+    pytest.param(epochs_h5_fnames, marks=[pytest.mark.slowtest]),
 ])
 def test_io_set_epochs(fnames):
     """Test importing EEGLAB .set epochs files."""
@@ -262,14 +257,14 @@ def test_io_set_epochs(fnames):
 
 
 @testing.requires_testing_data
-def test_io_set_epochs_events(tmpdir):
+def test_io_set_epochs_events(tmp_path):
     """Test different combinations of events and event_ids."""
-    tmpdir = str(tmpdir)
-    out_fname = op.join(tmpdir, 'test-eve.fif')
+    tmp_path = str(tmp_path)
+    out_fname = op.join(tmp_path, 'test-eve.fif')
     events = np.array([[4, 0, 1], [12, 0, 2], [20, 0, 3], [26, 0, 3]])
     write_events(out_fname, events)
     event_id = {'S255/S8': 1, 'S8': 2, 'S255/S9': 3}
-    out_fname = op.join(tmpdir, 'test-eve.fif')
+    out_fname = op.join(tmp_path, 'test-eve.fif')
     epochs = read_epochs_eeglab(epochs_fname_mat, events, event_id)
     assert_equal(len(epochs.events), 4)
     assert epochs.preload
@@ -282,14 +277,14 @@ def test_io_set_epochs_events(tmpdir):
 
 
 @testing.requires_testing_data
-def test_degenerate(tmpdir):
+def test_degenerate(tmp_path):
     """Test some degenerate conditions."""
     # test if .dat file raises an error
-    tmpdir = str(tmpdir)
+    tmp_path = str(tmp_path)
     eeg = io.loadmat(epochs_fname_mat, struct_as_record=False,
                      squeeze_me=True)['EEG']
     eeg.data = 'epochs_fname.dat'
-    bad_epochs_fname = op.join(tmpdir, 'test_epochs.set')
+    bad_epochs_fname = op.join(tmp_path, 'test_epochs.set')
     io.savemat(bad_epochs_fname,
                {'EEG': {'trials': eeg.trials, 'srate': eeg.srate,
                         'nbchan': eeg.nbchan, 'data': eeg.data,
@@ -297,7 +292,7 @@ def test_degenerate(tmpdir):
                         'chanlocs': eeg.chanlocs, 'pnts': eeg.pnts}},
                appendmat=False, oned_as='row')
     shutil.copyfile(op.join(base_dir, 'test_epochs.fdt'),
-                    op.join(tmpdir, 'test_epochs.dat'))
+                    op.join(tmp_path, 'test_epochs.dat'))
     with pytest.warns(RuntimeWarning, match='multiple events'):
         pytest.raises(NotImplementedError, read_epochs_eeglab,
                       bad_epochs_fname)
@@ -339,7 +334,6 @@ def test_eeglab_read_annotations():
 @testing.requires_testing_data
 def test_eeglab_event_from_annot():
     """Test all forms of obtaining annotations."""
-    base_dir = op.join(testing.data_path(download=False), 'EEGLAB')
     raw_fname_mat = op.join(base_dir, 'test_raw.set')
     raw_fname = raw_fname_mat
     event_id = {'rt': 1, 'square': 2}
@@ -358,7 +352,7 @@ def _assert_array_allclose_nan(left, right):
 
 
 @pytest.fixture(scope='session')
-def one_chanpos_fname(tmpdir_factory):
+def one_chanpos_fname(tmp_path_factory):
     """Test file with 3 channels to exercise EEGLAB reader.
 
     File characteristics
@@ -369,15 +363,15 @@ def one_chanpos_fname(tmpdir_factory):
     Notes from when this code was factorized:
     # test reading file with one event (read old version)
     """
-    fname = str(tmpdir_factory.mktemp('data').join('test_chanpos.set'))
+    fname = str(tmp_path_factory.mktemp('data') / 'test_chanpos.set')
     file_conent = dict(EEG={
         'trials': 1, 'nbchan': 3, 'pnts': 3, 'epoch': [], 'event': [],
         'srate': 128, 'times': np.array([0., 0.1, 0.2]),
         'data': np.empty([3, 3]),
         'chanlocs': np.array(
             [(b'F3', 1., 4., 7.),
-             (b'unknown', 2., 5., 8.),
-             (b'FPz', np.nan, np.nan, np.nan)],
+             (b'unknown', np.nan, np.nan, np.nan),
+             (b'FPz', 2., 5., 8.)],
             dtype=[('labels', 'S10'), ('X', 'f8'), ('Y', 'f8'), ('Z', 'f8')]
         )
     })
@@ -394,8 +388,8 @@ def test_position_information(one_chanpos_fname):
     nan = np.nan
     EXPECTED_LOCATIONS_FROM_FILE = np.array([
         [-4.,  1.,  7.,  0.,  0.,  0., nan, nan, nan, nan, nan, nan],
-        [-5.,  2.,  8.,  0.,  0.,  0., nan, nan, nan, nan, nan, nan],
         [nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan, nan],
+        [-5.,  2.,  8.,  0.,  0.,  0., nan, nan, nan, nan, nan, nan],
     ])
 
     EXPECTED_LOCATIONS_FROM_MONTAGE = np.array([
@@ -420,9 +414,6 @@ def test_position_information(one_chanpos_fname):
     _assert_array_allclose_nan(np.array([ch['loc'] for ch in raw.info['chs']]),
                                EXPECTED_LOCATIONS_FROM_MONTAGE)
 
-    _assert_array_allclose_nan(np.array([ch['loc'] for ch in raw.info['chs']]),
-                               EXPECTED_LOCATIONS_FROM_MONTAGE)
-
 
 @testing.requires_testing_data
 def test_io_set_raw_2021():
@@ -437,3 +428,41 @@ def test_read_single_epoch():
     """Test reading raw set file as an Epochs instance."""
     with pytest.raises(ValueError, match='trials less than 2'):
         read_epochs_eeglab(raw_fname_mat)
+
+
+@testing.requires_testing_data
+def test_get_montage_info_with_ch_type():
+    """Test that the channel types are properly returned."""
+    mat = pymatreader.read_mat(raw_fname_onefile_mat, uint16_codec=None)
+    n = len(mat['EEG']['chanlocs']['labels'])
+    mat['EEG']['chanlocs']['type'] = ['eeg'] * (n - 2) + ['eog'] + ['stim']
+    mat['EEG']['chanlocs'] = _dol_to_lod(mat['EEG']['chanlocs'])
+    mat['EEG'] = Bunch(**mat['EEG'])
+    ch_names, ch_types, montage = _get_montage_information(mat['EEG'], False)
+    assert len(ch_names) == len(ch_types) == n
+    assert ch_types == ['eeg'] * (n - 2) + ['eog'] + ['stim']
+    assert montage is None
+
+    # test unknown type warning
+    mat = pymatreader.read_mat(raw_fname_onefile_mat, uint16_codec=None)
+    n = len(mat['EEG']['chanlocs']['labels'])
+    mat['EEG']['chanlocs']['type'] = ['eeg'] * (n - 2) + ['eog'] + ['unknown']
+    mat['EEG']['chanlocs'] = _dol_to_lod(mat['EEG']['chanlocs'])
+    mat['EEG'] = Bunch(**mat['EEG'])
+    with pytest.warns(RuntimeWarning, match='Unknown types found'):
+        ch_names, ch_types, montage = \
+            _get_montage_information(mat['EEG'], False)
+
+
+@testing.requires_testing_data
+def test_fidsposition_information():
+    """Test reading file with 3 fiducial locations."""
+    raw = read_raw_eeglab(raw_fname_chanloc_fids)
+    montage = raw.get_montage()
+    pos = montage.get_positions()
+    assert pos['nasion'] is not None
+    assert pos['lpa'] is not None
+    assert pos['rpa'] is not None
+    assert len(pos['nasion']) == 3
+    assert len(pos['lpa']) == 3
+    assert len(pos['rpa']) == 3
