@@ -3,7 +3,7 @@
 #          Matti Hämäläinen <msh@nmr.mgh.harvard.edu>
 #          Martin Luessi <mluessi@nmr.mgh.harvard.edu>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 from copy import deepcopy
 import re
@@ -12,7 +12,7 @@ import numpy as np
 
 from .constants import FIFF
 from ..utils import (logger, verbose, _validate_type, fill_doc, _ensure_int,
-                     _check_option)
+                     _check_option, warn)
 
 
 def get_channel_type_constants(include_defaults=False):
@@ -169,13 +169,13 @@ _second_rules = {
 }
 
 
+@fill_doc
 def channel_type(info, idx):
     """Get channel type.
 
     Parameters
     ----------
-    info : instance of Info
-        A measurement info object.
+    %(info_not_none)s
     idx : int
         Index of channel.
 
@@ -366,6 +366,7 @@ def _check_info_exclude(info, exclude):
     return exclude
 
 
+@fill_doc
 def pick_types(info, meg=False, eeg=False, stim=False, eog=False, ecg=False,
                emg=False, ref_meg='auto', misc=False, resp=False, chpi=False,
                exci=False, ias=False, syst=False, seeg=False, dipole=False,
@@ -375,8 +376,7 @@ def pick_types(info, meg=False, eeg=False, stim=False, eog=False, ecg=False,
 
     Parameters
     ----------
-    info : dict
-        The measurement info.
+    %(info_not_none)s
     meg : bool | str
         If True include MEG channels. If string it can be 'mag', 'grad',
         'planar1' or 'planar2' to select only magnetometers, all gradiometers,
@@ -514,8 +514,7 @@ def pick_info(info, sel=(), copy=True, verbose=None):
 
     Parameters
     ----------
-    info : dict
-        Info structure from evoked or raw data.
+    %(info_not_none)s
     sel : list of int | None
         Indices of channels to include. If None, all channels
         are included.
@@ -550,11 +549,12 @@ def pick_info(info, sel=(), copy=True, verbose=None):
             logger.info('Removing %d compensators from info because '
                         'not all compensation channels were picked.'
                         % (len(info['comps']),))
-            info['comps'] = []
-    info['chs'] = [info['chs'][k] for k in sel]
+            with info._unlock():
+                info['comps'] = []
+    with info._unlock():
+        info['chs'] = [info['chs'][k] for k in sel]
     info._update_redundant()
     info['bads'] = [ch for ch in info['bads'] if ch in info['ch_names']]
-
     if 'comps' in info:
         comps = deepcopy(info['comps'])
         for c in comps:
@@ -566,8 +566,10 @@ def pick_info(info, sel=(), copy=True, verbose=None):
             c['data']['nrow'] = len(row_names)
             c['data']['row_names'] = row_names
             c['data']['data'] = c['data']['data'][row_idx]
-        info['comps'] = comps
+        with info._unlock():
+            info['comps'] = comps
     info._check_consistency()
+
     return info
 
 
@@ -696,7 +698,8 @@ def pick_channels_forward(orig, include=[], exclude=[], ordered=False,
     fwd['sol']['row_names'] = ch_names
 
     # Pick the appropriate channel names from the info-dict using sel_info
-    fwd['info']['chs'] = [fwd['info']['chs'][k] for k in sel_info]
+    with fwd['info']._unlock():
+        fwd['info']['chs'] = [fwd['info']['chs'][k] for k in sel_info]
     fwd['info']._update_redundant()
     fwd['info']['bads'] = [b for b in fwd['info']['bads'] if b in ch_names]
 
@@ -759,8 +762,7 @@ def channel_indices_by_type(info, picks=None):
 
     Parameters
     ----------
-    info : instance of Info
-        A measurement info object.
+    %(info_not_none)s
     %(picks_all)s
 
     Returns
@@ -844,13 +846,13 @@ def _mag_grad_dependent(info):
                for ph in info.get('proc_history', []))
 
 
+@fill_doc
 def _contains_ch_type(info, ch_type):
     """Check whether a certain channel type is in an info object.
 
     Parameters
     ----------
-    info : instance of Info
-        The measurement information.
+    %(info_not_none)s
     ch_type : str
         the channel type to be checked for
 
@@ -873,13 +875,13 @@ def _contains_ch_type(info, ch_type):
                for ii in range(info['nchan']))
 
 
+@fill_doc
 def _picks_by_type(info, meg_combined=False, ref_meg=False, exclude='bads'):
     """Get data channel indices as separate list of tuples.
 
     Parameters
     ----------
-    info : instance of mne.measuerment_info.Info
-        The info.
+    %(info_not_none)s
     meg_combined : bool | 'auto'
         Whether to return combined picks for grad and mag.
         Can be 'auto' to choose based on Maxwell filtering status.
@@ -1106,14 +1108,13 @@ def _picks_str_to_idx(info, picks, exclude, with_ref_meg, return_kind,
     # second: match all to channel names
     #
 
-    bad_name = None
+    bad_names = []
     picks_name = list()
     for pick in picks:
         try:
             picks_name.append(info['ch_names'].index(pick))
         except ValueError:
-            bad_name = pick
-            break
+            bad_names.append(pick)
 
     #
     # third: match all to types
@@ -1160,7 +1161,7 @@ def _picks_str_to_idx(info, picks, exclude, with_ref_meg, return_kind,
                 'picks (%s) could not be interpreted as '
                 'channel names (no channel "%s"), channel types (no '
                 'type "%s"), or a generic type (just "all" or "data")'
-                % (repr(orig_picks) + extra_repr, bad_name, bad_type))
+                % (repr(orig_picks) + extra_repr, str(bad_names), bad_type))
         picks = np.array([], int)
     elif sum(any_found) > 1:
         raise RuntimeError('Some channel names are ambiguously equivalent to '
@@ -1168,8 +1169,13 @@ def _picks_str_to_idx(info, picks, exclude, with_ref_meg, return_kind,
                            'picks for these')
     else:
         picks = np.array(all_picks[np.where(any_found)[0][0]])
+
+    picked_ch_type_or_generic = not len(picks_name)
+    if len(bad_names) > 0 and not picked_ch_type_or_generic:
+        warn(f'Channel(s) {bad_names} could not be picked, because '
+             'they are not present in the info instance.')
+
     if return_kind:
-        picked_ch_type_or_generic = not len(picks_name)
         return picks, picked_ch_type_or_generic
     return picks
 
@@ -1198,4 +1204,7 @@ def _get_channel_types(info, picks=None, unique=False, only_data_chs=False):
     if only_data_chs:
         ch_types = [ch_type for ch_type in ch_types
                     if ch_type in _DATA_CH_TYPES_SPLIT]
-    return set(ch_types) if unique is True else ch_types
+    if unique:
+        # set does not preserve order but dict does, so let's just use it
+        ch_types = list({k: k for k in ch_types}.keys())
+    return ch_types
