@@ -3,7 +3,7 @@
 # Authors: MNE Developers
 #          Stefan Appelhoff <stefan.appelhoff@mailbox.org>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 from contextlib import redirect_stdout
 from io import StringIO
@@ -18,19 +18,22 @@ import numpy as np
 from numpy.testing import (assert_allclose, assert_array_almost_equal,
                            assert_array_equal, assert_array_less)
 
+import mne
 from mne import concatenate_raws, create_info, Annotations, pick_types
 from mne.datasets import testing
-from mne.externals.h5io import read_hdf5, write_hdf5
 from mne.io import read_raw_fif, RawArray, BaseRaw, Info, _writing_info_hdf5
 from mne.io.base import _get_scaling
 from mne.utils import (_TempDir, catch_logging, _raw_annot, _stamp_to_dt,
                        object_diff, check_version, requires_pandas,
-                       _check_eeglabio_installed)
+                       _import_h5io_funcs)
 from mne.io.meas_info import _get_valid_units
 from mne.io._digitization import DigPoint
 from mne.io.proj import Projection
 from mne.io.utils import _mult_cal_one
-from mne.io import read_raw_eeglab
+from mne.io.constants import FIFF
+
+raw_fname = op.join(op.dirname(__file__), '..', '..', 'io', 'tests',
+                    'data', 'test_raw.fif')
 
 
 def assert_named_constants(info):
@@ -129,6 +132,10 @@ def _test_raw_reader(reader, test_preloading=True, test_kwargs=True,
             meg, atol = 'grad', 1e-24
         elif 'mag' in raw:
             meg, atol = 'mag', 1e-24
+        elif 'hbo' in raw:
+            fnirs, atol = 'hbo', 1e-10
+        elif 'hbr' in raw:
+            fnirs, atol = 'hbr', 1e-10
         else:
             assert 'fnirs_cw_amplitude' in raw, 'New channel type necessary?'
             fnirs, atol = 'fnirs_cw_amplitude', 1e-10
@@ -136,7 +143,7 @@ def _test_raw_reader(reader, test_preloading=True, test_kwargs=True,
             other_raw.info, meg=meg, eeg=eeg, fnirs=fnirs)
         col_names = [other_raw.ch_names[pick] for pick in picks]
         proj = np.ones((1, len(picks)))
-        proj /= proj.shape[1]
+        proj /= np.sqrt(proj.shape[1])
         proj = Projection(
             data=dict(data=proj, nrow=1, row_names=None,
                       col_names=col_names, ncol=len(picks)),
@@ -201,15 +208,18 @@ def _test_raw_reader(reader, test_preloading=True, test_kwargs=True,
         # ranks should all be reduced by 1
         if test_rank == 'less':
             cmp = np.less
-        else:
+        elif test_rank is False:
+            cmp = None
+        else:  # anything else is like True or 'equal'
+            assert test_rank is True or test_rank == 'equal', test_rank
             cmp = np.equal
         rank_load_apply_get = np.linalg.matrix_rank(data_load_apply_get)
         rank_apply_get = np.linalg.matrix_rank(data_apply_get)
         rank_apply_load_get = np.linalg.matrix_rank(data_apply_load_get)
-        rank_apply_load_get = np.linalg.matrix_rank(data_apply_load_get)
-        assert cmp(rank_load_apply_get, len(col_names) - 1)
-        assert cmp(rank_apply_get, len(col_names) - 1)
-        assert cmp(rank_apply_load_get, len(col_names) - 1)
+        if cmp is not None:
+            assert cmp(rank_load_apply_get, len(col_names) - 1)
+            assert cmp(rank_apply_get, len(col_names) - 1)
+            assert cmp(rank_apply_load_get, len(col_names) - 1)
         # and they should all match
         t_kw = dict(
             atol=atol, err_msg='before != after, likely _mult_cal_one prob')
@@ -228,13 +238,21 @@ def _test_raw_reader(reader, test_preloading=True, test_kwargs=True,
             this_proj = other_raw.info['projs'][0]['data']
             assert this_proj['col_names'] == col_names
             assert this_proj['data'].shape == proj['data']['data'].shape
+            assert_allclose(
+                np.linalg.norm(proj['data']['data']), 1., atol=1e-6)
+            assert_allclose(
+                np.linalg.norm(this_proj['data']), 1., atol=1e-6)
             assert_allclose(this_proj['data'], proj['data']['data'])
             proj = other_raw.apply_proj().get_data()
             assert_allclose(proj[picks], data_load_apply_get, atol=1e-10)
             assert_allclose(proj, direct, atol=1e-10, err_msg=t_kw['err_msg'])
     else:
         raw = reader(**kwargs)
+    n_samp = len(raw.times)
     assert_named_constants(raw.info)
+    # smoke test for gh #9743
+    ids = [id(ch['loc']) for ch in raw.info['chs']]
+    assert len(set(ids)) == len(ids)
 
     full_data = raw._data
     assert raw.__class__.__name__ in repr(raw)  # to test repr
@@ -254,6 +272,9 @@ def _test_raw_reader(reader, test_preloading=True, test_kwargs=True,
     meas_date = raw.info['meas_date']
     assert meas_date is None or meas_date >= _stamp_to_dt((0, 0))
 
+    # test repr_html
+    assert 'Good channels' in raw.info._repr_html_()
+
     # test resetting raw
     if test_kwargs:
         raw2 = reader(**raw._init_kwargs)
@@ -275,7 +296,7 @@ def _test_raw_reader(reader, test_preloading=True, test_kwargs=True,
     assert set(raw.info.keys()) == set(raw3.info.keys())
     assert_allclose(raw3[0:20][0], full_data[0:20], rtol=1e-6,
                     atol=1e-20)  # atol is very small but > 0
-    assert_array_almost_equal(raw.times, raw3.times)
+    assert_allclose(raw.times, raw3.times, atol=1e-6, rtol=1e-6)
 
     assert not math.isnan(raw3.info['highpass'])
     assert not math.isnan(raw3.info['lowpass'])
@@ -347,7 +368,8 @@ def _test_raw_reader(reader, test_preloading=True, test_kwargs=True,
 
     # Make sure that writing info to h5 format
     # (all fields should be compatible)
-    if check_version('h5py'):
+    if check_version('h5io'):
+        read_hdf5, write_hdf5 = _import_h5io_funcs()
         fname_h5 = op.join(tempdir, 'info.h5')
         with _writing_info_hdf5(raw.info):
             write_hdf5(fname_h5, raw.info)
@@ -385,9 +407,61 @@ def _test_raw_reader(reader, test_preloading=True, test_kwargs=True,
                 os.chdir(orig_dir)
             raw_chdir.load_data()
 
+    # make sure that cropping works (with first_samp shift)
+    if n_samp >= 50:  # we crop to this number of samples below
+        for t_prop in (0., 0.5):
+            _test_raw_crop(reader, t_prop, kwargs)
+            if test_preloading:
+                use_kwargs = kwargs.copy()
+                use_kwargs['preload'] = True
+                _test_raw_crop(reader, t_prop, use_kwargs)
+
     return raw
 
 
+def _test_raw_crop(reader, t_prop, kwargs):
+    raw_1 = reader(**kwargs)
+    n_samp = 50  # crop to this number of samples (per instance)
+    crop_t = n_samp / raw_1.info['sfreq']
+    t_start = t_prop * crop_t  # also crop to some fraction into the first inst
+    extra = f' t_start={t_start}, preload={kwargs.get("preload", False)}'
+    stop = (n_samp - 1) / raw_1.info['sfreq']
+    raw_1.crop(0, stop)
+    assert len(raw_1.times) == 50
+    first_time = raw_1.first_time
+    atol = 0.5 / raw_1.info['sfreq']
+    assert_allclose(raw_1.times[-1], stop, atol=atol)
+    raw_2, raw_3 = raw_1.copy(), raw_1.copy()
+    t_tot = raw_1.times[-1] * 3 + 2. / raw_1.info['sfreq']
+    raw_concat = concatenate_raws([raw_1, raw_2, raw_3])
+    assert len(raw_concat._filenames) == 3
+    assert_allclose(raw_concat.times[-1], t_tot)
+    assert_allclose(raw_concat.first_time, first_time)
+    # keep all instances, but crop to t_start at the beginning
+    raw_concat.crop(t_start, None)
+    assert len(raw_concat._filenames) == 3
+    assert_allclose(raw_concat.times[-1], t_tot - t_start, atol=atol)
+    assert_allclose(
+        raw_concat.first_time, first_time + t_start, atol=atol,
+        err_msg=f'Base concat, {extra}')
+    # drop the first instance
+    raw_concat.crop(crop_t, None)
+    assert len(raw_concat._filenames) == 2
+    assert_allclose(
+        raw_concat.times[-1], t_tot - t_start - crop_t, atol=atol)
+    assert_allclose(
+        raw_concat.first_time, first_time + t_start + crop_t,
+        atol=atol, err_msg=f'Dropping one, {extra}')
+    # drop the second instance, leaving just one
+    raw_concat.crop(crop_t, None)
+    assert len(raw_concat._filenames) == 1
+    assert_allclose(
+        raw_concat.times[-1], t_tot - t_start - 2 * crop_t, atol=atol)
+    assert_allclose(
+        raw_concat.first_time, first_time + t_start + 2 * crop_t,
+        atol=atol, err_msg=f'Dropping two, {extra}')
+
+
 def _test_concat(reader, *args):
     """Test concatenation of raw classes that allow not preloading."""
     data = None
@@ -423,19 +497,48 @@ def _test_concat(reader, *args):
 @testing.requires_testing_data
 def test_time_as_index():
     """Test indexing of raw times."""
-    raw_fname = op.join(op.dirname(__file__), '..', '..', 'io', 'tests',
-                        'data', 'test_raw.fif')
     raw = read_raw_fif(raw_fname)
 
     # Test original (non-rounding) indexing behavior
     orig_inds = raw.time_as_index(raw.times)
-    assert(len(set(orig_inds)) != len(orig_inds))
+    assert len(set(orig_inds)) != len(orig_inds)
 
     # Test new (rounding) indexing behavior
     new_inds = raw.time_as_index(raw.times, use_rounding=True)
     assert_array_equal(new_inds, np.arange(len(raw.times)))
 
 
+@pytest.mark.parametrize('meas_date', [None, "orig"])
+@pytest.mark.parametrize('first_samp', [0, 10000])
+def test_crop_by_annotations(meas_date, first_samp):
+    """Test crop by annotations of raw."""
+    raw = read_raw_fif(raw_fname)
+
+    if meas_date is None:
+        raw.set_meas_date(None)
+
+    raw = mne.io.RawArray(raw.get_data(), raw.info, first_samp=first_samp)
+
+    onset = np.array([0, 1.5], float)
+    if meas_date is not None:
+        onset += raw.first_time
+    annot = mne.Annotations(
+        onset=onset,
+        duration=[1, 0.5],
+        description=["a", "b"],
+        orig_time=raw.info['meas_date'])
+
+    raw.set_annotations(annot)
+    raws = raw.crop_by_annotations()
+    assert len(raws) == 2
+    assert len(raws[0].annotations) == 1
+    assert raws[0].times[-1] == pytest.approx(annot[:1].duration[0], rel=1e-3)
+    assert raws[0].annotations.description[0] == annot.description[0]
+    assert len(raws[1].annotations) == 1
+    assert raws[1].times[-1] == pytest.approx(annot[1:2].duration[0], rel=5e-3)
+    assert raws[1].annotations.description[0] == annot.description[1]
+
+
 @pytest.mark.parametrize('offset, origin', [
     pytest.param(0, None, id='times in s. relative to first_samp (default)'),
     pytest.param(0, 2.0, id='times in s. relative to first_samp'),
@@ -556,9 +659,11 @@ def test_repr():
     """Test repr of Raw."""
     sfreq = 256
     info = create_info(3, sfreq)
-    r = repr(RawArray(np.zeros((3, 10 * sfreq)), info))
+    raw = RawArray(np.zeros((3, 10 * sfreq)), info)
+    r = repr(raw)
     assert re.search('<RawArray | 3 x 2560 (10.0 s), ~.* kB, data loaded>',
                      r) is not None, r
+    assert raw._repr_html_()
 
 
 # A class that sets channel data to np.arange, for testing _test_raw_reader
@@ -600,9 +705,9 @@ def test_describe_print():
     assert re.match(
         r'<Raw | test_raw.fif, 376 x 14400 (24\.0 s), '
         r'~3\.. MB, data not loaded>', s[0]) is not None, s[0]
-    assert s[1] == " ch  name      type  unit        min        Q1    median        Q3       max"  # noqa
-    assert s[2] == "  0  MEG 0113  GRAD  fT/cm   -221.80    -38.57     -9.64     19.29    414.67"  # noqa
-    assert s[-1] == "375  EOG 061   EOG   µV      -231.41    271.28    277.16    285.66    334.69"  # noqa
+    assert s[1] == " ch  name      type  unit         min         Q1     median         Q3        max"  # noqa
+    assert s[2] == "  0  MEG 0113  GRAD  fT/cm    -221.80     -38.57      -9.64      19.29     414.67"  # noqa
+    assert s[-1] == "375  EOG 061   EOG   µV       -231.41     271.28     277.16     285.66     334.69"  # noqa
 
 
 @requires_pandas
@@ -697,22 +802,52 @@ def test_get_data_units():
         raw.get_data(units=['fT/cm', 'fT', 'uV'])
 
 
-@pytest.mark.skipif(not _check_eeglabio_installed(strict=False),
-                    reason='eeglabio not installed')
-def test_export_eeglab():
-    """Test saving a Raw instance to EEGLAB's set format."""
+def test_repr_dig_point():
+    """Test printing of DigPoint."""
+    dp = DigPoint(r=np.arange(3), coord_frame=FIFF.FIFFV_COORD_HEAD,
+                  kind=FIFF.FIFFV_POINT_EEG, ident=0)
+    assert 'mm' in repr(dp)
+
+    dp = DigPoint(r=np.arange(3), coord_frame=FIFF.FIFFV_MNE_COORD_MRI_VOXEL,
+                  kind=FIFF.FIFFV_POINT_CARDINAL, ident=0)
+    assert 'mm' not in repr(dp)
+    assert 'voxel' in repr(dp)
+
+
+def test_get_data_tmin_tmax():
+    """Test tmin and tmax parameters of get_data method."""
     fname = Path(__file__).parent / "data" / "test_raw.fif"
     raw = read_raw_fif(fname)
-    raw.load_data()
-    tmpdir = _TempDir()
-    temp_fname = op.join(str(tmpdir), 'test.set')
-    raw.export(temp_fname)
-    raw.drop_channels([ch for ch in ['epoc']
-                       if ch in raw.ch_names])
-    raw_read = read_raw_eeglab(temp_fname, preload=True)
-    assert raw.ch_names == raw_read.ch_names
-    cart_coords = np.array([d['loc'][:3] for d in raw.info['chs']])  # just xyz
-    cart_coords_read = np.array([d['loc'][:3] for d in raw_read.info['chs']])
-    assert_allclose(cart_coords, cart_coords_read)
-    assert_allclose(raw.times, raw_read.times)
-    assert_allclose(raw.get_data(), raw_read.get_data())
+
+    # tmin and tmax just use time_as_index under the hood
+    tmin, tmax = (1, 9)
+    d1 = raw.get_data()
+    d2 = raw.get_data(tmin=tmin, tmax=tmax)
+
+    idxs = raw.time_as_index([tmin, tmax])
+    assert_allclose(d1[:, idxs[0]:idxs[1]], d2)
+
+    # specifying a too low tmin truncates to idx 0
+    d3 = raw.get_data(tmin=-5)
+    assert_allclose(d3, d1)
+
+    # specifying a too high tmax truncates to idx n_times
+    d4 = raw.get_data(tmax=1e6)
+    assert_allclose(d4, d1)
+
+    # when start/stop are passed, tmin/tmax are ignored
+    d5 = raw.get_data(start=1, stop=2, tmin=tmin, tmax=tmax)
+    assert d5.shape[1] == 1
+
+    # validate inputs are properly raised
+    with pytest.raises(TypeError, match='start must be .* int'):
+        raw.get_data(start=None)
+
+    with pytest.raises(TypeError, match='stop must be .* int'):
+        raw.get_data(stop=2.3)
+
+    with pytest.raises(TypeError, match='tmin must be .* float'):
+        raw.get_data(tmin=[1, 2])
+
+    with pytest.raises(TypeError, match='tmax must be .* float'):
+        raw.get_data(tmax=[1, 2])
