@@ -3,7 +3,7 @@
 #          Yousra Bekhti <yousra.bekhti@gmail.com>
 #          Eric Larson <larson.eric.d@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 from collections.abc import Iterable
 
@@ -16,10 +16,9 @@ from ..io.pick import (pick_types, pick_info, pick_channels,
 from ..cov import make_ad_hoc_cov, read_cov, Covariance
 from ..bem import fit_sphere_to_headshape, make_sphere_model, read_bem_solution
 from ..io import RawArray, BaseRaw, Info
-from ..chpi import (read_head_pos, head_pos_to_trans_rot_t, _get_hpi_info,
+from ..chpi import (read_head_pos, head_pos_to_trans_rot_t, get_chpi_info,
                     _get_hpi_initial_fit)
 from ..io.constants import FIFF
-from ..fixes import einsum
 from ..forward import (_magnetic_dipole_field_vec, _merge_meg_eeg_fwds,
                        _stc_src_sel, convert_forward_solution,
                        _prepare_for_forward, _transform_orig_meg_coils,
@@ -32,25 +31,21 @@ from ..source_estimate import _BaseSourceEstimate
 from ..surface import _CheckInside
 from ..utils import (logger, verbose, check_random_state, _pl, _validate_type,
                      _check_preload)
-from ..parallel import check_n_jobs
 from .source import SourceSimulator
 
 
 def _check_cov(info, cov):
     """Check that the user provided a valid covariance matrix for the noise."""
+    _validate_type(cov, (Covariance, None, dict, str, 'path-like'), 'cov')
     if isinstance(cov, Covariance) or cov is None:
         pass
     elif isinstance(cov, dict):
         cov = make_ad_hoc_cov(info, cov, verbose=False)
-    elif isinstance(cov, str):
+    else:
         if cov == 'simple':
             cov = make_ad_hoc_cov(info, None, verbose=False)
         else:
             cov = read_cov(cov, verbose=False)
-    else:
-        raise TypeError('Covariance matrix type not recognized. Valid input '
-                        'types are: instance of Covariance, dict, str, None. '
-                        ', got %s' % (cov,))
     return cov
 
 
@@ -130,7 +125,7 @@ def _check_head_pos(head_pos, info, first_samp, times=None):
 
 @verbose
 def simulate_raw(info, stc=None, trans=None, src=None, bem=None, head_pos=None,
-                 mindist=1.0, interp='cos2', n_jobs=1, use_cps=True,
+                 mindist=1.0, interp='cos2', n_jobs=None, use_cps=True,
                  forward=None, first_samp=0, max_iter=10000, verbose=None):
     u"""Simulate raw data.
 
@@ -139,8 +134,7 @@ def simulate_raw(info, stc=None, trans=None, src=None, bem=None, head_pos=None,
 
     Parameters
     ----------
-    info : instance of Info
-        The channel information to use for simulation.
+    %(info_not_none)s Used for simulation.
 
         .. versionchanged:: 0.18
            Support for :class:`mne.Info`.
@@ -245,7 +239,6 @@ def simulate_raw(info, stc=None, trans=None, src=None, bem=None, head_pos=None,
     .. footbibliography::
     """  # noqa: E501
     _validate_type(info, Info, 'info')
-    raw_verbose = verbose
 
     if len(pick_types(info, meg=False, stim=True)) == 0:
         event_ch = None
@@ -253,7 +246,6 @@ def simulate_raw(info, stc=None, trans=None, src=None, bem=None, head_pos=None,
         event_ch = pick_channels(info['ch_names'],
                                  _get_stim_channel(None, info))[0]
 
-    n_jobs = check_n_jobs(n_jobs)
     if forward is not None:
         if any(x is not None for x in (trans, src, bem, head_pos)):
             raise ValueError('If forward is not None then trans, src, bem, '
@@ -319,7 +311,7 @@ def simulate_raw(info, stc=None, trans=None, src=None, bem=None, head_pos=None,
             None if n == 0 else verts)
         if event_ch is not None:
             this_data[event_ch, :] = stim_data[:n_doing]
-        this_data[meeg_picks] = einsum('svt,vt->st', fwd, stc_data)
+        this_data[meeg_picks] = np.einsum('svt,vt->st', fwd, stc_data)
         try:
             stc_counted = next(stc_enum)
         except StopIteration:
@@ -333,13 +325,12 @@ def simulate_raw(info, stc=None, trans=None, src=None, bem=None, head_pos=None,
     raw_data = np.concatenate(raw_datas, axis=-1)
     raw = RawArray(raw_data, info, first_samp=first_samp, verbose=False)
     raw.set_annotations(raw.annotations)
-    raw.verbose = raw_verbose
-    logger.info('Done')
+    logger.info('[done]')
     return raw
 
 
 @verbose
-def add_eog(raw, head_pos=None, interp='cos2', n_jobs=1, random_state=None,
+def add_eog(raw, head_pos=None, interp='cos2', n_jobs=None, random_state=None,
             verbose=None):
     """Add blink noise to raw data.
 
@@ -393,7 +384,7 @@ def add_eog(raw, head_pos=None, interp='cos2', n_jobs=1, random_state=None,
 
 
 @verbose
-def add_ecg(raw, head_pos=None, interp='cos2', n_jobs=1, random_state=None,
+def add_ecg(raw, head_pos=None, interp='cos2', n_jobs=None, random_state=None,
             verbose=None):
     """Add ECG noise to raw data.
 
@@ -528,7 +519,7 @@ def _add_exg(raw, kind, head_pos, interp, n_jobs, random_state):
     proc_lims = np.concatenate([np.arange(0, len(used), 10000), [len(used)]])
     for start, stop in zip(proc_lims[:-1], proc_lims[1:]):
         fwd, _ = interper.feed(stop - start)
-        data[picks, start:stop] += einsum(
+        data[picks, start:stop] += np.einsum(
             'svt,vt->st', fwd, exg_data[:, start:stop])
         assert not used[start:stop].any()
         used[start:stop] = True
@@ -536,7 +527,7 @@ def _add_exg(raw, kind, head_pos, interp, n_jobs, random_state):
 
 
 @verbose
-def add_chpi(raw, head_pos=None, interp='cos2', n_jobs=1, verbose=None):
+def add_chpi(raw, head_pos=None, interp='cos2', n_jobs=None, verbose=None):
     """Add cHPI activations to raw data.
 
     Parameters
@@ -564,7 +555,7 @@ def add_chpi(raw, head_pos=None, interp='cos2', n_jobs=1, verbose=None):
     if len(meg_picks) == 0:
         raise RuntimeError('Cannot add cHPI if no MEG picks are present')
     dev_head_ts, offsets = _check_head_pos(head_pos, info, first_samp, times)
-    hpi_freqs, hpi_pick, hpi_ons = _get_hpi_info(info)
+    hpi_freqs, hpi_pick, hpi_ons = get_chpi_info(info, on_missing='raise')
     hpi_rrs = _get_hpi_initial_fit(info, verbose='error')
     hpi_nns = hpi_rrs / np.sqrt(np.sum(hpi_rrs * hpi_rrs,
                                        axis=1))[:, np.newaxis]
@@ -575,7 +566,8 @@ def add_chpi(raw, head_pos=None, interp='cos2', n_jobs=1, verbose=None):
     sinusoids = 70e-9 * np.sin(2 * np.pi * hpi_freqs[:, np.newaxis] *
                                (np.arange(len(times)) / info['sfreq']))
     info = pick_info(info, meg_picks)
-    info.update(projs=[], bads=[])  # Ensure no 'projs' or 'bads'
+    with info._unlock():
+        info.update(projs=[], bads=[])  # Ensure no 'projs' or 'bads'
     megcoils, _, _, _ = _prep_meg_channels(info, ignore_ref=False)
     used = np.zeros(len(raw.times), bool)
     dev_head_ts.append(dev_head_ts[-1])  # ZOH after time ends
@@ -584,7 +576,7 @@ def add_chpi(raw, head_pos=None, interp='cos2', n_jobs=1, verbose=None):
     lims = np.concatenate([offsets, [len(raw.times)]])
     for start, stop in zip(lims[:-1], lims[1:]):
         fwd, = interper.feed(stop - start)
-        data[meg_picks, start:stop] += einsum(
+        data[meg_picks, start:stop] += np.einsum(
             'svt,vt->st', fwd, sinusoids[:, start:stop])
         assert not used[start:stop].any()
         used[start:stop] = True
@@ -691,7 +683,8 @@ def _iter_forward_solutions(info, trans, src, bem, dev_head_ts, mindist,
     """Calculate a forward solution for a subject."""
     logger.info('Setting up forward solutions')
     info = pick_info(info, picks)
-    info.update(projs=[], bads=[])  # Ensure no 'projs' or 'bads'
+    with info._unlock():
+        info.update(projs=[], bads=[])  # Ensure no 'projs' or 'bads'
     mri_head_t, trans = _get_trans(trans)
     megcoils, meg_info, compcoils, megnames, eegels, eegnames, rr, info, \
         update_kwargs, bem = _prepare_for_forward(
