@@ -1,15 +1,21 @@
+# -*- coding: utf-8 -*-
+# Author: Eric Larson <larson.eric.d@gmail.com>
+#
+# License: BSD-3-Clause
+
+from collections import Counter
 import inspect
 from inspect import getsource
 import os.path as op
+from pathlib import Path
 from pkgutil import walk_packages
 import re
 import sys
-from unittest import SkipTest
 
 import pytest
 
 import mne
-from mne.utils import run_tests_if_main, requires_numpydoc, _pl
+from mne.utils import requires_numpydoc, _pl, _record_warnings
 
 public_modules = [
     # the list of modules users need to access for all functionality
@@ -18,7 +24,6 @@ public_modules = [
     'mne.beamformer',
     'mne.channels',
     'mne.chpi',
-    'mne.connectivity',
     'mne.cov',
     'mne.cuda',
     'mne.datasets',
@@ -27,8 +32,10 @@ public_modules = [
     'mne.datasets.sample',
     'mne.decoding',
     'mne.dipole',
+    'mne.export',
     'mne.filter',
     'mne.forward',
+    'mne.gui',
     'mne.inverse_sparse',
     'mne.io',
     'mne.io.kit',
@@ -63,16 +70,12 @@ def _func_name(func, cls=None):
 
 # functions to ignore args / docstring of
 docstring_ignores = {
-    'mne.externals',
     'mne.fixes',
     'mne.io.write',
     'mne.io.meas_info.Info',
-    'mne.utils.docs.deprecated',
 }
 char_limit = 800  # XX eventually we should probably get this lower
 tab_ignores = [
-    'mne.externals.tqdm._tqdm.__main__',
-    'mne.externals.tqdm._tqdm.cli',
     'mne.channels.tests.test_montage',
     'mne.io.curry.tests.test_curry',
 ]
@@ -106,9 +109,7 @@ def check_parameters_match(func, cls=None):
     from numpydoc.validate import validate
     name = _func_name(func, cls)
     skip = (not name.startswith('mne.') or
-            any(re.match(d, name) for d in docstring_ignores) or
-            'deprecation_wrapped' in getattr(
-                getattr(func, '__code__', None), 'co_name', ''))
+            any(re.match(d, name) for d in docstring_ignores))
     if skip:
         return list()
     if cls is not None:
@@ -127,21 +128,14 @@ def check_parameters_match(func, cls=None):
 def test_docstring_parameters():
     """Test module docstring formatting."""
     from numpydoc import docscrape
-    # skip modules that require mayavi if mayavi is not installed
-    public_modules_ = public_modules[:]
-    try:
-        import mayavi  # noqa: F401 analysis:ignore
-        public_modules_.append('mne.gui')
-    except ImportError:
-        pass
 
     incorrect = []
-    for name in public_modules_:
+    for name in public_modules:
         # Assert that by default we import all public names with `import mne`
         if name not in ('mne', 'mne.gui'):
             extra = name.split('.')[1]
             assert hasattr(mne, extra)
-        with pytest.warns(None):  # traits warnings
+        with _record_warnings():  # traits warnings
             module = __import__(name, globals())
         for submod in name.split('.')[1:]:
             module = getattr(module, submod)
@@ -155,7 +149,8 @@ def test_docstring_parameters():
                 method = getattr(cls, method_name)
                 incorrect += check_parameters_match(method, cls=cls)
             if hasattr(cls, '__call__') and \
-                    'of type object' not in str(cls.__call__):
+                    'of type object' not in str(cls.__call__) and \
+                    'of ABCMeta object' not in str(cls.__call__):
                 incorrect += check_parameters_match(cls.__call__, cls)
         functions = inspect.getmembers(module, inspect.isfunction)
         for fname, func in functions:
@@ -171,21 +166,12 @@ def test_docstring_parameters():
 
 def test_tabs():
     """Test that there are no tabs in our source files."""
-    # avoid importing modules that require mayavi if mayavi is not installed
-    ignore = tab_ignores[:]
-    try:
-        import mayavi  # noqa: F401 analysis:ignore
-    except ImportError:
-        ignore.extend('mne.gui.' + name for name in
-                      ('_coreg_gui', '_fiducials_gui', '_file_traits', '_help',
-                       '_kit2fiff_gui', '_marker_gui', '_viewer'))
-
     for _, modname, ispkg in walk_packages(mne.__path__, prefix='mne.'):
         # because we don't import e.g. mne.tests w/mne
-        if not ispkg and modname not in ignore:
+        if not ispkg and modname not in tab_ignores:
             # mod = importlib.import_module(modname)  # not py26 compatible!
             try:
-                with pytest.warns(None):
+                with _record_warnings():
                     __import__(modname)
             except Exception:  # can't import properly
                 continue
@@ -221,7 +207,6 @@ adjust_axes
 apply_maxfilter
 apply_trans
 channel_type
-check_n_jobs
 combine_kit_markers
 combine_tfr
 combine_transforms
@@ -265,25 +250,16 @@ write_info
 
 def test_documented():
     """Test that public functions and classes are documented."""
-    # skip modules that require mayavi if mayavi is not installed
-    public_modules_ = public_modules[:]
-    try:
-        import mayavi  # noqa: F401, analysis:ignore
-    except ImportError:
-        pass
-    else:
-        public_modules_.append('mne.gui')
-
     doc_dir = op.abspath(op.join(op.dirname(__file__), '..', '..', 'doc'))
     doc_file = op.join(doc_dir, 'python_reference.rst')
     if not op.isfile(doc_file):
-        raise SkipTest('Documentation file not found: %s' % doc_file)
+        pytest.skip('Documentation file not found: %s' % doc_file)
     api_files = (
-        'connectivity', 'covariance', 'creating_from_arrays', 'datasets',
+        'covariance', 'creating_from_arrays', 'datasets',
         'decoding', 'events', 'file_io', 'forward', 'inverse', 'logging',
         'most_used_classes', 'mri', 'preprocessing', 'reading_raw_data',
         'realtime', 'report', 'sensor_space', 'simulation', 'source_space',
-        'statistics', 'time_frequency', 'visualization')
+        'statistics', 'time_frequency', 'visualization', 'export')
     known_names = list()
     for api_file in api_files:
         with open(op.join(doc_dir, f'{api_file}.rst'), 'rb') as fid:
@@ -297,8 +273,8 @@ def test_documented():
     known_names = set(known_names)
 
     missing = []
-    for name in public_modules_:
-        with pytest.warns(None):  # traits warnings
+    for name in public_modules:
+        with _record_warnings():  # traits warnings
             module = __import__(name, globals())
         for submod in name.split('.')[1:]:
             module = getattr(module, submod)
@@ -309,10 +285,10 @@ def test_documented():
             if not name.startswith('_') and name not in known_names:
                 from_mod = inspect.getmodule(cf).__name__
                 if (from_mod.startswith('mne') and
-                        not from_mod.startswith('mne.externals') and
                         not any(from_mod.startswith(x)
                                 for x in documented_ignored_mods) and
-                        name not in documented_ignored_names):
+                        name not in documented_ignored_names and
+                        not hasattr(cf, '_deprecated_original')):
                     missing.append('%s (%s.%s)' % (name, from_mod, name))
     if len(missing) > 0:
         raise AssertionError('\n\nFound new public members missing from '
@@ -320,4 +296,20 @@ def test_documented():
                              '\n* '.join(sorted(set(missing))))
 
 
-run_tests_if_main()
+def test_docdict_order():
+    """Test that docdict is alphabetical."""
+    docs_path = Path(__file__).parent.parent / 'utils' / 'docs.py'
+    assert docs_path.is_file(), docs_path
+    with open(docs_path, 'r') as fid:
+        docs = fid.read()
+    entries = re.findall(r'docdict\[["\']([a-z_\-0-9]+)["\']\] = ', docs)
+    # We can always modify the upper or lower bounds here as entries are made.
+    # This just makes sure our regex is reasonable.
+    # As of 2022/03/02 we're at 346
+    assert 340 < len(entries) < 500
+    # no dups (verbosely describe the error)
+    for key, count in Counter(entries).items():
+        assert count == 1, key
+    # this should be redundant, but why not
+    assert len(set(entries)) == len(entries)
+    assert sorted(entries) == entries
