@@ -187,6 +187,7 @@ def test_cov_rank_estimation(rank_method, proj, meg):
             assert rank[ch_type] == expected_rank
 
 
+@pytest.mark.slowtest  # ~3 sec apiece on Azure means overall it's slow
 @testing.requires_testing_data
 @pytest.mark.parametrize('fname, rank_orig', ((hp_fif_fname, 120),
                                               (mf_fif_fname, 67)))
@@ -204,7 +205,11 @@ def test_maxfilter_get_rank(n_proj, fname, rank_orig, meg, tol_kind, tol):
     assert raw.info['projs'] == []
     mf = raw.info['proc_history'][0]['max_info']
     assert mf['sss_info']['nfree'] == rank_orig
-    assert _get_rank_sss(raw) == rank_orig
+
+    assert compute_rank(raw, 'info')['meg'] == rank_orig
+    assert compute_rank(raw.copy().pick('grad'), 'info')['grad'] == rank_orig
+    assert compute_rank(raw.copy().pick('mag'), 'info')['mag'] == rank_orig
+
     mult = 1 + (meg == 'separate')
     rank = rank_orig - mult * n_proj
     if n_proj > 0:
@@ -257,3 +262,42 @@ def test_maxfilter_get_rank(n_proj, fname, rank_orig, meg, tol_kind, tol):
     rank_new = _compute_rank_int(raw, 'info')
     assert rank_new == rank
     assert_array_equal(raw[:][0], data_orig)
+
+
+def test_explicit_bads_pick():
+    """Test when bads channels are explicitly passed + default picks=None."""
+    raw = read_raw_fif(raw_fname).crop(0, 5).load_data()
+    raw.pick_types(eeg=True, meg=True, ref_meg=True)
+
+    # Covariance
+    # Default picks=None
+    raw.info['bads'] = list()
+    noise_cov_1 = compute_raw_covariance(raw, picks=None)
+    rank = compute_rank(noise_cov_1, info=raw.info)
+    assert rank == dict(meg=303, eeg=60)
+    assert raw.info['bads'] == []
+
+    raw.info['bads'] = ['EEG 002', 'EEG 012', 'EEG 015', 'MEG 0122']
+    noise_cov = compute_raw_covariance(raw, picks=None)
+    rank = compute_rank(noise_cov, info=raw.info)
+    assert rank == dict(meg=302, eeg=57)
+    assert raw.info['bads'] == ['EEG 002', 'EEG 012', 'EEG 015', 'MEG 0122']
+
+    # Explicit picks
+    picks = pick_types(raw.info, meg=True, eeg=True, exclude=[])
+    noise_cov_2 = compute_raw_covariance(raw, picks=picks)
+    rank = compute_rank(noise_cov_2, info=raw.info)
+    assert rank == dict(meg=303, eeg=60)
+    assert raw.info['bads'] == ['EEG 002', 'EEG 012', 'EEG 015', 'MEG 0122']
+
+    assert_array_equal(noise_cov_1['data'], noise_cov_2['data'])
+    assert noise_cov_1['names'] == noise_cov_2['names']
+
+    # Raw
+    raw.info['bads'] = list()
+    rank = compute_rank(raw)
+    assert rank == dict(meg=303, eeg=60)
+
+    raw.info['bads'] = ['EEG 002', 'EEG 012', 'EEG 015', 'MEG 0122']
+    rank = compute_rank(raw)
+    assert rank == dict(meg=302, eeg=57)
