@@ -3,22 +3,21 @@
 #          Susanna Aro <susanna.aro@aalto.fi>
 #          Roman Goj <roman.goj@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import copy as cp
 import numbers
 
 import numpy as np
+
 from .tfr import _cwt_array, morlet, _get_nfft
-from ..fixes import _import_fft
 from ..io.pick import pick_channels, _picks_to_idx
 from ..utils import (logger, verbose, warn, copy_function_doc_to_method_doc,
-                     ProgressBar)
+                     ProgressBar, _check_fname, _import_h5io_funcs)
 from ..viz.misc import plot_csd
 from ..time_frequency.multitaper import (_compute_mt_params, _mt_spectra,
                                          _csd_from_mt, _psd_from_mt_adaptive)
 from ..parallel import parallel_func
-from ..externals.h5io import read_hdf5, write_hdf5
 
 
 def pick_channels_csd(csd, include=[], exclude=[], ordered=False, copy=True):
@@ -443,7 +442,8 @@ class CrossSpectralDensity(object):
             projs=self.projs,
         )
 
-    def save(self, fname):
+    @verbose
+    def save(self, fname, *, overwrite=False, verbose=None):
         """Save the CSD to an HDF5 file.
 
         Parameters
@@ -451,15 +451,24 @@ class CrossSpectralDensity(object):
         fname : str
             The name of the file to save the CSD to. The extension '.h5' will
             be appended if the given filename doesn't have it already.
+        %(overwrite)s
+
+            .. versionadded:: 1.0
+        %(verbose)s
+
+            .. versionadded:: 1.0
 
         See Also
         --------
         read_csd : For reading CSD objects from a file.
         """
+        _, write_hdf5 = _import_h5io_funcs()
         if not fname.endswith('.h5'):
             fname += '.h5'
 
-        write_hdf5(fname, self.__getstate__(), overwrite=True, title='conpy')
+        fname = _check_fname(fname, overwrite=overwrite)
+        write_hdf5(fname, self.__getstate__(), overwrite=True,
+                   title='conpy')
 
     def copy(self):
         """Return copy of the CrossSpectralDensity object.
@@ -592,6 +601,7 @@ def read_csd(fname):
     --------
     CrossSpectralDensity.save : For saving CSD objects.
     """
+    read_hdf5, _ = _import_h5io_funcs()
     if not fname.endswith('.h5'):
         fname += '.h5'
 
@@ -601,7 +611,7 @@ def read_csd(fname):
 
 @verbose
 def csd_fourier(epochs, fmin=0, fmax=np.inf, tmin=None, tmax=None, picks=None,
-                n_fft=None, projs=None, n_jobs=1, verbose=None):
+                n_fft=None, projs=None, n_jobs=None, *, verbose=None):
     """Estimate cross-spectral density from an array using short-time fourier.
 
     Parameters
@@ -651,7 +661,7 @@ def csd_fourier(epochs, fmin=0, fmax=np.inf, tmin=None, tmax=None, picks=None,
 @verbose
 def csd_array_fourier(X, sfreq, t0=0, fmin=0, fmax=np.inf, tmin=None,
                       tmax=None, ch_names=None, n_fft=None, projs=None,
-                      n_jobs=1, verbose=None):
+                      n_jobs=None, *, verbose=None):
     """Estimate cross-spectral density from an array using short-time fourier.
 
     Parameters
@@ -699,7 +709,7 @@ def csd_array_fourier(X, sfreq, t0=0, fmin=0, fmax=np.inf, tmin=None,
     csd_morlet
     csd_multitaper
     """
-    rfftfreq = _import_fft('rfftfreq')
+    from scipy.fft import rfftfreq
     X, times, tmin, tmax, fmin, fmax = _prepare_csd_array(
         X, sfreq, t0, tmin, tmax, fmin, fmax)
 
@@ -732,7 +742,7 @@ def csd_array_fourier(X, sfreq, t0=0, fmin=0, fmax=np.inf, tmin=None,
 @verbose
 def csd_multitaper(epochs, fmin=0, fmax=np.inf, tmin=None, tmax=None,
                    picks=None, n_fft=None, bandwidth=None, adaptive=False,
-                   low_bias=True, projs=None, n_jobs=1, verbose=None):
+                   low_bias=True, projs=None, n_jobs=None, *, verbose=None):
     """Estimate cross-spectral density from epochs using a multitaper method.
 
     Parameters
@@ -791,8 +801,8 @@ def csd_multitaper(epochs, fmin=0, fmax=np.inf, tmin=None, tmax=None,
 @verbose
 def csd_array_multitaper(X, sfreq, t0=0, fmin=0, fmax=np.inf, tmin=None,
                          tmax=None, ch_names=None, n_fft=None, bandwidth=None,
-                         adaptive=False, low_bias=True, projs=None, n_jobs=1,
-                         verbose=None):
+                         adaptive=False, low_bias=True, projs=None,
+                         n_jobs=None, *, verbose=None):
     """Estimate cross-spectral density from an array using a multitaper method.
 
     Parameters
@@ -847,7 +857,7 @@ def csd_array_multitaper(X, sfreq, t0=0, fmin=0, fmax=np.inf, tmin=None,
     csd_morlet
     csd_multitaper
     """
-    rfftfreq = _import_fft('rfftfreq')
+    from scipy.fft import rfftfreq
     X, times, tmin, tmax, fmin, fmax = _prepare_csd_array(
         X, sfreq, t0, tmin, tmax, fmin, fmax)
 
@@ -882,7 +892,7 @@ def csd_array_multitaper(X, sfreq, t0=0, fmin=0, fmax=np.inf, tmin=None,
 
 @verbose
 def csd_morlet(epochs, frequencies, tmin=None, tmax=None, picks=None,
-               n_cycles=7, use_fft=True, decim=1, projs=None, n_jobs=1,
+               n_cycles=7, use_fft=True, decim=1, projs=None, n_jobs=None, *,
                verbose=None):
     """Estimate cross-spectral density from epochs using Morlet wavelets.
 
@@ -942,7 +952,7 @@ def csd_morlet(epochs, frequencies, tmin=None, tmax=None, picks=None,
 @verbose
 def csd_array_morlet(X, sfreq, frequencies, t0=0, tmin=None, tmax=None,
                      ch_names=None, n_cycles=7, use_fft=True, decim=1,
-                     projs=None, n_jobs=1, verbose=None):
+                     projs=None, n_jobs=None, *, verbose=None):
     """Estimate cross-spectral density from an array using Morlet wavelets.
 
     Parameters
@@ -1096,7 +1106,8 @@ def _prepare_csd_array(X, sfreq, t0, tmin, tmax, fmin=None, fmax=None):
 
 @verbose
 def _execute_csd_function(X, times, frequencies, csd_function, params, n_fft,
-                          ch_names=None, projs=None, n_jobs=1, verbose=None):
+                          ch_names=None, projs=None, n_jobs=None, *,
+                          verbose=None):
     """Estimate cross-spectral density with a given function.
 
     This function will apply the given CSD function in parallel across epochs.
@@ -1140,7 +1151,8 @@ def _execute_csd_function(X, times, frequencies, csd_function, params, n_fft,
 
     # Prepare the function that does the actual CSD computation for parallel
     # execution.
-    parallel, my_csd, _ = parallel_func(csd_function, n_jobs, verbose=verbose)
+    parallel, my_csd, n_jobs = parallel_func(
+        csd_function, n_jobs, verbose=verbose)
 
     # Compute CSD for each trial
     n_blocks = int(np.ceil(n_epochs / float(n_jobs)))
