@@ -4,7 +4,7 @@
 # Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Christian Brodbeck <christianbrodbeck@nyu.edu>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import os
 import os.path as op
@@ -13,13 +13,15 @@ import glob
 import numpy as np
 from copy import deepcopy
 
-from .fixes import einsum, jit, mean
+from .fixes import jit, mean, _get_img_fdata
 from .io.constants import FIFF
 from .io.open import fiff_open
 from .io.tag import read_tag
-from .io.write import start_file, end_file, write_coord_trans
+from .io.write import start_and_end_file, write_coord_trans
+from .defaults import _handle_default
 from .utils import (check_fname, logger, verbose, _ensure_int, _validate_type,
-                    _check_path_like, get_subjects_dir, fill_doc, _check_fname)
+                    _path_like, get_subjects_dir, fill_doc, _check_fname,
+                    _check_option, _require_version, wrapped_stdout)
 
 
 # transformation from anterior/left/superior coordinate system to
@@ -62,7 +64,9 @@ def _to_const(cf):
     """Convert string or int coord frame into int."""
     if isinstance(cf, str):
         if cf not in _str_to_frame:
-            raise ValueError('Unknown cf %s' % cf)
+            raise ValueError(
+                f'Unknown coordinate frame {cf}, '
+                'expected "' + '", "'.join(_str_to_frame.keys()) + '"')
         cf = _str_to_frame[cf]
     else:
         cf = _ensure_int(cf, 'coordinate frame', 'a str or int')
@@ -95,16 +99,17 @@ class Transform(dict):
         to = _to_const(to)
         trans = np.eye(4) if trans is None else np.asarray(trans, np.float64)
         if trans.shape != (4, 4):
-            raise ValueError('Transformation must be shape (4, 4) not %s'
-                             % (trans.shape,))
+            raise ValueError(
+                f'Transformation must be shape (4, 4) not {trans.shape}')
         self['from'] = fro
         self['to'] = to
         self['trans'] = trans
 
     def __repr__(self):  # noqa: D105
-        return ('<Transform | %s->%s>\n%s'
-                % (_coord_frame_name(self['from']),
-                   _coord_frame_name(self['to']), self['trans']))
+        with np.printoptions(suppress=True):  # suppress scientific notation
+            return '<Transform | {fro}->{to}>\n{trans}'.format(
+                fro=_coord_frame_name(self['from']),
+                to=_coord_frame_name(self['to']), trans=self['trans'])
 
     def __eq__(self, other, rtol=0., atol=0.):
         """Check for equality.
@@ -182,8 +187,8 @@ def _print_coord_trans(t, prefix='Coordinate transformation: ', units='m',
                        level='info'):
     # Units gives the units of the transformation. This always prints in mm.
     log_func = getattr(logger, level)
-    log_func(prefix + '%s -> %s'
-             % (_coord_frame_name(t['from']), _coord_frame_name(t['to'])))
+    log_func(prefix + '{fro} -> {to}'.format(
+             fro=_coord_frame_name(t['from']), to=_coord_frame_name(t['to'])))
     for ti, tt in enumerate(t['trans']):
         scale = 1000. if (ti != 3 and units != 'mm') else 1.
         text = ' mm' if ti != 3 else ''
@@ -328,10 +333,10 @@ def rotation3d_align_z_axis(target_z_axis):
                             target_z_axis[1] * target_z_axis[1])
 
     # assert that r is a rotation matrix r^t * r = I and det(r) = 1
-    assert(np.any((r.dot(r.T) - np.identity(3)) < 1E-12))
-    assert((np.linalg.det(r) - 1.0) < 1E-12)
+    assert np.any((r.dot(r.T) - np.identity(3)) < 1E-12)
+    assert (np.linalg.det(r) - 1.0) < 1E-12
     # assert that r maps [0 0 1] on the device z axis (target_z_axis)
-    assert(np.linalg.norm(target_z_axis - r.dot([0, 0, 1])) < 1e-12)
+    assert np.linalg.norm(target_z_axis - r.dot([0, 0, 1])) < 1e-12
 
     return r
 
@@ -414,8 +419,8 @@ def _ensure_trans(trans, fro='mri', to='head'):
         to_str = _frame_to_str[to]
         to_const = to
     del to
-    err_str = ('trans must be a Transform between %s<->%s, got'
-               % (from_str, to_str))
+    err_str = 'trans must be a Transform between ' \
+        f'{from_str}<->{to_str}, got'
     if not isinstance(trans, (list, tuple)):
         trans = [trans]
     # Ensure that we have exactly one match
@@ -423,15 +428,16 @@ def _ensure_trans(trans, fro='mri', to='head'):
     misses = list()
     for ti, this_trans in enumerate(trans):
         if not isinstance(this_trans, Transform):
-            raise ValueError('%s None' % err_str)
+            raise ValueError(f'{err_str} None')
         if {this_trans['from'],
                 this_trans['to']} == {from_const, to_const}:
             idx.append(ti)
         else:
-            misses += ['%s->%s' % (_frame_to_str[this_trans['from']],
-                                   _frame_to_str[this_trans['to']])]
+            misses += ['{fro}->{to}'.format(
+                fro=_frame_to_str[this_trans['from']],
+                to=_frame_to_str[this_trans['to']])]
     if len(idx) != 1:
-        raise ValueError('%s %s' % (err_str, ', '.join(misses)))
+        raise ValueError(f'{err_str} ' + ', '.join(misses))
     trans = trans[idx[0]]
     if trans['from'] != from_const:
         trans = invert_transform(trans)
@@ -444,13 +450,13 @@ def _get_trans(trans, fro='mri', to='head', allow_none=True):
     if allow_none:
         types += (None,)
     _validate_type(trans, types, 'trans')
-    if _check_path_like(trans):
+    if _path_like(trans):
         trans = str(trans)
         if trans == 'fsaverage':
             trans = op.join(op.dirname(__file__), 'data', 'fsaverage',
                             'fsaverage-trans.fif')
         if not op.isfile(trans):
-            raise IOError('trans file "%s" not found' % trans)
+            raise IOError(f'trans file "{trans}" not found')
         if op.splitext(trans)[1] in ['.fif', '.gz']:
             fro_to_t = read_trans(trans)
         else:
@@ -458,8 +464,7 @@ def _get_trans(trans, fro='mri', to='head', allow_none=True):
             # these are usually actually in to_fro form
             t = np.genfromtxt(trans)
             if t.ndim != 2 or t.shape != (4, 4):
-                raise RuntimeError('File "%s" did not have 4x4 entries'
-                                   % trans)
+                raise RuntimeError(f'File "{trans}" did not have 4x4 entries')
             fro_to_t = Transform(to, fro, t)
     elif isinstance(trans, Transform):
         fro_to_t = trans
@@ -495,31 +500,32 @@ def combine_transforms(t_first, t_second, fro, to):
     fro = _to_const(fro)
     to = _to_const(to)
     if t_first['from'] != fro:
-        raise RuntimeError('From mismatch: %s ("%s") != %s ("%s")'
-                           % (t_first['from'],
-                              _coord_frame_name(t_first['from']),
-                              fro, _coord_frame_name(fro)))
+        raise RuntimeError(
+            'From mismatch: {fro1} ("{cf1}") != {fro2} ("{cf2}")'.format(
+                fro1=t_first['from'], cf1=_coord_frame_name(t_first['from']),
+                fro2=fro, cf2=_coord_frame_name(fro)))
     if t_first['to'] != t_second['from']:
-        raise RuntimeError('Transform mismatch: t1["to"] = %s ("%s"), '
-                           't2["from"] = %s ("%s")'
-                           % (t_first['to'], _coord_frame_name(t_first['to']),
-                              t_second['from'],
-                              _coord_frame_name(t_second['from'])))
+        raise RuntimeError('Transform mismatch: t1["to"] = {to1} ("{cf1}"), '
+                           't2["from"] = {fro2} ("{cf2}")'.format(
+                               to1=t_first['to'],
+                               cf1=_coord_frame_name(t_first['to']),
+                               fro2=t_second['from'],
+                               cf2=_coord_frame_name(t_second['from'])))
     if t_second['to'] != to:
-        raise RuntimeError('To mismatch: %s ("%s") != %s ("%s")'
-                           % (t_second['to'],
-                              _coord_frame_name(t_second['to']),
-                              to, _coord_frame_name(to)))
+        raise RuntimeError(
+            'To mismatch: {to1} ("{cf1}") != {to2} ("{cf2}")'.format(
+                to1=t_second['to'], cf1=_coord_frame_name(t_second['to']),
+                to2=to, cf2=_coord_frame_name(to)))
     return Transform(fro, to, np.dot(t_second['trans'], t_first['trans']))
 
 
 @verbose
 def read_trans(fname, return_all=False, verbose=None):
-    """Read a -trans.fif file.
+    """Read a ``-trans.fif`` file.
 
     Parameters
     ----------
-    fname : str
+    fname : path-like
         The name of the file.
     return_all : bool
         If True, return all transformations in the file.
@@ -538,6 +544,7 @@ def read_trans(fname, return_all=False, verbose=None):
     write_trans
     mne.transforms.Transform
     """
+    fname = _check_fname(fname, overwrite='read', must_exist=True)
     fid, tree, directory = fiff_open(fname)
 
     trans = list()
@@ -552,15 +559,18 @@ def read_trans(fname, return_all=False, verbose=None):
     return trans if return_all else trans[0]
 
 
-def write_trans(fname, trans):
+@verbose
+def write_trans(fname, trans, *, overwrite=False, verbose=None):
     """Write a -trans.fif file.
 
     Parameters
     ----------
-    fname : str
-        The name of the file, which should end in '-trans.fif'.
+    fname : path-like
+        The name of the file, which should end in ``-trans.fif``.
     trans : dict
         Trans file data, as returned by read_trans.
+    %(overwrite)s
+    %(verbose)s
 
     See Also
     --------
@@ -568,9 +578,9 @@ def write_trans(fname, trans):
     """
     check_fname(fname, 'trans', ('-trans.fif', '-trans.fif.gz',
                                  '_trans.fif', '_trans.fif.gz'))
-    fid = start_file(fname)
-    write_coord_trans(fid, trans)
-    end_file(fid)
+    fname = _check_fname(fname=fname, overwrite=overwrite)
+    with start_and_end_file(fname) as fid:
+        write_coord_trans(fid, trans)
 
 
 def invert_transform(trans):
@@ -679,6 +689,22 @@ def get_ras_to_neuromag_trans(nasion, lpa, rpa):
     return trans
 
 
+def _get_transforms_to_coord_frame(info, trans, coord_frame='mri'):
+    """Get the transforms to a coordinate frame from device, head and mri."""
+    head_mri_t = _get_trans(trans, 'head', 'mri')[0]
+    dev_head_t = _get_trans(info['dev_head_t'], 'meg', 'head')[0]
+    mri_dev_t = invert_transform(combine_transforms(
+        dev_head_t, head_mri_t, 'meg', 'mri'))
+    to_cf_t = dict(
+        meg=_ensure_trans([dev_head_t, mri_dev_t, Transform('meg', 'meg')],
+                          fro='meg', to=coord_frame),
+        head=_ensure_trans([dev_head_t, head_mri_t, Transform('head', 'head')],
+                           fro='head', to=coord_frame),
+        mri=_ensure_trans([head_mri_t, mri_dev_t, Transform('mri', 'mri')],
+                          fro='mri', to=coord_frame))
+    return to_cf_t
+
+
 ###############################################################################
 # Spherical coordinates and harmonics
 
@@ -695,8 +721,8 @@ def _cart_to_sph(cart):
     sph_pts : ndarray, shape (n_points, 3)
         Array containing points in spherical coordinates (rad, azimuth, polar)
     """
-    assert cart.ndim == 2 and cart.shape[1] == 3
     cart = np.atleast_2d(cart)
+    assert cart.ndim == 2 and cart.shape[1] == 3
     out = np.empty((len(cart), 3))
     out[:, 0] = np.sqrt(np.sum(cart * cart, axis=1))
     norm = np.where(out[:, 0] > 0, out[:, 0], 1)  # protect against / 0
@@ -720,8 +746,8 @@ def _sph_to_cart(sph_pts):
         Array containing points in Cartesian coordinates (x, y, z)
 
     """
-    assert sph_pts.ndim == 2 and sph_pts.shape[1] == 3
     sph_pts = np.atleast_2d(sph_pts)
+    assert sph_pts.ndim == 2 and sph_pts.shape[1] == 3
     cart_pts = np.empty((len(sph_pts), 3))
     cart_pts[:, 2] = sph_pts[:, 0] * np.cos(sph_pts[:, 2])
     xy = sph_pts[:, 0] * np.sin(sph_pts[:, 2])
@@ -782,7 +808,7 @@ def _sph_to_cart_partials(az, pol, g_rad, g_az, g_pol):
     trans = np.array([[c_as * s_ps, -s_as, c_as * c_ps],
                       [s_as * s_ps, c_as, c_ps * s_as],
                       [c_ps, np.zeros_like(c_as), -s_ps]])
-    cart_grads = einsum('ijk,kj->ki', trans, sph_grads)
+    cart_grads = np.einsum('ijk,kj->ki', trans, sph_grads)
     return cart_grads
 
 
@@ -1255,11 +1281,12 @@ def _quat_to_affine(quat):
     return affine
 
 
-def _angle_between_quats(x, y):
+def _angle_between_quats(x, y=None):
     """Compute the ang between two quaternions w/3-element representations."""
     # z = conj(x) * y
     # conjugate just negates all but the first element in a 4-element quat,
     # so it's just a negative for us
+    y = np.zeros(3) if y is None else y
     z = _quat_mult(-x, y)
     z0 = _quat_real(z)
     return 2 * np.arctan2(np.linalg.norm(z, axis=-1), z0)
@@ -1521,3 +1548,269 @@ def _euler_to_quat(euler):
     quat[..., 2] = cphi * ctheta * spsi - sphi * stheta * cpsi
     quat *= mult
     return quat
+
+
+###############################################################################
+# Affine Registration and SDR
+
+_ORDERED_STEPS = ('translation', 'rigid', 'affine', 'sdr')
+
+
+def _validate_zooms(zooms):
+    _validate_type(zooms, (dict, list, tuple, 'numeric', None), 'zooms')
+    zooms = _handle_default('transform_zooms', zooms)
+    for key, val in zooms.items():
+        _check_option('zooms key', key, _ORDERED_STEPS)
+        if val is not None:
+            val = tuple(
+                float(x) for x in np.array(val, dtype=float).ravel())
+            _check_option(f'len(zooms[{repr(key)})', len(val), (1, 3))
+            if len(val) == 1:
+                val = val * 3
+            for this_zoom in val:
+                if this_zoom <= 1:
+                    raise ValueError(f'Zooms must be > 1, got {this_zoom}')
+            zooms[key] = val
+    return zooms
+
+
+def _validate_niter(niter):
+    _validate_type(niter, (dict, list, tuple, None), 'niter')
+    niter = _handle_default('transform_niter', niter)
+    for key, value in niter.items():
+        _check_option('niter key', key, _ORDERED_STEPS)
+        _check_option(f'len(niter[{repr(key)}])', len(value), (1, 2, 3))
+    return niter
+
+
+def _validate_pipeline(pipeline):
+    _validate_type(pipeline, (str, list, tuple), 'pipeline')
+    pipeline_defaults = dict(
+        all=_ORDERED_STEPS,
+        rigids=_ORDERED_STEPS[:_ORDERED_STEPS.index('rigid') + 1],
+        affines=_ORDERED_STEPS[:_ORDERED_STEPS.index('affine') + 1])
+    if isinstance(pipeline, str):  # use defaults
+        _check_option('pipeline', pipeline, ('all', 'rigids', 'affines'),
+                      extra='when str')
+        pipeline = pipeline_defaults[pipeline]
+    for ii, step in enumerate(pipeline):
+        name = f'pipeline[{ii}]'
+        _validate_type(step, str, name)
+        _check_option(name, step, _ORDERED_STEPS)
+    ordered_pipeline = tuple(sorted(
+        pipeline, key=lambda x: _ORDERED_STEPS.index(x)))
+    if tuple(pipeline) != ordered_pipeline:
+        raise ValueError(
+            f'Steps in pipeline are out of order, expected {ordered_pipeline} '
+            f'but got {pipeline} instead')
+    if len(set(pipeline)) != len(pipeline):
+        raise ValueError('Steps in pipeline should not be repeated')
+    return tuple(pipeline)
+
+
+def _compute_r2(a, b):
+    return 100 * (a.ravel() @ b.ravel()) / \
+        (np.linalg.norm(a) * np.linalg.norm(b))
+
+
+def _reslice_normalize(img, zooms):
+    from dipy.align.reslice import reslice
+    img_zooms = img.header.get_zooms()[:3]
+    img_affine = img.affine
+    img = _get_img_fdata(img)
+    if zooms is not None:
+        img, img_affine = reslice(img, img_affine, img_zooms, zooms)
+    img /= img.max()  # normalize
+    return img, img_affine
+
+
+@verbose
+def compute_volume_registration(moving, static, pipeline='all', zooms=None,
+                                niter=None, verbose=None):
+    """Align two volumes using an affine and, optionally, SDR.
+
+    Parameters
+    ----------
+    %(moving)s
+    %(static)s
+    %(pipeline)s
+    zooms : float | tuple | dict | None
+        The voxel size of volume for each spatial dimension in mm.
+        If None (default), MRIs won't be resliced (slow, but most accurate).
+        Can be a tuple to provide separate zooms for each dimension (X/Y/Z),
+        or a dict with keys ``['translation', 'rigid', 'affine', 'sdr']``
+        (each with values that are float`, tuple, or None) to provide separate
+        reslicing/accuracy for the steps.
+    %(niter)s
+    %(verbose)s
+
+    Returns
+    -------
+    %(reg_affine)s
+    %(sdr_morph)s
+
+    Notes
+    -----
+    This function is heavily inspired by and extends
+    :func:`dipy.align.affine_registration
+    <dipy.align._public.affine_registration>`.
+
+    .. versionadded:: 0.24
+    """
+    return _compute_volume_registration(
+        moving, static, pipeline, zooms, niter)[:2]
+
+
+def _compute_volume_registration(moving, static, pipeline, zooms, niter):
+    _require_version('nibabel', 'SDR morph', '2.1.0')
+    _require_version('dipy', 'SDR morph', '0.10.1')
+    import nibabel as nib
+    with np.testing.suppress_warnings():
+        from dipy.align.imaffine import AffineMap
+        from dipy.align import (affine_registration, center_of_mass,
+                                translation, rigid, affine,
+                                imwarp, metrics)
+
+    # input validation
+    _validate_type(moving, nib.spatialimages.SpatialImage, 'moving')
+    _validate_type(static, nib.spatialimages.SpatialImage, 'static')
+    original_zoom = np.mean(moving.header.get_zooms()[:3])
+    zooms = _validate_zooms(zooms)
+    niter = _validate_niter(niter)
+    pipeline = _validate_pipeline(pipeline)
+
+    logger.info('Computing registration...')
+
+    # affine optimizations
+    reg_affine = None
+    sdr_morph = None
+    pipeline_options = dict(translation=[center_of_mass, translation],
+                            rigid=[rigid], affine=[affine])
+    sigmas_mm = np.array([3.0, 1.0, 0.0])  # default for affine_registration
+    sigma_diff_mm = 2.0
+    factors = [4, 2, 1]
+    current_zoom = None
+    for i, step in enumerate(pipeline):
+        # reslice image with zooms
+        if i == 0 or zooms[step] != zooms[pipeline[i - 1]]:
+            if zooms[step] is not None:
+                logger.info(f'Reslicing to zooms={zooms[step]} for {step} ...')
+                current_zoom = np.mean(zooms[step])
+            else:
+                logger.info(f'Using original zooms for {step} ...')
+                current_zoom = original_zoom
+            static_zoomed, static_affine = _reslice_normalize(
+                static, zooms[step])
+            moving_zoomed, moving_affine = _reslice_normalize(
+                moving, zooms[step])
+        logger.info(f'Optimizing {step}:')
+        if step == 'sdr':  # happens last
+            sigma_diff_vox = sigma_diff_mm / current_zoom
+            affine_map = AffineMap(reg_affine,  # apply registration here
+                                   static_zoomed.shape, static_affine,
+                                   moving_zoomed.shape, moving_affine)
+            moving_zoomed = affine_map.transform(moving_zoomed)
+            metric = metrics.CCMetric(
+                dim=3, sigma_diff=sigma_diff_vox,
+                radius=max(int(np.ceil(2 * sigma_diff_vox)), 1))
+            sdr = imwarp.SymmetricDiffeomorphicRegistration(
+                metric, niter[step])
+            with wrapped_stdout(indent='    ', cull_newlines=True):
+                sdr_morph = sdr.optimize(static_zoomed, moving_zoomed,
+                                         static_affine, static_affine)
+            moved_zoomed = sdr_morph.transform(moving_zoomed)
+        else:
+            sigmas_vox = list(sigmas_mm / current_zoom)
+            with wrapped_stdout(indent='    ', cull_newlines=True):
+                moved_zoomed, reg_affine = affine_registration(
+                    moving_zoomed, static_zoomed, moving_affine, static_affine,
+                    nbins=32, metric='MI', pipeline=pipeline_options[step],
+                    level_iters=niter[step], sigmas=sigmas_vox,
+                    factors=factors, starting_affine=reg_affine)
+
+            # report some useful information
+            if step in ('translation', 'rigid'):
+                dist = np.linalg.norm(reg_affine[:3, 3])
+                angle = np.rad2deg(_angle_between_quats(
+                    np.zeros(3), rot_to_quat(reg_affine[:3, :3])))
+                logger.info(f'    Translation: {dist:6.1f} mm')
+                if step == 'rigid':
+                    logger.info(f'    Rotation:    {angle:6.1f}°')
+        assert moved_zoomed.shape == static_zoomed.shape, step
+        r2 = _compute_r2(static_zoomed, moved_zoomed)
+        logger.info(f'    R²:          {r2:6.1f}%')
+    return (reg_affine, sdr_morph, static_zoomed.shape, static_affine,
+            moving_zoomed.shape, moving_affine)
+
+
+@verbose
+def apply_volume_registration(moving, static, reg_affine, sdr_morph=None,
+                              interpolation='linear', cval=0.,
+                              verbose=None):
+    """Apply volume registration.
+
+    Uses registration parameters computed by
+    :func:`~mne.transforms.compute_volume_registration`.
+
+    Parameters
+    ----------
+    %(moving)s
+    %(static)s
+    %(reg_affine)s
+    %(sdr_morph)s
+    interpolation : str
+        Interpolation to be used during the interpolation.
+        Can be "linear" (default) or "nearest".
+    cval : float | str
+        The constant value to assume exists outside the bounds of the
+        ``moving`` image domain. Can be a string percentage like ``'1%%'``
+        to use the given percentile of image data as the constant value.
+    %(verbose)s
+
+    Returns
+    -------
+    reg_img : instance of SpatialImage
+        The image after affine (and SDR, if provided) registration.
+
+    Notes
+    -----
+    .. versionadded:: 0.24
+    """
+    _require_version('nibabel', 'SDR morph', '2.1.0')
+    _require_version('dipy', 'SDR morph', '0.10.1')
+    from nibabel.spatialimages import SpatialImage
+    from dipy.align.imwarp import DiffeomorphicMap
+    from dipy.align.imaffine import AffineMap
+    _validate_type(moving, SpatialImage, 'moving')
+    _validate_type(static, SpatialImage, 'static')
+    _validate_type(reg_affine, np.ndarray, 'reg_affine')
+    _check_option('reg_affine.shape', reg_affine.shape, ((4, 4),))
+    _validate_type(sdr_morph, (DiffeomorphicMap, None), 'sdr_morph')
+    _validate_type(cval, ('numeric', str), 'cval')
+    perc = None
+    if isinstance(cval, str):
+        if not cval.endswith('%'):
+            raise ValueError(f'cval must end with % if str, got {cval}')
+        perc = float(cval[:-1])
+    logger.info('Applying affine registration ...')
+    moving_affine = moving.affine
+    moving = np.asarray(moving.dataobj, dtype=float)
+    if perc is not None:
+        cval = np.percentile(moving, perc)
+        logger.info(f'Using a lower bound at the {perc} percentile: {cval}')
+    moving -= cval
+    static, static_affine = np.asarray(static.dataobj), static.affine
+    affine_map = AffineMap(reg_affine,
+                           static.shape, static_affine,
+                           moving.shape, moving_affine)
+    reg_data = affine_map.transform(moving, interpolation=interpolation)
+    if sdr_morph is not None:
+        logger.info('Appling SDR warp ...')
+        reg_data = sdr_morph.transform(
+            reg_data, interpolation=interpolation,
+            image_world2grid=np.linalg.inv(static_affine),
+            out_shape=static.shape, out_grid2world=static_affine)
+    reg_data += cval
+    reg_img = SpatialImage(reg_data, static_affine)
+    logger.info('[done]')
+    return reg_img
