@@ -3,54 +3,56 @@
 # is changed here!
 _doc_special_members = ('__contains__', '__getitem__', '__iter__', '__len__',
                         '__add__', '__sub__', '__mul__', '__div__',
-                        '__neg__', '__hash__')
+                        '__neg__')
 
 from ._bunch import Bunch, BunchConst, BunchConstNamed
 from .check import (check_fname, check_version, check_random_state,
                     _check_fname, _check_subject, _check_pandas_installed,
-                    _check_pandas_index_arguments, _check_mayavi_version,
+                    _check_pandas_index_arguments,
                     _check_event_id, _check_ch_locs, _check_compensation_grade,
-                    _check_if_nan, _is_numeric, _ensure_int,  _check_preload,
-                    _validate_type, _check_info_inv, _check_pylsl_installed,
+                    _check_if_nan, _is_numeric, _ensure_int, _check_preload,
+                    _validate_type, _check_info_inv,
                     _check_channels_spatial_filter, _check_one_ch_type,
                     _check_rank, _check_option, _check_depth, _check_combine,
-                    _check_path_like, _check_src_normal, _check_stc_units,
-                    _check_pyqt5_version, _check_sphere, _check_time_format,
+                    _path_like, _check_src_normal, _check_stc_units,
+                    _check_qt_version, _check_sphere, _check_time_format,
                     _check_freesurfer_home, _suggest, _require_version,
                     _on_missing, _check_on_missing, int_like, _safe_input,
                     _check_all_same_channel_names, path_like, _ensure_events,
-                    _check_eeglabio_installed, _infer_check_export_fmt)
+                    _check_eeglabio_installed, _check_pybv_installed,
+                    _check_edflib_installed, _to_rgb, _soft_import,
+                    _check_dict_keys,
+                    _import_h5py, _import_h5io_funcs,
+                    _import_pymatreader_funcs)
 from .config import (set_config, get_config, get_config_path, set_cache_dir,
                      set_memmap_min_size, get_subjects_dir, _get_stim_channel,
                      sys_info, _get_extra_data_path, _get_root_dir,
                      _get_numpy_libs)
 from .docs import (copy_function_doc_to_method_doc, copy_doc, linkcode_resolve,
                    open_docs, deprecated, fill_doc, deprecated_alias,
-                   copy_base_doc_to_subclass_doc)
-from .fetching import _fetch_file, _url_to_local_path
+                   copy_base_doc_to_subclass_doc, docdict as _docdict)
+from .fetching import _url_to_local_path
 from ._logging import (verbose, logger, set_log_level, set_log_file,
                        use_log_level, catch_logging, warn, filter_out_warnings,
-                       ETSContext, wrapped_stdout, _get_call_line,
+                       wrapped_stdout, _get_call_line, _record_warnings,
                        ClosingStringIO)
 from .misc import (run_subprocess, _pl, _clean_names, pformat, _file_like,
                    _explain_exception, _get_argvalues, sizeof_fmt,
                    running_subprocess, _DefaultEventParser,
-                   _assert_no_instances)
+                   _assert_no_instances, _resource_path)
 from .progressbar import ProgressBar
-from ._testing import (run_tests_if_main, run_command_if_main,
-                       requires_sklearn,
-                       requires_version, requires_nibabel, requires_mayavi,
-                       requires_good_network, requires_mne, requires_pandas,
-                       requires_h5py, traits_test, requires_pysurfer,
+from ._testing import (run_command_if_main, requires_sklearn,
+                       requires_version, requires_nibabel, requires_mne,
+                       requires_good_network, requires_pandas, requires_h5py,
                        ArgvSetter, SilenceStdout, has_freesurfer, has_mne_c,
-                       _TempDir, has_nibabel, _import_mlab, buggy_mkl_svd,
-                       requires_numpydoc, requires_vtk, requires_freesurfer,
+                       _TempDir, has_nibabel, buggy_mkl_svd,
+                       requires_numpydoc, requires_freesurfer,
                        requires_nitime, requires_dipy,
-                       requires_neuromag2ft, requires_pylsl, assert_object_equal,
-                       assert_and_remove_boundary_annot, _raw_annot,
-                       assert_dig_allclose, assert_meg_snr, assert_snr,
-                       assert_stcs_equal, modified_env, _click_ch_name,
-                       _close_event)
+                       requires_neuromag2ft, requires_pylsl,
+                       assert_object_equal, assert_and_remove_boundary_annot,
+                       _raw_annot, assert_dig_allclose, assert_meg_snr,
+                       assert_snr, assert_stcs_equal, modified_env,
+                       _click_ch_name)
 from .numerics import (hashfunc, _compute_row_norms,
                        _reg_pinv, random_permutation, _reject_data_segments,
                        compute_corr, _get_inst_data, array_split_idx,
@@ -62,9 +64,11 @@ from .numerics import (hashfunc, _compute_row_norms,
                        _mask_to_onsets_offsets, _array_equal_nan,
                        _julian_to_cal, _cal_to_julian, _dt_to_julian,
                        _julian_to_dt, _dt_to_stamp, _stamp_to_dt,
-                       _check_dt, _ReuseCycle, _arange_div)
-from .mixin import (SizeMixin, GetEpochsMixin, _prepare_read_metadata,
-                    _prepare_write_metadata, _FakeNoPandas, ShiftTimeMixin)
+                       _check_dt, _ReuseCycle, _arange_div, _hashable_ndarray,
+                       _custom_lru_cache)
+from .mixin import (SizeMixin, GetEpochsMixin, TimeMixin,
+                    _prepare_read_metadata, _prepare_write_metadata,
+                    _FakeNoPandas)
 from .linalg import (_svd_lwork, _repeated_svd, _sym_mat_pow, sqrtm_sym, eigh,
                      _get_blas_funcs)
 from .dataframe import (_set_pandas_dtype, _scale_dataframe_data,
