@@ -24,7 +24,8 @@ from ..io.meas_info import create_info, _validate_type
 
 from ..io.pick import (_get_channel_types, _picks_to_idx, _DATA_CH_TYPES_SPLIT,
                        _VALID_CHANNEL_TYPES)
-from .utils import (tight_layout, _setup_vmin_vmax, plt_show, _check_cov,
+from .utils import (tight_layout, _setup_vmin_vmax, plt_show,
+                    _check_cov, _handle_precompute,
                     _compute_scalings, DraggableColorbar, _setup_cmap,
                     _handle_decim, _set_title_multiple_electrodes,
                     _make_combine_callable, _set_window_title,
@@ -277,7 +278,8 @@ def plot_epochs_image(epochs, picks=None, sigma=0., vmin=None,
         this_info = create_info(sfreq=epochs.info['sfreq'],
                                 ch_names=list(these_ch_names),
                                 ch_types=[this_ch_type] * len(these_picks))
-        this_info['chs'] = this_ch_info
+        with this_info._unlock():
+            this_info['chs'] = this_ch_info
         this_epochs = EpochsArray(this_data, this_info, tmin=epochs.times[0])
         # apply scalings (only to image, not epochs object), combine channels
         this_image = combine_func(this_data * scalings[this_ch_type])
@@ -560,8 +562,8 @@ def _plot_epochs_image(image, style_axes=True, epochs=None, picks=None,
     return fig
 
 
-def plot_drop_log(drop_log, threshold=0, n_max_plot=20, subject='Unknown subj',
-                  color=(0.8, 0.8, 0.8), width=0.8, ignore=('IGNORED',),
+def plot_drop_log(drop_log, threshold=0, n_max_plot=20, subject=None,
+                  color='lightgray', width=0.8, ignore=('IGNORED',),
                   show=True):
     """Show the channel stats based on a drop_log from Epochs.
 
@@ -580,6 +582,9 @@ def plot_drop_log(drop_log, threshold=0, n_max_plot=20, subject='Unknown subj',
 
         .. versionchanged:: 0.23
            Added support for ``None``.
+
+        .. versionchanged:: 1.0
+           Defaults to ``None``.
     color : tuple | str
         Color to use for the bars.
     width : float
@@ -601,12 +606,18 @@ def plot_drop_log(drop_log, threshold=0, n_max_plot=20, subject='Unknown subj',
         logger.info('Percent dropped epochs < supplied threshold; not '
                     'plotting drop log.')
         return
+    absolute = len([x for x in drop_log if len(x)
+                    if not any(y in ignore for y in x)])
+    n_epochs_before_drop = len([x for x in drop_log
+                                if not any(y in ignore for y in x)])
+
     scores = Counter([ch for d in drop_log for ch in d if ch not in ignore])
     ch_names = np.array(list(scores.keys()))
     counts = np.array(list(scores.values()))
     # init figure, handle easy case (no drops)
     fig, ax = plt.subplots()
-    title = f'{percent:.1f}% of all epochs rejected'
+    title = (f'{absolute} of {n_epochs_before_drop} epochs removed '
+             f'({percent:.1f}%)')
     if subject is not None:
         title = f'{subject}: {title}'
     ax.set_title(title)
@@ -625,7 +636,7 @@ def plot_drop_log(drop_log, threshold=0, n_max_plot=20, subject='Unknown subj',
     ax.set_xticks(x)
     ax.set_xticklabels(ch_names[order[:n_bars]], rotation=45, size=10,
                        horizontalalignment='right')
-    ax.set_ylabel('% of epochs rejected')
+    ax.set_ylabel('% of epochs removed')
     ax.grid(axis='y')
     tight_layout(pad=1, fig=fig)
     plt_show(show)
@@ -637,7 +648,9 @@ def plot_epochs(epochs, picks=None, scalings=None, n_epochs=20, n_channels=20,
                 title=None, events=None, event_color=None,
                 order=None, show=True, block=False, decim='auto',
                 noise_cov=None, butterfly=False, show_scrollbars=True,
-                epoch_colors=None, event_id=None, group_by='type'):
+                show_scalebars=True, epoch_colors=None, event_id=None,
+                group_by='type', precompute=None, use_opengl=None, *,
+                theme=None, overview_mode=None):
     """Visualize epochs.
 
     Bad epochs can be marked with a left click on top of the epoch. Bad
@@ -650,17 +663,7 @@ def plot_epochs(epochs, picks=None, scalings=None, n_epochs=20, n_channels=20,
     epochs : instance of Epochs
         The epochs object.
     %(picks_good_data)s
-    scalings : dict | 'auto' | None
-        Scaling factors for the traces. If any fields in scalings are 'auto',
-        the scaling factor is set to match the 99.5th percentile of a subset of
-        the corresponding data. If scalings == 'auto', all scalings fields are
-        set to 'auto'. If any fields are 'auto' and data is not preloaded,
-        a subset of epochs up to 100 Mb will be loaded. If None, defaults to::
-
-            dict(mag=1e-12, grad=4e-11, eeg=20e-6, eog=150e-6, ecg=5e-4,
-                 emg=1e-3, ref_meg=1e-12, misc=1e-3, stim=1, resp=1, chpi=1e-4,
-                 whitened=10.)
-
+    %(scalings)s
     n_epochs : int
         The number of epochs per view. Defaults to 20.
     n_channels : int
@@ -715,6 +718,9 @@ def plot_epochs(epochs, picks=None, scalings=None, n_epochs=20, n_channels=20,
 
         .. versionadded:: 0.18.0
     %(show_scrollbars)s
+    %(show_scalebars)s
+
+        .. versionadded:: 0.24.0
     epoch_colors : list of (n_epochs) list (of n_channels) | None
         Colors to use for individual epochs. If None, use default colors.
     event_id : dict | None
@@ -725,12 +731,19 @@ def plot_epochs(epochs, picks=None, scalings=None, n_epochs=20, n_channels=20,
         ``epochs.event_id`` when there are overlapping keys.
 
         .. versionadded:: 0.20
-    %(browse_group_by)s
+    %(group_by_browse)s
+    %(precompute)s
+    %(use_opengl)s
+    %(theme_pg)s
+
+        .. versionadded:: 1.0
+    %(overview_mode)s
+
+        .. versionadded:: 1.1
 
     Returns
     -------
-    fig : instance of matplotlib.figure.Figure
-        The figure.
+    %(browser)s
 
     Notes
     -----
@@ -739,16 +752,15 @@ def plot_epochs(epochs, picks=None, scalings=None, n_epochs=20, n_channels=20,
     keys, but this depends on the backend matplotlib is configured to use
     (e.g., mpl.use(``TkAgg``) should work). Full screen mode can be toggled
     with f11 key. The amount of epochs and channels per view can be adjusted
-    with home/end and page down/page up keys. These can also be set through
-    options dialog by pressing ``o`` key. ``h`` key plots a histogram of
+    with home/end and page down/page up keys. ``h`` key plots a histogram of
     peak-to-peak values along with the used rejection thresholds. Butterfly
-    plot can be toggled with ``b`` key. Right mouse click adds a vertical line
+    plot can be toggled with ``b`` key. Left mouse click adds a vertical line
     to the plot. Click 'help' button at bottom left corner of the plotter to
     view all the options.
 
     .. versionadded:: 0.10.0
     """
-    from ._figure import _browse_figure
+    from ._figure import _get_browser
 
     epochs.drop_bad()
     info = epochs.info.copy()
@@ -756,7 +768,8 @@ def plot_epochs(epochs, picks=None, scalings=None, n_epochs=20, n_channels=20,
     projs = info['projs']
     projs_on = np.full_like(projs, epochs.proj, dtype=bool)
     if not epochs.proj:
-        info['projs'] = list()
+        with info._unlock():
+            info['projs'] = list()
 
     # handle defaults / check arg validity
     color = _handle_default('color', None)
@@ -795,16 +808,29 @@ def plot_epochs(epochs, picks=None, scalings=None, n_epochs=20, n_channels=20,
     # events
     if events is not None:
         event_nums = events[:, 2]
-        boundary_samps = epochs.events[1:, 0] - epochs.time_as_index(0)
-        event_ixs = np.searchsorted(boundary_samps, events[:, 0], side='right')
-        event_samp_offsets = (events[:, 0] - epochs.events[:, 0][event_ixs]
-                              + epochs.time_as_index(0))
-        event_times = ((np.arange(len(epochs)) * len(epochs.times))[event_ixs]
-                       + event_samp_offsets) / sfreq
-        # don't show events that are beyond the range of their epoch
-        mask = event_samp_offsets <= len(epochs.times)
-        event_times = event_times[mask]
-        event_nums = event_nums[mask]
+        event_samps = events[:, 0]
+        epoch_n_samps = len(epochs.times)
+        # handle overlapping epochs (each event may show up in multiple places)
+        boundaries = (epochs.events[:, [0]] + np.array([-1, 1])
+                      * epochs.time_as_index(0))
+        in_bounds = np.logical_and(boundaries[:, [0]] <= event_samps,
+                                   event_samps < boundaries[:, [1]])
+        event_ixs = [np.nonzero(a)[0] for a in in_bounds.T]
+        warned = False
+        event_times = list()
+        event_numbers = list()
+        for samp, num, _ixs in zip(event_samps, event_nums, event_ixs):
+            relevant_epoch_events = epochs.events[:, 0][_ixs]
+            if len(relevant_epoch_events) > 1 and not warned:
+                logger.info('You seem to have overlapping epochs. Some event '
+                            'lines may be duplicated in the plot.')
+                warned = True
+            offsets = samp - relevant_epoch_events + epochs.time_as_index(0)
+            this_event_times = (_ixs * epoch_n_samps + offsets) / sfreq
+            event_times.extend(this_event_times)
+            event_numbers.extend([num] * len(_ixs))
+        event_nums = np.array(event_numbers)
+        event_times = np.array(event_times)
     else:
         event_nums = None
         event_times = None
@@ -831,6 +857,7 @@ def plot_epochs(epochs, picks=None, scalings=None, n_epochs=20, n_channels=20,
     elif not isinstance(title, str):
         raise TypeError(f'title must be None or a string, got a {type(title)}')
 
+    precompute = _handle_precompute(precompute)
     params = dict(inst=epochs,
                   info=info,
                   n_epochs=n_epochs,
@@ -848,6 +875,7 @@ def plot_epochs(epochs, picks=None, scalings=None, n_epochs=20, n_channels=20,
                   duration=duration,
                   n_times=n_times,
                   first_time=0,
+                  time_format='float',
                   decim=decim,
                   boundary_times=boundary_times,
                   # events
@@ -869,7 +897,7 @@ def plot_epochs(epochs, picks=None, scalings=None, n_epochs=20, n_channels=20,
                   units=units,
                   unit_scalings=unit_scalings,
                   # colors
-                  ch_color_bad=(0.8, 0.8, 0.8),
+                  ch_color_bad='lightgray',
                   ch_color_dict=color,
                   epoch_color_bad=(1, 0, 0),
                   epoch_colors=epoch_colors,
@@ -877,22 +905,17 @@ def plot_epochs(epochs, picks=None, scalings=None, n_epochs=20, n_channels=20,
                   butterfly=butterfly,
                   clipping=None,
                   scrollbars_visible=show_scrollbars,
-                  scalebars_visible=False,
+                  scalebars_visible=show_scalebars,
                   window_title=title,
-                  xlabel='Epoch number')
-    fig = _browse_figure(**params)
-    fig._update_picks()
+                  xlabel='Epoch number',
+                  # pyqtgraph-specific
+                  precompute=precompute,
+                  use_opengl=use_opengl,
+                  theme=theme,
+                  overview_mode=overview_mode)
 
-    # make channel selection dialog, if requested (doesn't work well in init)
-    if group_by in ('selection', 'position'):
-        fig._create_selection_fig()
+    fig = _get_browser(show=show, block=block, **params)
 
-    fig._update_projector()
-    fig._update_trace_offsets()
-    fig._update_data()
-    fig._draw_traces()
-
-    plt_show(show, block=block)
     return fig
 
 
@@ -901,9 +924,9 @@ def plot_epochs_psd(epochs, fmin=0, fmax=np.inf, tmin=None, tmax=None,
                     proj=False, bandwidth=None, adaptive=False, low_bias=True,
                     normalization='length', picks=None, ax=None, color='black',
                     xscale='linear', area_mode='std', area_alpha=0.33,
-                    dB=True, estimate='auto', show=True, n_jobs=1,
+                    dB=True, estimate='auto', show=True, n_jobs=None,
                     average=False, line_alpha=None, spatial_colors=True,
-                    sphere=None, verbose=None):
+                    sphere=None, exclude='bads', verbose=None):
     """%(plot_psd_doc)s.
 
     Parameters
@@ -929,25 +952,28 @@ def plot_epochs_psd(epochs, fmin=0, fmax=np.inf, tmin=None, tmax=None,
     low_bias : bool
         Only use tapers with more than 90%% spectral concentration within
         bandwidth.
-    normalization : str
-        Either "full" or "length" (default). If "full", the PSD will
-        be normalized by the sampling rate as well as the length of
-        the signal (as in nitime).
-    %(plot_psd_picks_good_data)s
+    %(normalization)s
+    %(picks_plot_psd_good_data)s
     ax : instance of Axes | None
         Axes to plot into. If None, axes will be created.
-    %(plot_psd_color)s
-    %(plot_psd_xscale)s
-    %(plot_psd_area_mode)s
-    %(plot_psd_area_alpha)s
-    %(plot_psd_dB)s
-    %(plot_psd_estimate)s
+    %(color_plot_psd)s
+    %(xscale_plot_psd)s
+    %(area_mode_plot_psd)s
+    %(area_alpha_plot_psd)s
+    %(dB_plot_psd)s
+    %(estimate_plot_psd)s
     %(show)s
     %(n_jobs)s
-    %(plot_psd_average)s
-    %(plot_psd_line_alpha)s
-    %(plot_psd_spatial_colors)s
-    %(topomap_sphere_auto)s
+    %(average_plot_psd)s
+    %(line_alpha_plot_psd)s
+    %(spatial_colors_plot_psd)s
+    %(sphere_topomap_auto)s
+    exclude : list of str | 'bads'
+        Channels names to exclude from being shown. If 'bads', the bad channels
+        are excluded. Pass an empty list to plot all channels (including
+        channels marked "bad", if any).
+
+        .. versionadded:: 0.24.0
     %(verbose)s
 
     Returns
@@ -955,7 +981,7 @@ def plot_epochs_psd(epochs, fmin=0, fmax=np.inf, tmin=None, tmax=None,
     fig : instance of Figure
         Figure with frequency spectra of the data channels.
     """
-    from ._figure import _psd_figure
+    from ._mpl_figure import _psd_figure
 
     # generate figure
     # epochs always use multitaper, not Welch, so no need to allow "window"
@@ -967,6 +993,6 @@ def plot_epochs_psd(epochs, fmin=0, fmax=np.inf, tmin=None, tmax=None,
         line_alpha=line_alpha, area_alpha=area_alpha, color=color,
         spatial_colors=spatial_colors, n_jobs=n_jobs, bandwidth=bandwidth,
         adaptive=adaptive, low_bias=low_bias, normalization=normalization,
-        window='hamming')
+        window='hamming', exclude=exclude)
     plt_show(show)
     return fig
