@@ -2,6 +2,7 @@
 """
 .. _tut-artifact-ssp:
 
+============================
 Repairing artifacts with SSP
 ============================
 
@@ -15,6 +16,8 @@ from repeatedly typing ``mne.preprocessing`` we'll directly import a handful of
 functions from that submodule:
 """
 
+# %%
+
 import os
 import numpy as np
 import matplotlib.pyplot as plt
@@ -22,7 +25,7 @@ import mne
 from mne.preprocessing import (create_eog_epochs, create_ecg_epochs,
                                compute_proj_ecg, compute_proj_eog)
 
-###############################################################################
+# %%
 # .. note::
 #     Before applying SSP (or any artifact repair strategy), be sure to observe
 #     the artifacts in your data to make sure you choose the right repair tool.
@@ -66,9 +69,11 @@ from mne.preprocessing import (create_eog_epochs, create_ecg_epochs,
 sample_data_folder = mne.datasets.sample.data_path()
 sample_data_raw_file = os.path.join(sample_data_folder, 'MEG', 'sample',
                                     'sample_audvis_raw.fif')
-raw = mne.io.read_raw_fif(sample_data_raw_file)
+# here we crop and resample just for speed
+raw = mne.io.read_raw_fif(sample_data_raw_file).crop(0, 60)
+raw.load_data().resample(100)
 
-###############################################################################
+# %%
 # The :ref:`example data <sample-dataset>` also includes an "empty room"
 # recording taken the same day as the recording of the subject. This will
 # provide a more accurate estimate of environmental noise than the projectors
@@ -82,15 +87,16 @@ system_projs = raw.info['projs']
 raw.del_proj()
 empty_room_file = os.path.join(sample_data_folder, 'MEG', 'sample',
                                'ernoise_raw.fif')
-empty_room_raw = mne.io.read_raw_fif(empty_room_file)
+# cropped to 60 sec just for speed
+empty_room_raw = mne.io.read_raw_fif(empty_room_file).crop(0, 30)
 
-###############################################################################
+# %%
 # Notice that the empty room recording itself has the system-provided SSP
 # projectors in it — we'll remove those from the empty room file too.
 
 empty_room_raw.del_proj()
 
-###############################################################################
+# %%
 # Visualizing the empty-room noise
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
@@ -101,7 +107,7 @@ empty_room_raw.del_proj()
 for average in (False, True):
     empty_room_raw.plot_psd(average=average, dB=False, xscale='log')
 
-###############################################################################
+# %%
 # Creating the empty-room projectors
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
@@ -114,12 +120,11 @@ for average in (False, True):
 # smoothness. Note that for the function to know the types of channels in a
 # projector, you must also provide the corresponding `~mne.Info` object:
 
-# sphinx_gallery_thumbnail_number = 3
 empty_room_projs = mne.compute_proj_raw(empty_room_raw, n_grad=3, n_mag=3)
 mne.viz.plot_projs_topomap(empty_room_projs, colorbar=True, vlim='joint',
                            info=empty_room_raw.info)
 
-###############################################################################
+# %%
 # Notice that the gradiometer-based projectors seem to reflect problems with
 # individual sensor units rather than a global noise source (indeed, planar
 # gradiometers are much less sensitive to distant sources). This is the reason
@@ -134,7 +139,7 @@ for idx, _projs in enumerate([system_projs, empty_room_projs[3:]]):
     mne.viz.plot_projs_topomap(_projs, axes=axs[idx], colorbar=True,
                                vlim='joint', info=empty_room_raw.info)
 
-###############################################################################
+# %%
 # Visualizing how projectors affect the signal
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
@@ -150,11 +155,12 @@ mags = mne.pick_types(raw.info, meg='mag')
 for title, projs in [('system', system_projs),
                      ('subject-specific', empty_room_projs[3:])]:
     raw.add_proj(projs, remove_existing=True)
-    fig = raw.plot(proj=True, order=mags, duration=1, n_channels=2)
+    with mne.viz.use_browser_backend('matplotlib'):
+        fig = raw.plot(proj=True, order=mags, duration=1, n_channels=2)
     fig.subplots_adjust(top=0.9)  # make room for title
     fig.suptitle('{} projectors'.format(title), size='xx-large', weight='bold')
 
-###############################################################################
+# %%
 # The effect is sometimes easier to see on averaged data. Here we use an
 # interactive feature of `mne.Evoked.plot_topomap` to turn projectors on
 # and off to see the effect on the data. Of course, the interactivity won't
@@ -176,7 +182,7 @@ times = np.linspace(0.05, 0.15, 5)
 epochs = mne.Epochs(raw, events, event_id, proj='delayed', reject=reject)
 fig = epochs.average().plot_topomap(times, proj='interactive')
 
-###############################################################################
+# %%
 # Plotting the ERP/F using ``evoked.plot()`` or ``evoked.plot_joint()`` with
 # and without projectors applied can also be informative, as can plotting with
 # ``proj='reconstruct'``, which can reduce the signal bias introduced by
@@ -197,7 +203,7 @@ regexp = r'(MEG [12][45][123]1|EEG 00.)'
 artifact_picks = mne.pick_channels_regexp(raw.ch_names, regexp=regexp)
 raw.plot(order=artifact_picks, n_channels=len(artifact_picks))
 
-###############################################################################
+# %%
 # Repairing ECG artifacts with SSP
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
@@ -210,14 +216,14 @@ raw.plot(order=artifact_picks, n_channels=len(artifact_picks))
 ecg_evoked = create_ecg_epochs(raw).average()
 ecg_evoked.plot_joint()
 
-###############################################################################
+# %%
 # Looks like the EEG channels are pretty spread out; let's baseline-correct and
 # plot again:
 
 ecg_evoked.apply_baseline((None, None))
 ecg_evoked.plot_joint()
 
-###############################################################################
+# %%
 # To compute SSP projectors for the heartbeat artifact, you can use
 # `~mne.preprocessing.compute_proj_ecg`, which takes a
 # `~mne.io.Raw` object as input and returns the requested number of
@@ -230,7 +236,7 @@ ecg_evoked.plot_joint()
 
 projs, events = compute_proj_ecg(raw, n_grad=1, n_mag=1, n_eeg=1, reject=None)
 
-###############################################################################
+# %%
 # The first line of output tells us that
 # `~mne.preprocessing.compute_proj_ecg` found three existing projectors
 # already in the `~mne.io.Raw` object, and will include those in the
@@ -243,13 +249,36 @@ projs, events = compute_proj_ecg(raw, n_grad=1, n_mag=1, n_eeg=1, reject=None)
 ecg_projs = projs[3:]
 print(ecg_projs)
 
-###############################################################################
+# %%
 # Just like with the empty-room projectors, we can visualize the scalp
 # distribution:
 
 mne.viz.plot_projs_topomap(ecg_projs, info=raw.info)
 
-###############################################################################
+# %%
+# Moreover, because these projectors were created using epochs chosen
+# specifically because they contain time-locked artifacts, we can do a
+# joint plot of the projectors and their effect on the time-averaged epochs.
+# This figure has three columns:
+#
+# 1. The left shows the data traces before (black) and after (green)
+#    projection. We can see that the ECG artifact is well suppressed by one
+#    projector per channel type.
+# 2. The center shows the topomaps associated with the projectors, in this case
+#    just a single topography for our one projector per channel type.
+# 3. The right again shows the data traces (black), but this time with those
+#    traces also projected onto the first projector for each channel type (red)
+#    plus one surrogate ground truth for an ECG channel (MEG 0111).
+
+# sphinx_gallery_thumbnail_number = 17
+
+# ideally here we would just do `picks_trace='ecg'`, but this dataset did not
+# have a dedicated ECG channel recorded, so we just pick a channel that was
+# very sensitive to the artifact
+fig = mne.viz.plot_projs_joint(ecg_projs, ecg_evoked, picks_trace='MEG 0111')
+fig.suptitle('ECG projectors')
+
+# %%
 # Since no dedicated ECG sensor channel was detected in the
 # `~mne.io.Raw` object, by default
 # `~mne.preprocessing.compute_proj_ecg` used the magnetometers to
@@ -291,12 +320,13 @@ mne.viz.plot_projs_topomap(ecg_projs, info=raw.info)
 raw.del_proj()
 for title, proj in [('Without', empty_room_projs), ('With', ecg_projs)]:
     raw.add_proj(proj, remove_existing=False)
-    fig = raw.plot(order=artifact_picks, n_channels=len(artifact_picks))
+    with mne.viz.use_browser_backend('matplotlib'):
+        fig = raw.plot(order=artifact_picks, n_channels=len(artifact_picks))
     fig.subplots_adjust(top=0.9)  # make room for title
     fig.suptitle('{} ECG projectors'.format(title), size='xx-large',
                  weight='bold')
 
-###############################################################################
+# %%
 # Finally, note that above we passed ``reject=None`` to the
 # `~mne.preprocessing.compute_proj_ecg` function, meaning that all
 # detected ECG epochs would be used when computing the projectors (regardless
@@ -343,11 +373,11 @@ for title, proj in [('Without', empty_room_projs), ('With', ecg_projs)]:
 # seen above the large deflections in frontal EEG channels in the raw data;
 # here is how the ocular artifacts manifests across all the sensors:
 
-eog_evoked = create_eog_epochs(raw).average()
+eog_evoked = create_eog_epochs(raw).average(picks='all')
 eog_evoked.apply_baseline((None, None))
 eog_evoked.plot_joint()
 
-###############################################################################
+# %%
 # Just like we did with the heartbeat artifact, we can compute SSP projectors
 # for the ocular artifact using `~mne.preprocessing.compute_proj_eog`,
 # which again takes a `~mne.io.Raw` object as input and returns the
@@ -361,13 +391,40 @@ eog_evoked.plot_joint()
 eog_projs, _ = compute_proj_eog(raw, n_grad=1, n_mag=1, n_eeg=1, reject=None,
                                 no_proj=True)
 
-###############################################################################
+# %%
 # Just like with the empty-room and ECG projectors, we can visualize the scalp
 # distribution:
 
 mne.viz.plot_projs_topomap(eog_projs, info=raw.info)
 
-###############################################################################
+# %%
+# And we can do a joint image:
+
+fig = mne.viz.plot_projs_joint(eog_projs, eog_evoked, 'eog')
+fig.suptitle('EOG projectors')
+
+# %%
+# And finally, we can make a joint visualization with our EOG evoked. We will
+# also make a bad choice here and select *two* EOG projectors for EEG and
+# magnetometers, and we will see them show up as noise in the plot. Even though
+# the projected time course (left column) looks perhaps okay, problems show
+# up in the center (topomaps) and right plots (projection of channel data
+# onto the projection vector):
+#
+# 1. The second magnetometer topomap has a bilateral auditory field pattern.
+# 2. The uniformly-scaled projected temporal time course (solid lines) show
+#    that, while the first projector trace (red) has a large EOG-like
+#    amplitude, the second projector trace (blue-green) is much smaller.
+# 3. The re-normalized projected temporal time courses show that the
+#    second PCA trace is very noisy relative to the EOG channel data (yellow).
+
+eog_projs_bad, _ = compute_proj_eog(
+    raw, n_grad=1, n_mag=2, n_eeg=2, reject=None,
+    no_proj=True)
+fig = mne.viz.plot_projs_joint(eog_projs_bad, eog_evoked, picks_trace='eog')
+fig.suptitle('Too many EOG projectors')
+
+# %%
 # Now we repeat the plot from above (with empty room and ECG projectors) and
 # compare it to a plot with empty room, ECG, and EOG projectors, to see how
 # well the ocular artifacts have been repaired:
@@ -375,12 +432,13 @@ mne.viz.plot_projs_topomap(eog_projs, info=raw.info)
 for title in ('Without', 'With'):
     if title == 'With':
         raw.add_proj(eog_projs)
-    fig = raw.plot(order=artifact_picks, n_channels=len(artifact_picks))
+    with mne.viz.use_browser_backend('matplotlib'):
+        fig = raw.plot(order=artifact_picks, n_channels=len(artifact_picks))
     fig.subplots_adjust(top=0.9)  # make room for title
     fig.suptitle('{} EOG projectors'.format(title), size='xx-large',
                  weight='bold')
 
-###############################################################################
+# %%
 # Notice that the small peaks in the first to magnetometer channels (``MEG
 # 1411`` and ``MEG 1421``) that occur at the same time as the large EEG
 # deflections have also been removed.
@@ -404,7 +462,8 @@ for title in ('Without', 'With'):
 #
 # Visualizing SSP sensor-space bias via signal reconstruction
 # ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-# .. sidebar:: SSP reconstruction
+# .. admonition:: SSP reconstruction
+#     :class: sidebar note
 #
 #     Internally, the reconstruction is performed by effectively using a
 #     minimum-norm source localization to a spherical source space with the
@@ -423,32 +482,30 @@ for title in ('Without', 'With'):
 # source amplitudes. However, for sensor space analyses, it can be useful to
 # visualize the extent to which SSP projection has biased the data. This can be
 # explored by using ``proj='reconstruct'`` in evoked plotting functions, for
-# example via `evoked.plot() <mne.Evoked.plot>`:
+# example via `evoked.plot() <mne.Evoked.plot>`, here restricted to just
+# EEG channels for speed:
 
-evoked = epochs.average()
-# Apply the average ref first:
-# It's how we typically view EEG data, and here we're really just interested
-# in the effect of the EOG+ECG SSPs
-evoked.del_proj().set_eeg_reference(projection=True).apply_proj()
-evoked.add_proj(ecg_projs).add_proj(eog_projs)
-fig, axes = plt.subplots(3, 3, figsize=(8, 6))
-for ii in range(3):
+evoked_eeg = epochs.average().pick('eeg')
+evoked_eeg.del_proj().add_proj(ecg_projs).add_proj(eog_projs)
+fig, axes = plt.subplots(1, 3, figsize=(8, 3), squeeze=False)
+for ii in range(axes.shape[0]):
     axes[ii, 0].get_shared_y_axes().join(*axes[ii])
 for pi, proj in enumerate((False, True, 'reconstruct')):
-    evoked.plot(proj=proj, axes=axes[:, pi], spatial_colors=True)
+    evoked_eeg.plot(proj=proj, axes=axes[:, pi], spatial_colors=True)
     if pi == 0:
         for ax in axes[:, pi]:
             parts = ax.get_title().split('(')
             ax.set(ylabel=f'{parts[0]} ({ax.get_ylabel()})\n'
                           f'{parts[1].replace(")", "")}')
     axes[0, pi].set(title=f'proj={proj}')
-    axes[0, pi].texts.clear()
+    for text in list(axes[0, pi].texts):
+        text.remove()
 plt.setp(axes[1:, :].ravel(), title='')
 plt.setp(axes[:, 1:].ravel(), ylabel='')
 plt.setp(axes[:-1, :].ravel(), xlabel='')
 mne.viz.tight_layout()
 
-###############################################################################
+# %%
 # Note that here the bias in the EEG and magnetometer channels is reduced by
 # the reconstruction. This suggests that the application of SSP has slightly
 # reduced the amplitude of our signals in sensor space, but that it should not
